#!/usr/bin/env python3
#
# Copyright (C) 2023
# ASTRON (Netherlands Institute for Radio Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This file is part of the LOFAR software suite.
# The LOFAR software suite is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The LOFAR software suite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the LOFAR software suite. If not, see <http://www.gnu.org/licenses/>.
#
# $Id$
import datetime

from lofar.messaging import DEFAULT_BUSNAME, DEFAULT_BROKER, RPCClient, RPCClientContextManagerMixin

import logging
logger = logging.getLogger(__name__)


class TMSSCalibratorSelectorRPCClient(RPCClientContextManagerMixin):
    def __init__(self, rpc_client : RPCClient = RPCClient(service_name="TMSSCalibratorSelector.Service")):
        super().__init__()
        self._rpc_client = rpc_client

    @staticmethod
    def create(exchange=DEFAULT_BUSNAME, broker=DEFAULT_BROKER) -> 'TMSSCalibratorSelectorRPCClient':
        """Create a TMSSCalibratorSelectorRPCClient connecting to the given exchange/broker to send RPC calls to the TMSSCalibratorSelector.Service"""
        return TMSSCalibratorSelectorRPCClient(RPCClient(service_name="TMSSCalibratorSelector.Service",
                                                         exchange=exchange, broker=broker))

    def get_calibrator_for_target_coordinates(self, calibrator_type: str, timestamp: datetime.datetime, angle1: float, angle2: float, direction_type='J2000', min_elevation=.5):
        """
        :param calibrator_type: the type of calibrator (used to select the list of possible calibrators)
        :param timestamp: timestamp at which the calibrator should be close to given coordinates
        :param angle1: angle1 of target pointing
        :param angle2: angle2 of target pointing
        :param direction_type: direction type of target pointing
        :param min_elevation: minimum elevation that the calibrator has to have
        :return: 3C name of the calibrator
        """
        return self._rpc_client.execute('GetCalibratorForTargetCoordinates', calibrator_type=calibrator_type, timestamp=timestamp,
                                        angle1=angle1, angle2=angle2, direction_type=direction_type,
                                        min_elevation=min_elevation)
