#!/usr/bin/env python3
#
# Copyright (C) 2023
# ASTRON (Netherlands Institute for Radio Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This file is part of the LOFAR software suite.
# The LOFAR software suite is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The LOFAR software suite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the LOFAR software suite. If not, see <http://www.gnu.org/licenses/>.
#
# $Id$

from lofar.messaging import DEFAULT_BUSNAME, DEFAULT_BROKER

from lofar.sas.tmss.client.tmssbuslistener import TMSSBusListener

from lofar.sas.tmss.services.lobster.message_handler import L2TMSSObservationControlMessageHandler
from lofar.sas.tmss.services.lobster.observation_pool import ObservationPool


class L2TMSSObservationControlBusListener(TMSSBusListener):
    """This buslistener subscribes via a designated queue to the lofar exchange,
    and handles the subtask status events in the L2TMSSObservationControlMessageHandler to start the observations"""

    def __init__(self, exchange: str = DEFAULT_BUSNAME, broker: str = DEFAULT_BROKER,
                 observation_pool: ObservationPool = ObservationPool(), tmss_client_credentials_id: str = None):
        super().__init__(
            handler_type=L2TMSSObservationControlMessageHandler,
            handler_kwargs={
                'observation_pool': observation_pool,
                'tmss_client_credentials_id': tmss_client_credentials_id
            },
            exchange=exchange,
            broker=broker,
            num_threads=1
        )
