#!/usr/bin/env python3
#
# Copyright (C) 2023
# ASTRON (Netherlands Institute for Radio Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This file is part of the LOFAR software suite.
# The LOFAR software suite is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The LOFAR software suite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the LOFAR software suite. If not, see <http://www.gnu.org/licenses/>.
#
# $Id$

from lofar.messaging import DEFAULT_BUSNAME, DEFAULT_BROKER, RPCClient, RPCClientContextManagerMixin

import logging
logger = logging.getLogger(__name__)


class L2TMSSObservationControlRPCClient(RPCClientContextManagerMixin):
    # TODO: move rpc client into its own module. maybe even to a client repo.
    def __init__(self, rpc_client : RPCClient = RPCClient(service_name="L2TMSSObservationControl.Service")):
        super().__init__()
        self._rpc_client = rpc_client

    @staticmethod
    def create(exchange=DEFAULT_BUSNAME, broker=DEFAULT_BROKER) -> 'L2TMSSObservationControlRPCClient':
        """Create a L2TMSSObservationControlRPCClient connecting to the given exchange/broker to send RPC calls to the L2TMSSObservationControl.Service"""
        return L2TMSSObservationControlRPCClient(RPCClient(service_name="L2TMSSObservationControl.Service",
                                                           exchange=exchange, broker=broker))

    def abort_observation(self, observation_subtask_id):
        return self._rpc_client.execute('AbortObservation', observation_subtask_id=observation_subtask_id)
