#!/usr/bin/env python3
#
# Copyright (C) 2023
# ASTRON (Netherlands Institute for Radio Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This file is part of the LOFAR software suite.
# The LOFAR software suite is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The LOFAR software suite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the LOFAR software suite. If not, see <http://www.gnu.org/licenses/>.
#
# $Id$

from typing import List
import logging

from .config import station_to_host

logger = logging.getLogger(__name__)


def extract_stations(l2specification: dict) -> List[str]:
    """Combine l2specficiation from TMSS into MultiStationObservation format

    :param l2specification: Specification as retrieved from
                            :method:`TMSSsession.get_subtask_l2stationspecs`
    :return: List of all identified stations in full TangoDB host:port format
    """

    stations = []
    for station in l2specification['stations']:
        stations.append(station_to_host(station["station"]))

    return stations
