
#
# auto-generated by remakemigrations.py
#
# ! Please make sure to apply any changes to the template in that script !
#
from django.db import migrations

from lofar.sas.tmss.tmss.tmssapp.populate import *

class Migration(migrations.Migration):

    dependencies = [
        ('tmssapp', '0001_initial'),
    ]

    operations = [ migrations.RunSQL('ALTER SEQUENCE tmssapp_SubTask_id_seq RESTART WITH 2000000;'), # Start SubTask id with 2 000 000 to avoid overlap with 'old' (test/production) OTDB
                   # add an SQL trigger in the database enforcing correct state transitions.
                   # it is crucial that illegal subtask state transitions are block at the "lowest level" (i.e.: in the database) so we can guarantee that the subtask state machine never breaks.
                   # see: https://support.astron.nl/confluence/display/TMSS/Subtask+State+Machine
                   # Explanation of SQl below: A trigger function is called upon each create/update of the subtask.
                   # If the state changes, then it is checked if the state transition from old to new is present in the SubtaskAllowedStateTransitions table.
                   # If not an Exception is raised, thus enforcing a rollback, thus enforcing the state machine to follow the design.
                   # It is thereby enforced upon the user/caller to handle these blocked illegal state transitions, and act more wisely.
                   migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_check_subtask_state_transition()
                                     RETURNS trigger AS
                                     $BODY$
                                     BEGIN
                                       IF TG_OP = 'INSERT' THEN
                                         IF NOT (SELECT EXISTS(SELECT id FROM tmssapp_subtaskallowedstatetransitions WHERE old_state_id IS NULL AND new_state_id=NEW.state_id)) THEN
                                            RAISE EXCEPTION 'ILLEGAL SUBTASK STATE TRANSITION FROM % TO %', NULL, NEW.state_id;
                                         END IF;
                                       END IF;
                                       IF TG_OP = 'UPDATE' THEN
                                         IF OLD.state_id <> NEW.state_id AND NOT (SELECT EXISTS(SELECT id FROM tmssapp_subtaskallowedstatetransitions WHERE old_state_id=OLD.state_id AND new_state_id=NEW.state_id)) THEN
                                           RAISE EXCEPTION 'ILLEGAL SUBTASK STATE TRANSITION FROM "%" TO "%"', OLD.state_id, NEW.state_id;
                                         END IF;
                                       END IF;
                                     RETURN NEW;
                                     END;
                                     $BODY$
                                     LANGUAGE plpgsql VOLATILE;
                                     DROP TRIGGER IF EXISTS tmssapp_trigger_on_check_subtask_state_transition ON tmssapp_SubTask ;
                                     CREATE TRIGGER tmssapp_trigger_on_check_subtask_state_transition
                                     BEFORE INSERT OR UPDATE ON tmssapp_SubTask
                                     FOR EACH ROW EXECUTE PROCEDURE tmssapp_check_subtask_state_transition();'''),

                   # use database triggers to block updates on blueprint tables for immutable fields
                   migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_block_scheduling_unit_blueprint_immutable_fields_update()
                                     RETURNS trigger AS
                                     $BODY$
                                     BEGIN
                                       IF OLD.specifications_template_id <> NEW.specifications_template_id OR
                                          OLD.name <> NEW.name OR
                                          OLD.description <> NEW.description THEN
                                         RAISE EXCEPTION 'ILLEGAL UPDATE OF IMMUTABLE BLUEPRINT FIELD';
                                       END IF;
                                     RETURN NEW;
                                     END;
                                     $BODY$
                                     LANGUAGE plpgsql VOLATILE;
                                     DROP TRIGGER IF EXISTS tmssapp_trigger_block_scheduling_unit_blueprint_immutable_fields_update ON tmssapp_SchedulingUnitBlueprint ;
                                     CREATE TRIGGER tmssapp_block_scheduling_unit_blueprint_immutable_fields_update
                                     BEFORE UPDATE ON tmssapp_SchedulingUnitBlueprint
                                     FOR EACH ROW EXECUTE PROCEDURE tmssapp_block_scheduling_unit_blueprint_immutable_fields_update();
                                     '''),
                   migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_block_task_blueprint_immutable_fields_update()
                                     RETURNS trigger AS
                                     $BODY$
                                     BEGIN
                                       IF OLD.specifications_doc <> NEW.specifications_doc OR
                                          OLD.name <> NEW.name OR
                                          OLD.description <> NEW.description OR
                                          OLD.specifications_template_id <> NEW.specifications_template_id OR
                                          OLD.scheduling_unit_blueprint_id <> NEW.scheduling_unit_blueprint_id THEN
                                         RAISE EXCEPTION 'ILLEGAL UPDATE OF IMMUTABLE BLUEPRINT FIELD';
                                       END IF;
                                     RETURN NEW;
                                     END;
                                     $BODY$
                                     LANGUAGE plpgsql VOLATILE;
                                     DROP TRIGGER IF EXISTS tmssapp_trigger_block_task_blueprint_immutable_fields_update ON tmssapp_TaskBlueprint ;
                                     CREATE TRIGGER tmssapp_block_task_blueprint_immutable_fields_update
                                     BEFORE UPDATE ON tmssapp_TaskBlueprint
                                     FOR EACH ROW EXECUTE PROCEDURE tmssapp_block_task_blueprint_immutable_fields_update();
                                     '''),
                   migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_block_task_relation_blueprint_immutable_fields_update()
                                     RETURNS trigger AS
                                     $BODY$
                                     BEGIN
                                       IF OLD.selection_doc <> NEW.selection_doc OR
                                          OLD.producer_id <> NEW.producer_id OR
                                          OLD.consumer_id <> NEW.consumer_id OR
                                          OLD.input_role_id <> NEW.input_role_id OR
                                          OLD.output_role_id <> NEW.output_role_id OR
                                          OLD.input_role_id <> NEW.input_role_id OR
                                          OLD.selection_template_id <> NEW.selection_template_id THEN
                                         RAISE EXCEPTION 'ILLEGAL UPDATE OF IMMUTABLE BLUEPRINT FIELD';
                                       END IF;
                                     RETURN NEW;
                                     END;
                                     $BODY$
                                     LANGUAGE plpgsql VOLATILE;
                                     DROP TRIGGER IF EXISTS tmssapp_trigger_block_task_relation_blueprint_immutable_fields_update ON tmssapp_TaskRelationBlueprint ;
                                     CREATE TRIGGER tmssapp_block_task_relation_blueprint_immutable_fields_update
                                     BEFORE UPDATE ON tmssapp_TaskRelationBlueprint
                                     FOR EACH ROW EXECUTE PROCEDURE tmssapp_block_task_relation_blueprint_immutable_fields_update();
                                     '''),
                   migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_block_subtask_immutable_fields_update()
                                     RETURNS trigger AS
                                     $BODY$
                                     BEGIN
                                       IF OLD.specifications_doc <> NEW.specifications_doc OR
                                          OLD.primary <> NEW.primary OR
                                          OLD.task_blueprint_id <> NEW.task_blueprint_id OR
                                          OLD.specifications_template_id <> NEW.specifications_template_id OR
                                          OLD.cluster_id <> NEW.cluster_id THEN
                                         RAISE EXCEPTION 'ILLEGAL UPDATE OF IMMUTABLE SUBTASK FIELD';
                                       END IF;
                                     RETURN NEW;
                                     END;
                                     $BODY$
                                     LANGUAGE plpgsql VOLATILE;
                                     DROP TRIGGER IF EXISTS tmssapp_trigger_block_subtask_immutable_fields_update ON tmssapp_Subtask ;
                                     CREATE TRIGGER tmssapp_block_subtask_immutable_fields_update
                                     BEFORE UPDATE ON tmssapp_Subtask
                                     FOR EACH ROW EXECUTE PROCEDURE tmssapp_block_subtask_immutable_fields_update();
                                     '''),
                   migrations.RunPython(populate_choices),
                   migrations.RunPython(populate_subtask_allowed_state_transitions),
                   migrations.RunPython(populate_settings),
                   migrations.RunPython(populate_misc),
                   migrations.RunPython(populate_resources),
                   migrations.RunPython(populate_cycles) ]

