from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tmssapp', '0028_subtask_no_raw_feedback'),
    ]

    operations = [
        migrations.AlterField(
            model_name='schedulingunitblueprint',
            name='output_pinned',
            field=models.BooleanField(default=False,
                                      help_text='tri-state boolean (true/false/null, default false), which blocks deleting unpinned dataproducts. When set to True/False, then all its tasks are set to output_pinned=True/False. When one or more tasks are toggled, the aggregated state for this scheduling_unit.output_pinned can be NULL, indicating that not all tasks have the same value.',
                                      null=True),
        ),
        migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_propagate_output_pinned_schedulingunit_to_task()
                          RETURNS trigger AS
                          $BODY$
                          BEGIN
                            IF NEW.output_pinned IS NOT NULL AND OLD.output_pinned IS DISTINCT FROM NEW.output_pinned THEN
                              UPDATE tmssapp_taskblueprint SET output_pinned=NEW.output_pinned WHERE scheduling_unit_blueprint_id=NEW.id;
                            END IF;
                          RETURN NEW;
                          END;
                          $BODY$
                          LANGUAGE plpgsql VOLATILE;
                          DROP TRIGGER IF EXISTS tmssapp_propagate_output_pinned_schedulingunit_to_task ON tmssapp_schedulingunitblueprint;
                          CREATE TRIGGER tmssapp_propagate_output_pinned_schedulingunit_to_task
                          AFTER UPDATE ON tmssapp_schedulingunitblueprint
                          FOR EACH ROW EXECUTE PROCEDURE tmssapp_propagate_output_pinned_schedulingunit_to_task();
                          '''),
        migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_aggregate_output_pinned_from_task_to_schedulingunit()
                      RETURNS trigger AS
                      $BODY$
                      BEGIN
                        IF OLD.output_pinned <> NEW.output_pinned THEN
                            IF (SELECT COUNT(id) FROM tmssapp_taskblueprint WHERE output_pinned=NEW.output_pinned AND scheduling_unit_blueprint_id=NEW.scheduling_unit_blueprint_id)=(SELECT COUNT(id) FROM tmssapp_taskblueprint WHERE scheduling_unit_blueprint_id=NEW.scheduling_unit_blueprint_id) THEN
                                UPDATE tmssapp_schedulingunitblueprint SET output_pinned=NEW.output_pinned WHERE id=NEW.scheduling_unit_blueprint_id;
                            ELSE
                                UPDATE tmssapp_schedulingunitblueprint SET output_pinned=NULL WHERE id=NEW.scheduling_unit_blueprint_id;
                            END IF;
                        END IF;
                      RETURN NEW;
                      END;
                      $BODY$
                      LANGUAGE plpgsql VOLATILE;
                      DROP TRIGGER IF EXISTS tmssapp_aggregate_output_pinned_from_task_to_schedulingunit ON tmssapp_taskblueprint ;
                      CREATE TRIGGER tmssapp_aggregate_output_pinned_from_task_to_schedulingunit
                      AFTER UPDATE ON tmssapp_taskblueprint
                      FOR EACH ROW EXECUTE PROCEDURE tmssapp_aggregate_output_pinned_from_task_to_schedulingunit();
                      '''),
    ]
