# Generated by Django 3.0.9 on 2022-03-31 08:04

from django.db import migrations, models

class Migration(migrations.Migration):

    dependencies = [
        ('tmssapp', '0031_patches'),
    ]

    operations = [
        migrations.AddField(
            model_name='subtaskoutput',
            name='total_nr_of_dataproducts',
            field=models.IntegerField(default=0, editable=False, help_text='Total number of dataproducts in this output.'),
        ),
        migrations.AddField(
            model_name='subtaskoutput',
            name='total_size_of_dataproducts',
            field=models.BigIntegerField(default=0, editable=False, help_text='Total summed size of all dataproducts in this output, in bytes.'),
        ),
        migrations.AddField(
            model_name='subtaskoutput',
            name='total_size_of_dataproducts_on_filesysem',
            field=models.BigIntegerField(default=0, editable=False, help_text='Total summed size of all non-deleleted dataproducts in this output, in bytes.'),
        ),
        migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_dataproduct_compute_aggretates_for_subtaskoutput()
             RETURNS trigger AS
             $BODY$
             DECLARE
               CHANGED_ROW tmssapp_dataproduct%ROWTYPE := CASE WHEN TG_OP='DELETE' THEN OLD ELSE NEW END;
               agg_total_nr_of_dataproducts tmssapp_subtaskoutput.total_nr_of_dataproducts%TYPE;
               agg_total_size_of_dataproducts tmssapp_subtaskoutput.total_size_of_dataproducts%TYPE;
               agg_total_size_of_dataproducts_on_filesysem tmssapp_subtaskoutput.total_size_of_dataproducts_on_filesysem%TYPE;
             BEGIN
                   -- Rationale:
                   -- This trigger function is called for each and every insert/update/delete of each dataproduct
                   -- aggregating the total_nr_of_dataproducts and sizes over all dataproducts in the same subtaskoutput
                   -- This means we have O(N^2) behaviour, which might be bad for large numbers of dataproducts.
                   -- In reality N is usually max 488 dataproducts, and O(N^2) is not (very) bad.
                   -- The cost is also only at dp inserts/updates/deletes, 
                   -- whereas the benifit lies in much faster selects of the aggregated results, which also happen many more times.
                   -- Conclusion: the O(N^2) behaviour pays off for way faster selects.
             
                   -- aggregate the various properties
                   WITH dp_agg AS (SELECT COUNT(id) as total_nr_of_dataproducts,
                                          SUM(COALESCE(size, 0)) as total_size_of_dataproducts,
                                          SUM(CASE WHEN deleted_since IS NULL THEN COALESCE(size, 0) ELSE 0 END) as total_size_of_dataproducts_on_filesysem
                                          FROM tmssapp_dataproduct
                                          WHERE tmssapp_dataproduct.producer_id=CHANGED_ROW.producer_id)
                   -- temporarily store them in the declared variables
                   SELECT COALESCE(total_nr_of_dataproducts, 0), COALESCE(total_size_of_dataproducts, 0), COALESCE(total_size_of_dataproducts_on_filesysem, 0)
                   INTO   agg_total_nr_of_dataproducts, agg_total_size_of_dataproducts, agg_total_size_of_dataproducts_on_filesysem
                   FROM dp_agg;

                   -- and store them into the tmssapp_subtaskoutput table
                   -- note that this will cause a new trigger on to tmssapp_subtaskoutput_compute_aggretates_for_subtask
                   UPDATE tmssapp_subtaskoutput
                   SET total_nr_of_dataproducts=agg_total_nr_of_dataproducts, 
                       total_size_of_dataproducts=agg_total_size_of_dataproducts,
                       total_size_of_dataproducts_on_filesysem=agg_total_size_of_dataproducts_on_filesysem
                   WHERE id=CHANGED_ROW.producer_id;
               RETURN CHANGED_ROW;
               END;
               $BODY$
               LANGUAGE plpgsql VOLATILE;
               DROP TRIGGER IF EXISTS tmssapp_dataproduct_compute_aggretates_for_subtaskoutput ON tmssapp_dataproduct ;
               CREATE TRIGGER tmssapp_dataproduct_compute_aggretates_for_subtaskoutput
               AFTER INSERT OR UPDATE OR DELETE ON tmssapp_dataproduct
               FOR EACH ROW EXECUTE PROCEDURE tmssapp_dataproduct_compute_aggretates_for_subtaskoutput();
             '''),
        migrations.AddField(
            model_name='subtask',
            name='total_nr_of_dataproducts',
            field=models.IntegerField(default=0, editable=False,
                                      help_text='Total number of output dataproducts.')
        ),
        migrations.AddField(
            model_name='subtask',
            name='total_size_of_dataproducts',
            field=models.BigIntegerField(default=0, editable=False,
                                         help_text='Total summed size of all output dataproducts, in bytes.')
        ),
        migrations.AddField(
            model_name='subtask',
            name='total_size_of_dataproducts_on_filesysem',
            field=models.BigIntegerField(default=0, editable=False,
                                         help_text='Total summed size of all non-deleleted output dataproducts, in bytes.')
        ),
        migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_subtaskoutput_compute_aggretates_for_subtask()
         RETURNS trigger AS
         $BODY$
         DECLARE
           CHANGED_ROW tmssapp_subtaskoutput%ROWTYPE := CASE WHEN TG_OP='DELETE' THEN OLD ELSE NEW END;
           agg_total_nr_of_dataproducts tmssapp_subtask.total_nr_of_dataproducts%TYPE;
           agg_total_size_of_dataproducts tmssapp_subtask.total_size_of_dataproducts%TYPE;
           agg_total_size_of_dataproducts_on_filesysem tmssapp_subtask.total_size_of_dataproducts_on_filesysem%TYPE;
         BEGIN
               -- aggregate the various properties
               WITH subtaskoutput_agg AS (SELECT SUM(total_nr_of_dataproducts) as total_nr_of_dataproducts,
                                                 SUM(total_size_of_dataproducts) as total_size_of_dataproducts,
                                                 SUM(total_size_of_dataproducts_on_filesysem) as total_size_of_dataproducts_on_filesysem
                                          FROM tmssapp_subtaskoutput
                                          WHERE tmssapp_subtaskoutput.subtask_id=CHANGED_ROW.subtask_id)
               -- temporarily store them in the declared variables
               SELECT COALESCE(total_nr_of_dataproducts, 0), COALESCE(total_size_of_dataproducts, 0), COALESCE(total_size_of_dataproducts_on_filesysem, 0)
               INTO   agg_total_nr_of_dataproducts, agg_total_size_of_dataproducts, agg_total_size_of_dataproducts_on_filesysem
               FROM subtaskoutput_agg;

               -- and store them into the tmssapp_subtask table
               UPDATE tmssapp_subtask
               SET total_nr_of_dataproducts=agg_total_nr_of_dataproducts, 
                   total_size_of_dataproducts=agg_total_size_of_dataproducts,
                   total_size_of_dataproducts_on_filesysem=agg_total_size_of_dataproducts_on_filesysem
               WHERE id=CHANGED_ROW.subtask_id;
           RETURN CHANGED_ROW;
           END;
           $BODY$
           LANGUAGE plpgsql VOLATILE;
           DROP TRIGGER IF EXISTS tmssapp_subtaskoutput_compute_aggretates_for_subtask ON tmssapp_dataproduct ;
           CREATE TRIGGER tmssapp_subtaskoutput_compute_aggretates_for_subtask
           AFTER INSERT OR UPDATE OR DELETE ON tmssapp_subtaskoutput
           FOR EACH ROW EXECUTE PROCEDURE tmssapp_subtaskoutput_compute_aggretates_for_subtask();
         '''),
    ]
