# Generated by Django 3.0.9 on 2021-10-07 09:17

from django.db import migrations, models
import django.db.models.deletion

from lofar.sas.tmss.tmss.tmssapp.populate import populate_task_schedulingunit_choices


class Migration(migrations.Migration):

    dependencies = [
        ('tmssapp', '0037_populate_subtask_allowed_state_transitions_on_hold')
    ]

    operations = [
        migrations.RunSQL('''CREATE OR REPLACE FUNCTION tmssapp_subtask_compute_aggretates()
              RETURNS trigger AS
              $BODY$
              DECLARE
                subtask_type character varying(128);
              BEGIN
                -- this trigger function runs before the actual row is stored, so we can modify/compute some values  
                -- compute the aggregated on_sky/process start/stop times, based on what is already known

                -- process_start/stop_times are the 'best result' of what happened, or what should happen. (use COALESCE)
                -- if actual_process_start/stop_time was already set, then these are used
                -- if actual_process_start/stop_time was still NULL, then are scheduled_start/stop_time are used (which can be NULL)
                NEW.process_start_time := COALESCE(NEW.actual_process_start_time, NEW.scheduled_start_time); 
                NEW.process_stop_time := COALESCE(NEW.actual_process_stop_time, NEW.scheduled_stop_time);
                
                -- any on_sky time may only be set/computed for observations
                SELECT tmssapp_subtasktemplate.type_id INTO subtask_type 
                FROM tmssapp_subtask 
                INNER JOIN tmssapp_subtasktemplate on tmssapp_subtasktemplate.id=tmssapp_subtask.specifications_template_id
                WHERE tmssapp_subtask.id=NEW.id
                LIMIT 1;
                
                IF subtask_type='observation' THEN
                    -- process_start/stop_times are the 'best result' of what happened, or what should happen. (use COALESCE)
                    -- if actual_on_sky_start/stop_time was already set, then these are used
                    -- if actual_on_sky_start/stop_time was still NULL, then are scheduled_start/stop_time are used (which can be NULL)
                    NEW.on_sky_start_time := COALESCE(NEW.actual_on_sky_start_time, NEW.scheduled_start_time); 
                    NEW.on_sky_stop_time := COALESCE(NEW.actual_on_sky_stop_time, NEW.scheduled_stop_time); 
                ELSE
                    -- erase any user-set on_sky time for non-observations
                    NEW.actual_on_sky_start_time := NULL; 
                    NEW.actual_on_sky_stop_time := NULL; 
                    NEW.on_sky_start_time := NULL; 
                    NEW.on_sky_stop_time := NULL; 
                END IF;

                -- compute the durations using the just computed on_sky/process start/stoptimes
                NEW.duration := AGE(NEW.process_stop_time, NEW.process_start_time); 
                -- on_sky_duration can/should be NULL for non-observations. 
                NEW.on_sky_duration := AGE(NEW.on_sky_stop_time, NEW.on_sky_start_time); 
                
                -- return modified NEW row with aggregated times, which are then stored during the actual insert/update
                RETURN NEW;
              END;
              $BODY$
              LANGUAGE plpgsql VOLATILE;
              DROP TRIGGER IF EXISTS tmssapp_subtask_compute_aggretates ON tmssapp_subtask ;
              CREATE TRIGGER tmssapp_subtask_compute_aggretates
              BEFORE INSERT OR UPDATE ON tmssapp_subtask
              FOR EACH ROW EXECUTE PROCEDURE tmssapp_subtask_compute_aggretates();
              ''')
    ]
