from rest_framework.renderers import JSONRenderer, SHORT_SEPARATORS, LONG_SEPARATORS, JSONRenderer
import logging
logger = logging.getLogger(__name__)

try:
    import orjson

    class ORJSONRenderer(JSONRenderer):
        """
        Faster version of plain drf JSONRenderer using orjson which supports datetime serialization
        """
        def render(self, data, accepted_media_type=None, renderer_context=None):
            """
            Render `data` into JSON, returning a bytestring.
            """
            if data is None:
                return b''

            options = orjson.OPT_NAIVE_UTC | orjson.OPT_UTC_Z
            if self.get_indent(accepted_media_type, renderer_context or {}):
                options = options | orjson.OPT_INDENT_2

            return orjson.dumps(data, option=options)

except ImportError as e:
    logger.warning("Using default JSONRenderer because of import error. %s" % e)
    class ORJSONRenderer(JSONRenderer):
        pass
