from rest_framework.renderers import JSONRenderer, SHORT_SEPARATORS, LONG_SEPARATORS, JSONRenderer
import logging
logger = logging.getLogger(__name__)

try:
    import ujson

    class UJSONRenderer(JSONRenderer):
        """
        Faster version of plain drf JSONRenderer using ujson (which does NOT support datetime serialization)
        """
        def render(self, data, accepted_media_type=None, renderer_context=None):
            """
            Render `data` into JSON, returning a bytestring.
            """
            if data is None:
                return b''

            renderer_context = renderer_context or {}
            indent = self.get_indent(accepted_media_type, renderer_context)

            ret = ujson.dumps(data,
                              indent=indent or 0,
                              ensure_ascii=self.ensure_ascii,
                              escape_forward_slashes=False)

            # For speed reasons we do not replace \u2028 and \u2029,
            # as we hardly encounter them and there is a severe speed penalty in replacing them.
            # See remarks in super().render

            return ret.encode()

except ImportError as e:
    logger.warning("Using default JSONRenderer because of import error. %s" % e)
    class UJSONRenderer(JSONRenderer):
        pass
