from datetime import datetime
from lofar.sas.tmss.tmss.tmssapp import models

def get_reserved_stations_in_timewindow(lower_bound: datetime,
                                        upper_bound: datetime=None,
                                        ignore_reservations_allowing_scheduling_for_project: models.Project=None,
                                        ignore_reservations_allowing_fixed_time_scheduling: bool=False,
                                        ignore_reservations_allowing_dynamic_scheduling: bool=False) -> list:
    """
    Get a list of all stations which are reserved in the given timewindow (upper_bound can be None)
    """
    reserved_stations = set()
    queryset = models.Reservation.objects.filter(specifications_doc__resources__stations__isnull=False)

    if upper_bound is not None:
        queryset = queryset.filter(start_time__lt=upper_bound)

    if ignore_reservations_allowing_scheduling_for_project is None:
        if ignore_reservations_allowing_fixed_time_scheduling:
            queryset = queryset.exclude(specifications_doc__schedulability__fixed_time=True)

        if ignore_reservations_allowing_dynamic_scheduling:
            queryset = queryset.exclude(specifications_doc__schedulability__dynamic=True)
    else:
        queryset = queryset.exclude(specifications_doc__schedulability__project_exclusive=True,
                                    project=ignore_reservations_allowing_scheduling_for_project.name)

    for res in queryset.filter(stop_time=None).values('specifications_doc__resources__stations').all():
        reserved_stations.update(res["specifications_doc__resources__stations"])

    if lower_bound is not None:
        for res in queryset.filter(stop_time__gt=lower_bound).values('specifications_doc__resources__stations').all():
            reserved_stations.update(res["specifications_doc__resources__stations"])

    reserved_stations = sorted(list(reserved_stations))
    return reserved_stations
