# Create your models here.

from django.db.models import TextField, IntegerField,BooleanField, ForeignKey, CASCADE, Model,NullBooleanField
from viewflow.models import Process, Task
from viewflow.fields import FlowReferenceField
from viewflow.compat import _

from lofar.sas.tmss.tmss.tmssapp.models import SchedulingUnitBlueprint
from lofar.sas.tmss.tmss.tmssapp.models.common import ProjectPropertyMixin

class QAReportingTO(Model, ProjectPropertyMixin):
    operator_report = TextField()
    operator_accept = BooleanField(default=False)
    path_to_project = 'flow_process__su__draft__scheduling_set__project'

class QAReportingSOS(Model, ProjectPropertyMixin):
    sos_report = TextField()
    quality_within_policy = BooleanField(default=False)
    sos_accept_show_pi = BooleanField(default=False)
    path_to_project = 'flow_process__su__draft__scheduling_set__project'

class PIVerification(Model, ProjectPropertyMixin):
    pi_report = TextField()
    pi_accept = BooleanField(default=False)
    path_to_project = 'flow_process__su__draft__scheduling_set__project'

class DecideAcceptance(Model, ProjectPropertyMixin):
    sos_accept_after_pi = BooleanField(default=False)
    path_to_project = 'flow_process__su__draft__scheduling_set__project'

class UnpinData(Model, ProjectPropertyMixin):
    unpin_data = BooleanField(default=False)
    path_to_project = 'flow_process__su__draft__scheduling_set__project'

class SchedulingUnitTask(Task, ProjectPropertyMixin):
    path_to_project = 'flow_process__su__draft__scheduling_set__project'  # Note: flow_process is not a db field but a property here

class SchedulingUnitProcess(Process, ProjectPropertyMixin):
    """
    The viewflow process object contains attributes that capture the state of a running workflow.
    # TODO: Review this when we implement another workflow.
    #   Is this setup with separate data models for the individual steps reasonable?
    #   This could simply be a collection of the fields in the individual models.
    #   It seems as if this is explicitly modeled so that we can have separate views outside of what viewflow offers and
    #   if I recall correctly than this was mainly driven by the wish that e.g. a report can be changed later.
    #   This implementation is insofar understandable as the viewflow REST stuff is a non-free feature and we kind of
    #   created our own viewsets now. But if this is purely to allow alterations at a later point in time, then I would
    #   give this another thought. It probably makes more sense to come up with a workflow that matches the order of how
    #   things are actually done then to find ways to allow meddling with things at a point in time where decisions have
    #   already been made based on the original value.
    #   This whole setup with tons of workflow-specific models/serializers/views feels very clunky and non-generic for
    #   something that should be relatively lightweight to implement.
    #   Also, at least I find this quite confusing because instead of a simple state container, we now have models that
    #   represent individual steps of a workflow, i.e. their objects represent individual tasks and we have a REST
    #   interface to use them. However, in viewflow logic there is a separate model for (all kinds of) tasks that is
    #   usually the interface for the user to interact with the system.
    """
    su = ForeignKey(SchedulingUnitBlueprint, blank=True, null=True, on_delete=CASCADE)
    qa_reporting_to = ForeignKey(QAReportingTO, blank=True, null=True, related_name='flow_process', on_delete=CASCADE)
    qa_reporting_sos = ForeignKey(QAReportingSOS, blank=True, null=True, related_name='flow_process', on_delete=CASCADE)
    pi_verification = ForeignKey(PIVerification, blank=True, null=True, related_name='flow_process', on_delete=CASCADE)
    decide_acceptance = ForeignKey(DecideAcceptance, blank=True, null=True, related_name='flow_process', on_delete=CASCADE)
    unpin_data = ForeignKey(UnpinData, blank=True, null=True, related_name='flow_process', on_delete=CASCADE)
    path_to_project = 'su__draft__scheduling_set__project'
