import  { useEffect ,useState} from "react";
import {
    Route,
    Redirect,
} from 'react-router-dom';
import AuthStore from '../authenticate/auth.store';
import AuthUtil from "../utils/auth.util";
import PropTypes from 'prop-types';

export default function ProtectedRoute({permissions,name, component, path,  location}) {

    const [permissionSet, setPermissionSet] = useState(AuthStore.getState());
    const init = async () =>{
        const permission = await AuthUtil.getUserRolePermission();
        setPermissionSet(permission);
    }
    useEffect(() => {
        init();
    },[]);
    const hasPermission =()=> {
        const permission = permissions
        if(permissionSet['userRolePermission']){
            if(permission[0] !== undefined && permission[1] !== undefined) {
                if(permissionSet['userRolePermission'][permission[0]] && permissionSet['userRolePermission'][permission[0]][permission[1]]) {
                    return true;
                } else {
                    return false;
                }
            }
        }
        return true
    }

    const updatePageTracker = (path)=> {
        let pageTracker = sessionStorage.getItem("pageTracker");
        pageTracker = pageTracker?JSON.parse(pageTracker):{};
        if (pageTracker.currPath !== path) {
            pageTracker.prevPath = pageTracker.currPath || path;
        }
        pageTracker.currPath = path;
        sessionStorage.setItem("pageTracker", JSON.stringify(pageTracker));
    }


    
    const  doRender =()=>{
        if(permissions) {
           if(hasPermission()) {
                updatePageTracker(location.pathname);
                return <Route path={path} name={name} component={component} />
            }
            else {
                return <Redirect to={{
                    pathname: '/access-denied',
                    state: { from: location }
                }}/> 
            }
        }
        else {
            updatePageTracker(location.pathname);
            return <Route path={path} name={name} component={component} />
        }
    }
    return doRender();
}

ProtectedRoute.propTypes = {
  permissions: PropTypes.array,
  name: PropTypes.string,
  component: PropTypes.oneOfType([PropTypes.object, PropTypes.func]),
  path: PropTypes.string,
  location: PropTypes.oneOfType([PropTypes.object, PropTypes.string])
}
