import React,{ Component } from 'react';
import PropTypes from 'prop-types';
import { matchPath} from 'react-router-dom';
import { routes } from '../../routes';
import { withRouter } from 'react-router-dom/cjs/react-router-dom.min';
export class AppBreadcrumb extends Component {
    
    static propTypes = {
    location: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),
    section: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),
    setPageTitle: PropTypes.func,
    history: PropTypes.object,
    onBreadcrumbClick: PropTypes.func
  }

    constructor(props) {
        super(props);
        this.state = {
            breadcrumbs: []
        }
    }

    componentDidUpdate(prev) {
        if (prev.location?.pathname !== this.props.location?.pathname) {
            if (this.props.section !== prev.section) {
                this.onRoute(true);
                return;
            }
            this.onRoute();
        }
    }

    componentDidMount() {
        this.onRoute();
    }

    onRoute(reset) {
        const { breadcrumbs } = this.state;
        const { setPageTitle } = this.props;
        const currentRoute = routes.find(route => matchPath(this.props.location?.pathname, {path: route.path, exact: true, strict: true}));
		//for intial route ,there wont be any route object so it failed 
		if(!currentRoute){
			return;
        }
        setPageTitle(currentRoute.pageTitle);
        if (!breadcrumbs.length || reset) {
            this.setState({ breadcrumbs: [{...this.props.location, name: currentRoute.name}] });
            return;
        }
        const index = breadcrumbs.map(i => i.name).indexOf(currentRoute.name);
        if (index === -1) {
            this.setState({ breadcrumbs: [...breadcrumbs, {...this.props.location, name: currentRoute.name}] });
            return;
        }
        this.setState({ breadcrumbs: breadcrumbs.slice(0, index+1) });
    }

    onNavigate = (item) => {
        this.props.history.push({
            pathname: item.pathname,
            state: item.state
        });
    }
	
	render() {
        const { breadcrumbs } = this.state;
        return (
            <div className="p-breadcrumb" >
                <span className="pi b-home"></span> <span className="pi pi-home"> </span>
                {breadcrumbs.map((item, index) => (
                    <span key={"bc_" + item.name} >
                        <li className="pi pi-chevron-right b-separator"></li>
                        {index !== breadcrumbs.length - 1 ? 
                            <span className="b-link"     onClick={() => this.props.onBreadcrumbClick(() => this.onNavigate(item))}      
                             onKeyDown={(e) => {  if (e.key === 'Enter') { this.props.onBreadcrumbClick(() => this.onNavigate(item)); }  }}                >{item.name}</span> 
                            : <span className="b-text">{item.name}</span>}
                    </span>
                ))}
            </div>
        );
    }
}

export default React.memo(withRouter(AppBreadcrumb));