import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import _ from 'lodash';
import PropTypes from 'prop-types';

/**
 * Average efficiency report component for cycle report.
 */

export default function CycleReportAvgEfficiency(props) {
    let { data } = props

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    const getBarData = ()=> {
        return {
            labels: _.map(data, 'cycle'),
            datasets: [
                {
                    type: 'line',
                    label: 'Target',
                    borderColor: '#ff0000',
                    backgroundColor: '#ff0000',
                    borderWidth: 2,
                    fill: false,
                    data: _.map(data, 'target')
                },
                {
                    type: 'bar',
                    label: 'Efficiency',
                    borderColor: '#44a3ce',
                    backgroundColor: '#44a3ce',
                    barThickness: 50,
                    // barPercentage: 0.3,
                    data: _.map(data, 'efficiency'),
                }
            ],
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    const  getBarOptions = ()=> {
        return {
            // indexAxis: 'y',
            // Elements options apply to all of the options unless overridden in a dataset
            // In this case, we are setting the border of each horizontal bar to be 1px wide
            elements: {
              bar: {
                borderWidth: 1,
              },
            },
            scales: {
                y: {
                    max: 100
                }
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'Average Efficiency',
              },
            }
        };
    }

    /**
     * Function to format data list after calculations and include average if require for table or chart.
     * @param {Boolean} forTable - flag to incluce average value if the data is for table component.
     * @returns Array
     */


        return (
            <>
                <Bar data={getBarData()} options={getBarOptions()} />
                <DataTable value={data} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                    <Column field="cycle" header="Cycle code"></Column>
                    <Column field="efficiency" header="Efficiency (%)"></Column>
                    <Column field="target" header="Target (%)"></Column>
                </DataTable>
            </>
        );

}

CycleReportAvgEfficiency.propTypes = {
  data: PropTypes.array
}