import React  from "react";
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import PropTypes from 'prop-types';


/**
 * Component to show data ingested per category in cycle(s).
 */
export default function CycleCategoryObservations(props) {
    let { data } = props;
    const getBarData = () => {
        return {
            labels: _.map(data, 'cycle'),
            datasets: [
                {
                    label: 'Prio A',
                    data: _.map(data, 'prio_A'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                },
                {
                    label: 'Prio B',
                    data: _.map(data, 'prio_B'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,

                },
                {
                    label: 'DDT_Com_Rep',
                    data: _.map(data, 'ddt_com_rep'),
                    backgroundColor: '#A5A5A5',
                    barThickness: 75,
                },
                {
                    label: 'TMSS Com',
                    data: _.map(data, 'tmss_com'),
                    backgroundColor: '#FFC000',
                    barThickness: 75,
                },
                {
                    label: 'System Unavailability/O_com',
                    data: _.map(data, 'unavailable'),
                    borderColor: '#ff0000',
                    backgroundColor: '#ff0000',
                    barThickness: 75,
                },
                {
                    label: 'Failed',
                    data: _.map(data, 'failed'),
                    backgroundColor: '#44a3ce',
                    barThickness: 75,
                },
                {
                    label: 'System/Idle',
                    data: _.map(data, 'idle'),
                    backgroundColor: '#4caf50',
                    barThickness: 75,
                }
            ]
        };
    }

    const getBarOptions = () => {
        return {
            indexAxis: 'x',
            elements: {
                bar: {
                    borderWidth: 0,
                },
            },
            scales: {
                x: {
                    stacked: true,
                    categoryPercentage: 1.0,
                    barPercentage: 1.0
                },
                y: {
                    stacked: true,
                    // max: 5000,
                }
            },
            responsive: true,
            plugins: {
                legend: {
                    position: 'bottom',
                },
                title: {
                    display: true,
                    text: 'Observation Hours per Category',
                }
            }
        };
    }

    return (<React.Fragment>
        <div id={`cycles-category-time`}
            style={{ paddingTop: "10px", paddingBottom: "10px" }}>
            <Bar data={getBarData()} options={getBarOptions()} width="50%" height="20" />
        </div>

        <div className="ttd-details" id={`cycle-ttd-details`}>
            <DataTable value={data} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em', textAlign: 'center' }}>
                <Column field="cycle" header="Cycle Code"></Column>
                <Column field="prio_A" header="Prio A (hrs)"></Column>
                <Column field="prio_B" header="Prio B (hrs)"></Column>
                <Column field="ddt_com_rep" header="DDT Com Rep (hrs)"></Column>
                <Column field="tmss_comm" header="TMSS Comm (hrs)"></Column>
                <Column field="unavailable" header="System Unavailability (hrs)"></Column>
                <Column field="failed" header="Failed (hrs)"></Column>
                <Column field="idle" header="System/Idle (hrs)"></Column>
            </DataTable>
        </div>

    </React.Fragment>
    )


}

CycleCategoryObservations.propTypes = {
  data: PropTypes.array
}