import  { useState  } from 'react';
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import { Dropdown } from 'primereact/dropdown';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import _ from 'lodash';
import PropTypes from 'prop-types';


/**
 * Weekly efficiency report component for cycle report.
 */
export default function CycleWeeklyEfficiency(props) {
    let { data  } = props;
    const [selectedCycle, setSelectedCycle] = useState(data?.cycles && data.cycles.length>0 && data.cycles[0]);

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    const getBarData = () => {
        const currentSelectedCycle = selectedCycle || (data.cycles && data.cycles.length > 0 && data.cycles[0]);
        const barData = _.filter(data.chartData, ['cycle', currentSelectedCycle]);
        return {
            labels: _.map(barData, 'Week Index'),
            datasets: [
                {
                    type: 'bar',
                    label: 'Efficiency %',
                    borderColor: '#44a3ce',
                    backgroundColor: '#44a3ce',
                    borderWidth: 2,
                    barThickness: 25,
                    data: _.map(barData, 'efficiency'),
                }
            ],
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    const getBarOptions = () => {
        const currentSelectedCycle = selectedCycle || (data.cycles && data.cycles.length > 0 && data.cycles[0]) || '';
        return {
            elements: {
                bar: {
                    borderWidth: 1,
                },
            },
            scales: {
                y: {
                    max: 100
                }
            },
            responsive: true,
            plugins: {
                legend: {
                    position: 'bottom',
                },
                title: {
                    display: true,
                    text: currentSelectedCycle + ' Weekly Efficiency (%)'
                },
            }
        };
    }

    const renderReportTable = () => {
        const reportTableData = data.tableData;
        const reportColumns = _.keys(reportTableData[0]).map((col) => {
            return <Column key={col} field={col} header={col} />;
        });
        return (
            <DataTable value={reportTableData} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                {reportColumns}
            </DataTable>
        );
    }


    return (
        <>
            {data?.cycles && <Dropdown value={selectedCycle} options={data?.cycles}
                onChange={(e) => { setSelectedCycle(e.value) }}
                style={{ width: '200px', marginLeft: '20px' }} />}
            {data &&
                <>
                    <Bar data={getBarData()} options={getBarOptions()} />
                    {renderReportTable()}
                </>
            }
        </>
    );

}


CycleWeeklyEfficiency.propTypes = {
  data: PropTypes.object,
  cycles: PropTypes.array
}