import PropTypes from 'prop-types';
import { Component } from 'react'
import 'primeflex/primeflex.css';
import moment from 'moment';
import AppLoader from "./../../layout/components/AppLoader";
import ViewTable from './../../components/ViewTable/ViewTable';
import { CustomPageSpinner } from '../../components/CustomPageSpinner';
import UnitConverter from '../../utils/unit.converter';
import _ from 'lodash';
import ScheduleService from '../../services/schedule.service';
import { Link } from 'react-router-dom';
import UIConstants from '../../utils/ui.constants';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { appGrowl } from '../../layout/components/AppGrowl';
import UtilService from '../../services/util.service';
import { Dropdown } from 'primereact/dropdown';
import SUBCreator from './sub.create';
import AuthUtil from '../../utils/auth.util';
import TimelineCommonUtils from '../Timeline/common.utils';
import Websocket from 'react-websocket';
import { InputText } from 'primereact/inputtext';
import { InputTextarea } from 'primereact/inputtextarea';
import ProjectService from '../../services/project.service';


const derviedscheduleconstraints = {};
class SchedulingUnitList extends Component{
    defaultSortColumn= [{id: "Name", desc: false}]; 
    SU_NOT_STARTED_STATUSES = ['defined', 'schedulable', 'scheduled', 'unschedulable'];
    SU_ACTIVE_STATUSES = ['started', 'observing', 'observed', 'processing', 'processed', 'ingesting'];
    SU_END_STATUSES = ['finished', 'error', 'cancelled'];
    //Expand details for API Query
    SU_DRAFT_EXPAND = 'scheduling_set,scheduling_set.project,observation_strategy_template,task_drafts,task_drafts.specifications_template,scheduling_unit_blueprints';
    SU_DRAFT_FIELDS = ['id', 'scheduling_unit_blueprints.name', 'scheduling_unit_blueprints.url', 'scheduling_unit_blueprints.name','scheduling_unit_blueprints_ids', 'scheduling_set.name',
    'scheduling_set.project.name','scheduling_set.project.rank', 'name', 'description', 'priority', 'priority_queue_value', 'rank', 'scheduling_unit_blueprints.id',
    'observation_strategy_template.id', 'observation_strategy_template.name', 'observation_strategy_template.description',
    'duration', 'do_cancel', 'output_pinned', 'created_at', 'updated_at', 'task_drafts.id', 'task_drafts.name', 'task_drafts.output_pinned', 'task_drafts.url',
    'task_drafts.specifications_doc', 'task_drafts.specifications_template.id', 'task_drafts.specifications_template.name',
    'task_drafts.specifications_template.type_value', 'scheduling_constraints_template', 'observation_strategy_template.scheduling_unit_template', 'scheduling_constraints_doc'
    ];
    SU_BLUEPRINT_EXPAND = 'draft,draft.scheduling_set,draft.scheduling_set.project,draft.observation_strategy_template,task_blueprints,task_blueprints.specifications_template,task_blueprints.subtasks';
    SU_BLUEPRINT_FIELDS = ['id', 'status', 'draft_id', 'draft.id', 'draft.scheduling_set.name', 'draft.scheduling_set.project.name','draft.scheduling_set.project.rank', 'scheduling_set.id', 'draft.name',
    'name', 'description', 'priority', 'priority_queue_value', 'rank', 'unschedulable_reason', 'draft.observation_strategy_template.id', 'draft.scheduling_constraints_doc',
    'draft.observation_strategy_template.name', 'draft.observation_strategy_template.description', 'on_sky_start_time', 'draft.observation_strategy_template.scheduling_unit_template',
    'on_sky_stop_time', 'scheduled_central_lst', 'process_start_time', 'process_stop_time', 'duration', 'do_cancel', 'results_accepted', 'output_pinned', 'created_at',
    'updated_at', 'task_blueprints.id', 'task_blueprints.name', 'task_blueprints.specifications_doc', 'task_blueprints.specifications_template.id', 'draft.scheduling_unit_blueprints',
    'task_blueprints.specifications_template.name', 'task_blueprints.specifications_template.type_value','task_blueprints.subtasks.specifications_doc', 'scheduling_constraints_template', 'scheduling_constraints_doc', 'observation_strategy_template.specifications_template'
    ];
    SU_BPCONSTRAINTS_EXPAND = 'draft,draft.scheduling_set,draft.scheduling_set.project,draft.observation_strategy_template,task_blueprints,task_blueprints.specifications_template,task_blueprints.subtasks'
    SU_BPCONSTRAINTS_FIELDS = ['id', 'status', 'draft_id', 'draft.scheduling_set.name', 'draft.scheduling_set.project.name','draft.scheduling_set.project.rank', 'name', 'description', 'priority', 'results_accepted', 'unschedulable_reason', 'scheduling_constraints_doc','task_blueprints.id', 'task_blueprints.name', 'task_blueprints.specifications_doc', 'task_blueprints.specifications_template.id',
    'task_blueprints.specifications_template.name', 'task_blueprints.specifications_template.type_value','task_blueprints.subtasks', 'draft.scheduling_set.id'];                        
    COMMA_SEPARATE_TOOLTIP ='Enter Id and press ‘Enter’ key to search. For multiple values enter comma separated values. For range, provide input like 1..6';

    constructor(props){
       super(props);
       this.setToggleBySorting();
       this.selectedSURows = [];
       this.defaultcolumns = {
        status: {
            name:"Status",
            filter: "multiselect-filter",
            showTag: true,
            tagClass: 'su-$dynamic$' //$dynamic$ will be replace with cell value
        },
        unschedulable_reason: {
            name:"Unschedulable Reason",    
        },
        workflowStatus: {name: "Workflow Status",filter: "select"},
        results_accepted: {name: "Data Accepted", filter: 'none'},
        suid: {name: "Scheduling Unit ID", tooltip: this.COMMA_SEPARATE_TOOLTIP},
        project:{name:"Project"},
        name:{name:"Name"},
        description: {name: "Description"},
        project_rank: {name: "Project Rank", filter: 'rankMinMax'},
        priority_queue: {name: "Priority Queue", filter: "numberRangeMinMax"},
        scheduling_set: {name: "Scheduling Set",},       
        observation_strategy_template_name: {name: "Template Name"},
        observation_strategy_template_description: {name: "Template Description"},
        on_sky_start_time:{
            name:"Observation Start Time",
            filter:"flatpickrDateRange",
            format:UIConstants.CALENDAR_DATETIME_FORMAT
        },
        on_sky_stop_time:{
            name:"Observation End Time",
            filter:"flatpickrDateRange",
            format:UIConstants.CALENDAR_DATETIME_FORMAT
        },
        scheduled_central_lst:{
            name:"Central LST",
            format:UIConstants.CALENDAR_DATETIME_FORMAT
        },
        process_start_time:{
            name:"Process Start Time",
            filter:"flatpickrDateRange",
            format:UIConstants.CALENDAR_DATETIME_FORMAT
        },
        process_stop_time:{
            name:"Process End Time",
            filter:"flatpickrDateRange",
            format:UIConstants.CALENDAR_DATETIME_FORMAT
        },
        duration:{
            name:"Duration (HH:mm:ss)",
            filter: "durationMinMax",
            format:UIConstants.CALENDAR_TIME_FORMAT
        },
        station_group: {name: "Stations (CS/RS/IS)"},
        task_content: {name: "Tasks content (O/P/I)"},
        target_observation_sap: {name: "Number of SAPs in the target observation"},
        do_cancel: {
            name: "Cancelled",
            filter: "switch",
        },
        targetName: {name: 'Target - Name'},
        targetAngle1: {name: 'Target - Angle 1'},
        targetAngle2: {name: 'Target - Angle 2'},
        targetRef: {name: 'Target - Reference Frame'},
    }
       
        if (props.hideProjectColumn) {
            delete this.defaultcolumns['project'];
        }
        this.getUIAttr();
        this.STATUS_BEFORE_SCHEDULED = ['defining', 'defined', 'schedulable'];  // Statuses before scheduled to get station_group
        this.mainStationGroups = {};
        this.suTypeList = [{name: 'Draft'}, {name: 'Drafts Without BluePrint'},{name: 'Blueprint'}, {name: 'Constraints'}]
        this.totalPage = 0;
        this.statusList = [];
        this.definitionCache = [];
        this.state = {
            showSpinner: false,
            suType: this.suUIAttr['listType']?.replaceAll("_"," ") || "Draft",
            columnOrders: [ 
                "Status",
                "Workflow Status",
                "Data Accepted",
                "Scheduling Unit ID",
                "Linked Draft ID",
                "Project",
                "Name",
                "Description",
                "Project Rank",
                "Priority Queue",
                "Rank",
                "Scheduling Set",
                "Template ID",
                "Template Name",
                "Template Description",
                "Observation Start Time",
                "Observation End Time", 
                "Process Start Time",
                "Process End Time", 
                "Duration (HH:mm:ss)",
                "Pin Data",
                "Stations (CS/RS/IS)", 
                "Tasks content (O/P/I)", 
                "Number of SAPs in the target observation", 
                "Target - Name", 
                "Target - Angle 1", 
                "Target - Angle 2", 
                "Target - Reference Frame", 
                "Created_At", 
                "Updated_At",
                "Cancelled"
            ],
            constraintsColumnOrders: [ 
                "Status",
                "Type",
                "Workflow Status",
                "Scheduling Unit ID",
                "Project",
                "Project Rank",
                "Scheduling Set",
                "Name",
                "Description",
                "Stations (CS/RS/IS)",
            ],
            scheduleunit: [],
            paths: [{
                "View": "/schedulingunit/view",
            }],
            isLoading: true,
            defaultcolumns: [this.defaultcolumns],
            optionalcolumns:  [{
                actionpath:"actionpath",
                draft:{name: "Linked Blueprint Id", tooltip: this.COMMA_SEPARATE_TOOLTIP},
                //priority_queue: {name: "Priority Queue", filter: "select"},
                rank: {name: "Rank",filter: "rankMinMax"},
                observation_strategy_template_id:{name: "Template ID", tooltip: 'Enter Id and press ‘Enter’ key to search'},
                created_at:{
                    name:"Created_At",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                updated_at:{
                    name:"Updated_At",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                output_pinned:{name: "Pin Data", filter: "switch"},
            }],
            columnclassname: [{
                "Scheduling Unit ID":"filter-input-100",
                "Template":"filter-input-50",
                "Template Description": "filter-input-200",
                "Scheduling Set": "filter-input-150",
                "Project":"filter-input-100",
                "Project Rank":"filter-input-75",
                "Rank":"filter-input-75",
                "Duration (HH:mm:ss)":"filter-input-100",
                "Linked Draft ID":"filter-input-100",
                "Linked Blueprint ID":"filter-input-100",
                "Type": "filter-input-75",
                "Status":"filter-input-125",
                "workflowStatus":"filter-input-100",
                "Data Accepted":"filter-input-50",
                "Stations (CS/RS/IS)":"filter-input-50",
                "Tasks content (O/P/I)":"filter-input-50",
                "Number of SAPs in the target observation":"filter-input-50",
                "Observation Start Time":"filter-input-150",
                "Observation End Time":"filter-input-150",
                "Process Start Time":"filter-input-150",
                "Process End Time":"filter-input-150",
                "Created_At":"filter-input-150",
                "Updated_At":"filter-input-150",
                "Template ID":"filter-input-75",
            }],
            suPermissions: {},
            copySUParams: {},
            //defaultSortColumn: [{id: "Name", desc: false}],
            dialog: {header: 'Confirm', detail: 'Do you want to create a Scheduling Unit Blueprint?'}
        }
        this.idFilterValues = [];
        this.additionalConstColumnOrders = [];
        //minDistance => to appeand the text 'Degrees' along with header
        this.minDistance = ['min_distance_jupiter','min_distance_moon','min_distance_sun','sky_min_calibrator_elevation','sky_min_target_elevation'];
        this.constraintColumns = []; // Constraint column details - which will be removed when switch the su to draft/blueprint
        this.suTypeColumnToBeRemove = []; // Target 1 and related columns should be removed when constraint selected.
        this.access_denied_message = "Don't have permission";
        this.lsKeySortColumn = "SchedulingUnit_"+this.changesutype()+"_SortData";
        this.selectedRows = [];
        this.suDraftsList = []; // List of selected SU Drafts
        this.suBlueprintList = []; // List of selected SU Blueprints
        this.deletableDraftWithBlueprint = []; // List of deletable Scheduling Unit(s)
        this.deletableSUForDialogContent = []; // List of deletable Scheduling Unit Draft/Blueprint to show in dialog 
        this.columnMap = [];
        this.filterQry = '';
        this.orderBy = '';
        this.limit = 10;
        this.offset =  0;
        this.draftIds = []; // Ids used in drop down filter in viewtable
        this.pageUpdated = true;
        this.schUnits = [];
        this.affectedSU = [];
        this.changesutype= this.changesutype.bind(this);
        this.checkAndDeleteSchedulingUnit = this.checkAndDeleteSchedulingUnit.bind(this);
        this.deleteSchedulingUnit = this.deleteSchedulingUnit.bind(this);
        this.getSchedulingDialogContent = this.getSchedulingDialogContent.bind(this);
        this.getSUDeleteStatusContent = this.getSUDeleteStatusContent.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
        this.onRowSelection = this.onRowSelection.bind(this);
        this.updateDraftBlueprints = this.updateDraftBlueprints.bind(this);
        this.addTargetColumns = this.addTargetColumns.bind(this);
        this.fetchTableData = this.fetchTableData.bind(this);
        this.reloadData = this.reloadData.bind(this);
        this.changeScheduleUnitType = this.changeScheduleUnitType.bind(this);
        this.confirmCancelSchedulingUnit = this.confirmCancelSchedulingUnit.bind(this);
        this.confirmCopyingSU = this.confirmCopyingSU.bind(this);
        this.getAdvancedSUCopyContent = this.getAdvancedSUCopyContent.bind(this);
        this.copyAdvancedSchedulingUnit = this.copyAdvancedSchedulingUnit.bind(this);
        this.getSUCopyStatusContent = this.getSUCopyStatusContent.bind(this);
        this.copySchedulingUnit = this.copySchedulingUnit.bind(this);
        this.cancelSchedulingUnit = this.cancelSchedulingUnit.bind(this);
        this.getSUCancelConfirmContent = this.getSUCancelConfirmContent.bind(this);
        this.getSUCancelStatusContent = this.getSUCancelStatusContent.bind(this);
        this.getFilterOptions = this.getFilterOptions.bind(this);
        this.cleanUpSUTask = this.cleanUpSUTask.bind(this);
        this.getCleanUpDialogContent = this.getCleanUpDialogContent.bind(this);
        this.createSUCleanupTask = this.createSUCleanupTask.bind(this);
        this.getStatusContent = this.getStatusContent.bind(this);
        this.checkAndCreateSUB = this.checkAndCreateSUB.bind(this);
        this.setTableReloadFunction = this.setTableReloadFunction.bind(this);
        this.setTableState = this.setTableState.bind(this);
        this.updateAutoDeletion = this.updateAutoDeletion.bind(this);
        this.confirmAutoDeletion = this.confirmAutoDeletion.bind(this);
        this.setAutoDeletion = this.setAutoDeletion.bind(this);
        this.getAutoDeletionDialogContent = this.getAutoDeletionDialogContent.bind(this);
        this.confirmCopyFailedTasks = this.confirmCopyFailedTasks.bind(this);
        this.getCopyFailedTasksDlgContent = this.getCopyFailedTasksDlgContent.bind(this);
        this.copyFailedTasks = this.copyFailedTasks.bind(this);
        this.confirmUpdateSUBlueprint = this.confirmUpdateSUBlueprint.bind(this);
        this.getUpdateSUBDialogContent = this.getUpdateSUBDialogContent.bind(this);
        this.updateSUB = this.updateSUB.bind(this);
        this.confirmCopySpecAndFailedTasks = this.confirmCopySpecAndFailedTasks.bind(this);
        this.getCopySpecDlgContent = this.getCopySpecDlgContent.bind(this);
        this.copySpecAndFailedTasks = this.copySpecAndFailedTasks.bind(this);
        this.scheduleConstraintsArray=this.scheduleConstraintsArray.bind(this);
        this.capitalize = this.capitalize.bind(this);
        this.formatListToNewLineText = this.formatListToNewLineText.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.handleData = this.handleData.bind(this);
        this.refreshSUList = this.refreshSUList.bind(this);
        this.refreshSchedulingUnitList = this.refreshSchedulingUnitList.bind(this);
        this.updateSchedulingUnitConstraint = this.updateSchedulingUnitConstraint.bind(this);
        this.updateSchedulingUnitBlueprint = this.updateSchedulingUnitBlueprint.bind(this);
        this.updateSchedulingUnitDraft = this.updateSchedulingUnitDraft.bind(this);
        this.confirmChangeSUSet = this.confirmChangeSUSet.bind(this);
        this.getSUSetContent = this.getSUSetContent.bind(this);
        this.changeSUSet = this.changeSUSet.bind(this);
    }

    capitalize(s) {
        return s[0].toUpperCase() + s.slice(1);
    }

    //recursive function to get time constraints
    scheduleConstraintsArray(obj, parent, defaultcolumns) {        
        for (let key in obj) {
            let value = obj[key];
            if (typeof value === 'object') {
                if (key === "time") {
                    const timearray = obj[key];
                    if (timearray['at'] === undefined) {
                        Object.assign(derviedscheduleconstraints, {
                            'time_at': ''
                        });
                        defaultcolumns['time_at'] = {name: 'Time at'};
                    }
                    if (timearray['after'] === undefined) {
                        Object.assign(derviedscheduleconstraints, {
                            'time_after': ''
                        });
                        defaultcolumns['time_after'] = {name: 'Time after'};
                    }
                    if (timearray['before'] === undefined) {
                        Object.assign(derviedscheduleconstraints, {
                            'time_before': ''
                        });
                        defaultcolumns['time_before'] = {name: 'Time before'};
                    }
                    for (var timekey in timearray) {
                        //To retireve to and from values on same column between and not_between
                        if (timekey === 'between' || timekey === 'not_between') {
                            if (timearray[timekey].length > 0) {
                                let combinedval = [];
                                for (let i = 0; i < timearray[timekey].length; i++) {
                                    combinedval.push(moment(new Date((timearray[timekey][i]['from']).replace('Z', ''))).format(UIConstants.CALENDAR_DATETIME_FORMAT));
                                    combinedval.push(moment(new Date((timearray[timekey][i]['to']).replace('Z', ''))).format(UIConstants.CALENDAR_DATETIME_FORMAT));
                                    let combinedvalues = "",
                                    combinedvalLength =  combinedval?.length;
                                    combinedval.forEach((item, index) => {
                                        if(index % 2) {
                                            combinedvalues += `${item} ${index !== combinedvalLength - 1 ? ' | ' : ''}`;
                                        } else {
                                            combinedvalues += `${item}${index !== combinedvalLength - 1 ? ', ' : ''}`;
                                        }
                                    })
                                    Object.assign(derviedscheduleconstraints, {
                                        ["time_" + timekey]: combinedvalues
                                    });
                                    defaultcolumns["time_" + timekey] = {name: 'Time '+timekey};
                                }
                            } else {
                                Object.assign(derviedscheduleconstraints, {
                                    ["time_" + timekey]: ''
                                });
                                defaultcolumns["time_" + timekey] = {name: 'Time '+timekey};
                            }
                        } else {
                            Object.assign(derviedscheduleconstraints, {
                                ["time_" + timekey]: moment(timearray[timekey].replace('Z', '')).format('YYYY-MM-DD  HH:mm:ss')
                            });
                            defaultcolumns["time_" + timekey] = {name: 'Time '+timekey};
                        }
                    }
                } else {
                    this.scheduleConstraintsArray(value, key, defaultcolumns);
                }
            } else {
                if (parent !== undefined) {
                    if (/*parent === 'sky' || */parent === 'min_distance' ) {
                        Object.assign(derviedscheduleconstraints, {
                            [parent + '_' + key]: [parseFloat((value * 180) / Math.PI).toFixed(2), ]
                        });
                        defaultcolumns[parent + '_' + key] = {name: ` ${this.capitalize(parent)} ${key} (Degrees)`};
                    } else if (parent === 'transit_offset' && (key === 'from' || key === 'to'))   {
                        value = obj['from'];
                        Object.assign(derviedscheduleconstraints, {
                            [parent + "_from"]: ((value<0?'-':'')+UnitConverter.getSecsToHHmmss(Math.abs(value)))
                        });
                        value = obj['to'];
                        Object.assign(derviedscheduleconstraints, {
                            [parent + "_to"]: ((value<0?'-':'')+UnitConverter.getSecsToHHmmss(Math.abs(value)))
                        });
                        defaultcolumns[parent + '_from'] = {name: ` ${this.capitalize(parent)} from`};
                        defaultcolumns[parent + '_to'] = {name: ` ${this.capitalize(parent)} to`};
                    }   else  if (parent === 'min_elevation' ) {
                        Object.assign(derviedscheduleconstraints, {
                            [parent + '_' + key]: [parseFloat((value * 180) / Math.PI).toFixed(2), ]
                        });
                        defaultcolumns[parent + '_' + key] = {name: ` ${this.capitalize(parent)} ${key} (Degrees)`};
                    }   else    {
                        Object.assign(derviedscheduleconstraints, {
                            [parent + '_' + key]: value
                        });
                        defaultcolumns[parent + '_' + key] = {name: ` ${this.capitalize(parent)} ${key}`};
                    }
                } else {                    
                    Object.assign(derviedscheduleconstraints, {
                        [key]: value
                    });
                    if (key !== '$schema' ) {
                        defaultcolumns[key] = {name: ` ${this.capitalize(key)}`};
                    }
                }
            }
        }
        return defaultcolumns;
    }
    
    /**
     * Get count of tasks grouped by type (observation, pipeline, ingest)
     * @param {Array} tasks - array of task(draft or blueprint) objects
     */
    getTaskTypeGroupCounts(tasks = []) {
        const observation = tasks.filter(task => task.specifications_template.type_value === 'observation');
        const pipeline = tasks.filter(task => task.specifications_template.type_value === 'pipeline');
        const ingest = tasks.filter(task => task.specifications_template.type_value === 'ingest');
        return `${observation.length}/${pipeline.length}/${ingest.length}`;
    }

    changesutype(type = this.state.suType) {
        let queryUrl = type;
        if (type.toLowerCase() === 'blueprint_schedule') {
            return queryUrl = 'blueprint'
        }
        if (type.toLowerCase() === 'Drafts Without BluePrint')  {
            return queryUrl = 'draft'
        }

        
        

        return queryUrl.toLowerCase();
    }

    /**
     * Get all stations of the SUs from the observation task or subtask based on the SU status.
     * @param {Object} schedulingUnit
     */
    getSUStations(schedulingUnit) {
        let stations = [];
        let tasks = schedulingUnit.task_blueprints?schedulingUnit.task_blueprints:schedulingUnit.task_drafts;
        /* Get all observation tasks */
        const observationTasks = _.filter(tasks, (task) => { return task.specifications_template.type_value.toLowerCase() === "observation"});
        for (const observationTask of observationTasks) {
            /** If the status of SU is before scheduled, get all stations from the station_groups from the task specification_docs */
            if ((!schedulingUnit.status || this.STATUS_BEFORE_SCHEDULED.indexOf(schedulingUnit.status.toLowerCase()) >= 0)
                 && (observationTask.specifications_doc.station_configuration?.station_groups)) {
                for (const grpStations of _.map(observationTask.specifications_doc.station_configuration.station_groups, "stations")) {
                    stations = _.concat(stations, grpStations);
                }
            }   
            else if (schedulingUnit.status && this.STATUS_BEFORE_SCHEDULED.indexOf(schedulingUnit.status.toLowerCase()) < 0 
                        && observationTask.subtasks) {
            /** If the status of SU is scheduled or after get the stations from the subtask specification tasks */
                for (const subtask of observationTask.subtasks) {
                    if (subtask.specifications_doc.stations) {
                        stations = _.concat(stations, subtask.specifications_doc.stations.station_list);
                    }
                }
            }
        }
        return _.uniq(stations);
    }

    /**
     * Group the SU stations to main groups Core, Remote, International
     * @param {Object} stationList 
     */
    groupSUStations(stationList) {
        let suStationGroups = {};
        for (const group in this.mainStationGroups) {
            suStationGroups[group] = _.intersection(this.mainStationGroups[group], stationList);
        }
        return suStationGroups;
    }

    getStationGroup(itemSU) {
        const item = {};
        const itemStations = this.getSUStations(itemSU);
        const itemStationGroups = this.groupSUStations(itemStations);
        item.stations = {groups: "", counts: ""};
        item.suName = itemSU.name;
        for (const stationgroup of _.keys(itemStationGroups)) {
            let groups = item.stations.groups;
            let counts = item.stations.counts;
            if (groups) {
                groups = groups.concat("/");
                counts = counts.concat("/");
            }
            // Get station group 1st character and append 'S' to get CS,RS,IS 
            groups = groups.concat(stationgroup.substring(0,1).concat('S'));
            counts = counts.concat(itemStationGroups[stationgroup].length);
            item.stations.groups = groups;
            item.stations.counts = counts;
        }
        return item.stations;
    }

    /**
     * Function to get a component with list of links to a list of ids
     * @param {Array} linkedItems - list of ids
     * @param {String} type - blueprint or draft
     */
    getLinksList = (linkedItems, type) => {
        return (
            <>
                {linkedItems && linkedItems.length>0 && linkedItems.map((item) => (
                    <Link key={item} style={{paddingRight: '3px'}} to={`/schedulingunit/view/${type}/${item}`}>{item}</Link>
                ))}
            </>
        );                    
    }

    getAPIFilter(suFilters, columnDef) {
        const defaultColKeys = Object.keys(columnDef);
        defaultColKeys.forEach(key => {
            let tmpColMap = {};
            let tempKey = key;
            tmpColMap['orgField'] = tempKey;
            tmpColMap['tmpField'] = tempKey;
            if(key === 'suid') {
                tempKey = 'id';
                tmpColMap['orgField'] = tempKey;
                tmpColMap['tmpField'] = 'suid';
            }   else  if(key === 'observation_strategy_template_id') {
                tempKey = 'observation_strategy_template';
                tmpColMap['orgField'] = 'observation_strategy_template';
                tmpColMap['tmpField'] = tempKey;
            }
            if(columnDef[key]) {
                tmpColMap['displayName'] = columnDef[key]['name'];
            }
            this.columnMap.push(tmpColMap);
            //Set Enable/Disable the Filter & SortBy in each column          
            if(suFilters.data.filters[tempKey]) {
                columnDef[key]['disableSortBy'] = !_.includes(suFilters.data.ordering, tempKey);
                columnDef[key]['disableFilters'] = false;
                if(UIConstants.FILTER_MAP[suFilters.data.filters[tempKey].type]) {
                    if (tempKey === 'draft' || tempKey === 'observation_strategy_template') { //this condition because to avoid dropdown filter for 'Linked Draft ID'
                        suFilters.data.filters[tempKey].type = 'CharFilter';
                    }
                    columnDef[key]['filter'] = UIConstants.FILTER_MAP[suFilters.data.filters[tempKey].type];
                }
            }   else if (key === 'draft' && suFilters.data.filters['scheduling_unit_blueprints']) {
                columnDef[key]['disableSortBy'] = !_.includes(suFilters.data.ordering, 'scheduling_unit_blueprints');
                columnDef[key]['disableFilters'] = false;
            }   else if (key === 'duration' && suFilters.data.filters['duration_min']) {
                columnDef[key]['disableSortBy'] = !_.includes(suFilters.data.ordering, 'duration_min');
                columnDef[key]['disableFilters'] = false;
            }   else if (columnDef[key]['name']) {
                columnDef[key]['disableSortBy'] = true;
                columnDef[key]['disableFilters'] = true;
            }
        });
        return columnDef;
    }

    /**
     * Remove column in custom column order
     * @param {array} arrayValue 
     * @param {array} keys 
     */
    removeArrayIndex(arrayValue, keys) {
        keys.forEach(key => {
            let index = arrayValue.indexOf(key) ;
            if (index !== -1) {
                arrayValue.splice(index, 1);
            }
        });
    }
    
    /**
     * Remove column definition
     * @param {column definition array} arrayValue 
     * @param {array} keys 
     */
    removeColumns(arrayValue, keys) {
        keys.forEach(key => {
            delete arrayValue[key];
        });
    }

    /**
     * Get server side filter column details form API
     */


    
    async getFilterColumns(type) {
        const columnDefinitionToBeRemove = ['status', 'unschedulable_reason', 'workflowStatus', 'results_accepted', 'on_sky_start_time', 'on_sky_stop_time', 'process_start_time', 'process_stop_time','scheduled_central_lst'];
        //For Constraint
        const constColDefToBeRemove = ['observation_strategy_template_name', 'duration', 'observation_strategy_template_id', 'observation_strategy_template_description', 'on_sky_start_time', 'on_sky_stop_time', 'process_start_time', 'process_stop_time', 'task_content',
        'target_observation_sap', 'do_cancel', 'created_at', 'updated_at', 'project_rank','rank', 'priority_queue', 'output_pinned', 'draft', 'targetName','targetAngle1','targetAngle2','targetRef'];
        let  suFilters

        if (!this.definitionCache[type]) { // don't send out time after time a request to get the schema :)
            this.definitionCache[type] = "loading";
            suFilters = await ScheduleService.getSchedulingUnitFilterDefinition(type);
            this.definitionCache[type]=suFilters;
        }
        else     {
            if (this.definitionCache[type]=="loading")   {
                return
            }
            suFilters = this.definitionCache[type];
        }

        this.columnMap = [];
        let tmpDefaulColumns = _.cloneDeep(this.state.defaultcolumns[0]);
        let tmpOptionalColumns = _.cloneDeep(this.state.optionalcolumns[0]);
        let tmpColumnOrders = _.cloneDeep(this.state.columnOrders);  
        if (this.state.suType === 'Blueprint') {
            tmpOptionalColumns['draft'].name = 'Linked Draft ID';
            let index = tmpColumnOrders.indexOf('Linked Blueprint ID') ;
            if (index !== -1) {
                tmpColumnOrders[index] = 'Linked Draft ID';
            }
            this.removeColumns(tmpDefaulColumns, this.constraintColumns);// remove constraint columns
            this.removeColumns(tmpOptionalColumns, this.constraintColumns);         
        }   else if (this.state.suType === 'Constraints') {
            tmpOptionalColumns =[];
            this.removeColumns(tmpDefaulColumns, [...constColDefToBeRemove, ...this.suTypeColumnToBeRemove]); //Draft & Blueprint columns to be remove
            tmpDefaulColumns.type = {name: 'Type'};
            tmpDefaulColumns['actionpath']="actionpath";
            tmpColumnOrders =  [...this.state.constraintsColumnOrders, ...this.additionalConstColumnOrders] ;
            tmpColumnOrders.push('Data Accepted') ;
            tmpColumnOrders.push('Unschedulable Reason')
        }   else {
            tmpOptionalColumns['draft'].name = 'Linked Blueprint ID';
            let index = tmpColumnOrders.indexOf('Linked Draft ID') ;
            if (index !== -1) {
                tmpColumnOrders[index] = 'Linked Blueprint ID';
            }
            //Remove columns which is not relavent for Draft
            //this.removeArrayIndex(tmpColumnOrders, columnOrderToBeRemove);
            if(type.startsWith('draft') || type === 'constraints') {
                this.removeColumns(tmpDefaulColumns, [...columnDefinitionToBeRemove, ...this.minDistance]);//Blueprint columns to be remove
                this.removeColumns(tmpOptionalColumns, [...columnDefinitionToBeRemove, ...this.minDistance]);
            }
            this.removeColumns(tmpDefaulColumns, this.constraintColumns); // remove constraint columns
            this.removeColumns(tmpOptionalColumns, this.constraintColumns);
        }
        if (suFilters)  {
            this.getStatusList(suFilters);
            tmpDefaulColumns = this.getAPIFilter(suFilters, tmpDefaulColumns);
            tmpOptionalColumns = this.getAPIFilter(suFilters, tmpOptionalColumns);
             this.setState({tmpDefaulcolumns: [tmpDefaulColumns], tmpOptionalcolumns:[tmpOptionalColumns], tmpColumnOrders: tmpColumnOrders, columnMap: this.columnMap})
        }
    }

    /**
     * Get Status list frol filter
     * @param {Array} suFilters 
     */
    getStatusList(suFilters) {  
        if (suFilters.data.filters['status']) {
            this.statusList = [];
            suFilters.data.filters['status'].choices.forEach(choice => {
                const tmpVar = {name: _.startCase(choice.value), value: choice.value};
                this.statusList.push(tmpVar);
            });
        }
    }
    
    /**
     * Function to move SUD/SUB to different Scheduling Set
     * @returns 
     */
     async changeSUSet() {
        if(this.state.selectedSUSet) {
            let response = null;
            this.statusUpdate = null;
            let responseStatus = 0
            for (const selectedRow of this.selectedRows) {
                let schUnit = _.find(this.schUnits, ['id', selectedRow.id])
                if(this.state.suType === "Blueprint") {
                    schUnit = schUnit.draft;
                }
                else {
                    schUnit.scheduling_unit_blueprints = _.map(schUnit.scheduling_unit_blueprints, 'url') 
                }
                schUnit['task_drafts'] = _.map(schUnit.task_drafts, 'url');
                schUnit.specifications_template=schUnit.observation_strategy_template.scheduling_unit_template;
                delete schUnit.observation_strategy_template;
                schUnit.scheduling_set = this.state.selectedSUSet.url;
                try { 
                    response = await ScheduleService.updateSchedulingUnitDraft(schUnit);
                    if(response.status === 200) {
                        responseStatus = responseStatus + 1;
                        if(responseStatus === this.selectedRows.length) {
                            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Successfully moved to another Scheduling Set!' });
                            this.closeDialog();
                        }
                    }
                    else {
                        appGrowl.show({ severity: 'error', summary: 'Failed', detail: 'Unable to change Scheduling Set!'});
                    }
                } catch (error) {
                    this.statusUpdate = "Failed";
                }

            }
        }
        this.affectedSU = [];
        this.getSchedulingUnitList(false,this.changesutype(),this.filterQry, this.orderBy, this.limit, this.offset)
    }
    
    /**
     * Show confirmation dialog before moving SUD/SUB to different SU Set
     */
     async confirmChangeSUSet() {
        this.affectedSU = [];
        if (this.selectedRows.length > 0) {
            for (const su of this.selectedRows) {
                const schUnit = _.find(this.schUnits, ['id', su.id]);
                if(this.state.suType.toLowerCase() === "blueprint") {
                    this.affectedSU.push(schUnit.draft)
                }
                else {
                    
                    _.map(schUnit.scheduling_unit_blueprints, blueprint => {
                        this.affectedSU.push(blueprint)
                    })
                }
            }
            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header = "Move to different Scheduling Set";
            dialog.detail = "Select the Scheduling Set to which you want to move the Scheduling Unit(s)";
            dialog.content = this.getSUSetContent;
            dialog.actions = [{
                    id: 'yes',
                    title: 'Move',
                    callback: this.changeSUSet,
                    className: (this.props.project) ? "dialog-btn" : ""
                },
                {
                    id: 'no',
                    title: 'Cancel',
                    className:'act-btn-cancel',
                    callback: this.closeDialog
                }
            ];
            dialog.onSubmit = this.changeSUSet;
            dialog.width = '55vw';
            this.setState({
                copiedSchedulingUnit: this.state.scheduleunit, projectSchedSets: this.projectSchedSet,
                dialog: dialog,
                dialogVisible: true
            }); 
        }
        else {
            appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Select minimum one Scheduling Unit Draft/Blueprint to move to different Scheduling Set.'});
        }
    }

    /**
     * Content with confirmation message and affected SUB/SUD in confirmation dialog
     * @returns Component
     */
     getSUSetContent(){
        return  <> 
                    <div style={{marginTop: '1em'}}>
                        <DataTable value={this.projectSchedSets} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}
                        selectionMode="single" selection={this.state.selectedSUSet} onSelectionChange={e => this.setState({selectedSUSet: e.value})} sortField= "name">
                            <Column field="id" header="Id"></Column>
                            <Column field="name" header="Name"></Column>
                            <Column field="description" header="Description"></Column>
                        </DataTable>
                    </div>
                    <span></span>
                    {this.affectedSU.length > 0 && 
                    <div>
                        <div style={{marginTop: '1em'}}>
                        <span>Scheduling unit {this.state.suType.toLowerCase().startsWith("draft")?"blueprint":"draft"}(s) shown below will be affected. Do you want to continue? </span>
                        <DataTable value={this.affectedSU} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                            <Column field="id" header="Id"></Column>
                            <Column field="name" header="Name"></Column>
                        </DataTable>
                    </div>
                    </div>
                }
                </>
     }

    /**
     * 
     * @param {Boolean} isInitial - true => While loading page, false => while doing server side access
     * @param {String} suType - SU type ( blueprint/draft/blueprint constraints)
     * @param {String} filterQry - Table filter params
     * @param {String} orderBy - Table order by param
     * @param {Number} limit - Number of the rows to fetch
     * @param {Number} offset - fetch row from nth position
     */
    async getSchedulingUnitList (isInitial, suType, filterQry, orderBy, limit, offset) {
        //set query param for project if url has project param
        if (this.props.project) {
            filterQry += (filterQry === ''?'project=':'&project=')+this.props.project;
        }
        let optionalColumns = this.state.optionalcolumns[0];
        let columnclassname = this.state.columnclassname[0];
        let defaultcolumns = this.state.defaultcolumns[0];
        await this.getFilterColumns(this.changesutype());
        const output = [];
        let suPermissions = {}
        if (isInitial) {
            this.mainStationGroups = null;
            const promises = [
                ScheduleService.getMainGroupStations(),
            ];
            await Promise.all(promises).then(async responses => {
                if(responses) { 
                    this.mainStationGroups = responses[0];
                }
            });
            this.setState({scheduleunitDrafts: []});
        }   else {
            let scheduleunits = [];
            let expand,fields;
            let params=suType.toLowerCase();
             if(suType.toLowerCase().startsWith('draft')){
                expand=this.SU_DRAFT_EXPAND;
                fields=this.SU_DRAFT_FIELDS;      
                        
            }else if(suType.toLowerCase() === 'blueprint'){
                expand=this.SU_BLUEPRINT_EXPAND;
                fields=this.SU_BLUEPRINT_FIELDS;   
            }else{
                expand=this.SU_BPCONSTRAINTS_EXPAND;
                fields=this.SU_BPCONSTRAINTS_FIELDS;
                params='blueprint';
            } 
            
            if (suType=="drafts without blueprint") {
                filterQry = filterQry + "&blueprinted=false" 
            }
            let response = await ScheduleService.getSchedulingUnitsExpandWithFilter(params.toLowerCase(), expand, filterQry, orderBy, limit, offset, fields.join(","));    
            if (response && response.data) {
                this.totalPage = response.data.count;
                scheduleunits = response.data.results;
            }            
            this.draftIds = [];
            this.schUnits =  _.cloneDeep(scheduleunits)
            if(this.props.project) {
                this.projectSchedSets = await ScheduleService.getSchedulingSets(`project=${this.props.project}`);
            }
            let projectSUs = _.uniqBy(_.map(scheduleunits, function(scheduleunit) {
                                        return {
                                            project: suType.toLowerCase().startsWith( 'draft')? scheduleunit.scheduling_set.project.name: scheduleunit.draft.scheduling_set.project.name,
                                            id: scheduleunit.id
                                        }
                                    }),'project');
            for (let projectSU of projectSUs) {
                let suWithProjectPermission = await AuthUtil.getUserPermissionByModuleId(suType.toLowerCase() === 'draft'? 'scheduling_unit_draft' : 'scheduling_unit_blueprint', projectSU.id);
                projectSU.createSub = suWithProjectPermission[projectSU.id].createSub;
                projectSU.createRerun = suWithProjectPermission[projectSU.id].createRerun;
                projectSU.updateSub = suWithProjectPermission[projectSU.id].updateSub;
                projectSU.cancelSU = suWithProjectPermission[projectSU.id].cancelSU;
                projectSU.autoDeletion = suWithProjectPermission[projectSU.id].autoDeletion;
                projectSU.copySU = suWithProjectPermission[projectSU.id].copySU;
                projectSU.delete = suWithProjectPermission[projectSU.id].delete;
                projectSU.cleanupTask = suWithProjectPermission[projectSU.id].cleanupTask;
                projectSU.copyFailedTasks = suWithProjectPermission[projectSU.id].copyFailedTasks;
            }
            suPermissions = {
                createSub: _.filter(projectSUs, {createSub: true}).length === projectSUs.length,
                createRerun: _.filter(projectSUs, {createRerun: true}).length === projectSUs.length,
                updateSub: _.filter(projectSUs, {updateSub: true}).length === projectSUs.length,
                cancelSU: _.filter(projectSUs, {cancelSU: true}).length === projectSUs.length,
                autoDeletion: _.filter(projectSUs, {autoDeletion: true}).length === projectSUs.length,  
                copySU: _.filter(projectSUs, {copySU: true}).length === projectSUs.length,
                delete: _.filter(projectSUs, {delete: true}).length === projectSUs.length, 
                cleanupTask: _.filter(projectSUs, {cleanupTask: true}).length === projectSUs.length,
                copyFailedTasks: _.filter(projectSUs, {copyFailedTasks: true}).length === projectSUs.length,
            }
         
            if ( suType.toLowerCase().startsWith('draft')) {
                for( const scheduleunit  of scheduleunits){
                    let updatedSU = {};
                    if (!this.props.project || (this.props.project && this.props.project === scheduleunit.scheduling_set.project.name)) {
                        updatedSU = await this.updateSchedulingUnitDraft(scheduleunit);//.then((scheduleUnit) => {
                        updatedSU = await this.addTargetColumns(updatedSU);
                        //});
                        output.push(updatedSU)                  
                    }
                }  
            }  else if ( suType.toLowerCase() === 'blueprint') {
                const suIds = _.map(scheduleunits, 'id');
                let workflows = await this.timelineCommonUtils.getWorkflowsAndTasks(suIds);
                for( const scheduleunit  of scheduleunits){
                    let updatedSU = {};
                    this.draftIds.push(scheduleunit.draft_id);
                    updatedSU = await this.updateSchedulingUnitBlueprint(workflows, scheduleunit);
                    updatedSU = await this.addTargetColumns(updatedSU)
                    output.push(updatedSU);
                } 
            } else {
                const suIds = _.map(scheduleunits, 'id');
                let workflows = await this.timelineCommonUtils.getWorkflowsAndTasks(suIds);
                let constraintColumns = [];
                for( const scheduleunit  of scheduleunits){
                    let updatedSU = {};
                    this.draftIds.push(scheduleunit.draft_id);
                    updatedSU = await this.updateSchedulingUnitConstraint(constraintColumns, workflows, scheduleunit);
                    updatedSU = await this.addTargetColumns(updatedSU)
                    output.push(updatedSU);
                }
                // Sort the constraint column keys and add to default columns
                this.additionalConstColumnOrders = [];
                for (const constraintColName of _.sortBy(_.keys(constraintColumns))) {
                    defaultcolumns[constraintColName] = constraintColumns[constraintColName];
                    this.additionalConstColumnOrders.push(constraintColumns[constraintColName].name);
                }
                await this.getFilterColumns(this.changesutype());
            }
            this.projects = await ProjectService.getProjectList(false, 'name,url');
             this.setState({
                scheduleunit: output, isLoading: false, optionalColumns: [optionalColumns], columnclassname: [columnclassname], 
                showSpinner: false, suPermissions: suPermissions, loadingStatus: false
            });
        }
         this.setState({
            scheduleunit: output, isLoading: false, optionalColumns: [optionalColumns], columnclassname: [columnclassname], 
            showSpinner: false, suPermissions: suPermissions
        });     
        this.selectedRows = [];
    }

    /**
     * Prepare new line string in table column
     * @param {Array} arrayVal 
     * @returns 
     */
    formatListToNewLineText(arrayVal){
        return (
            <>
                {arrayVal.length>0 && arrayVal.map((item) => (
                    <div key={item}>{item}</div>
                ))}
            </>
        );                    
    }

    async componentDidMount(){ 
        const permission = await AuthUtil.getUserRolePermission();
        this.priorityQueueTypes = await  UtilService.getPriorityQueueType();
        this.setState({userrole: permission});
        this.pageUpdated = true;
        this.getFilterColumns(this.changesutype());
        this.getSchedulingUnitList(true, this.changesutype(), this.filterQry, this.orderBy, this.limit, this.offset); 
        this.setToggleBySorting();
        this.timelineCommonUtils = new TimelineCommonUtils();
    }

    /**
     * Function to get the UI attributes from local storage that are stored in the previous view
     */
    getUIAttr() {
        this.suUIAttr = UtilService.localStore({ type: 'get', 
                            key: this.props.project?'PROJECT_VIEW_UI_ATTR':'SCHEDULE_LIST_UI_ATTR' }) || {};
    }

    /**
     * Function that stores the preferred values in local storage to restore the same in the next view
     * @param {string} key 
     * @param {Object} value 
     */
    storeUIAttr(key, value) {
        this.suUIAttr[key] = value;
        UtilService.localStore({ type: 'set', 
                                    key: this.props.project?'PROJECT_VIEW_UI_ATTR':'SCHEDULE_LIST_UI_ATTR', 
                                    value: this.suUIAttr }); 
    }
    
    setToggleBySorting() {
        let sortData = null;
        if (this.lsKeySortColumn) {
            sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        }
        if(sortData && !_.isEmpty(sortData)){
            if(Object.prototype.toString.call(sortData) === '[object Array]'){
                this.defaultSortColumn = sortData;
            }
            else{
                this.defaultSortColumn = [{...sortData}];
            }
        }
        else {
            this.defaultSortColumn = [{id: "Name", desc: false}];
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        if (this.lsKeySortColumn) {
            UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] });
        }
    }

    toggleBySorting=(sortData) =>{
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: sortData });
    }

    /**
     * Callback function passed to ViewTable component to pass back the selected rows.
     * @param {Array} selectedRows - Subset of data passed to the ViewTable component based on selection.
     */
    onRowSelection(selectedRows) {
        this.selectedRows = selectedRows;      
    }

    /**
     * Funtion to update the draft with newly created blueprint data. This function can be called from the implementing component.
     */
    reloadData() {        
        this.pageUpdated = true;
        this.setState({isLoading: true});
        this.getSchedulingUnitList(true, this.changesutype(), this.filterQry, this.orderBy, this.limit, this.offset);
        this.setToggleBySorting();
    }


    /**
     * Funtion to update the draft with newly created blueprint data. This function can be called from the implementing component.
     */
    updateDraftBlueprints(newBlueprints) {
        let suList = this.state.scheduleunit;
        _.map(newBlueprints, (newBlueprint => {
            let schedulingUnit = _.find(suList, ['id', newBlueprint.draft_id]);
            schedulingUnit.draft = this.getLinksList([newBlueprint.id], 'blueprint');
            let selectedRow = _.find(this.selectedRows, ['id', newBlueprint.draft_id]);
            selectedRow.scheduling_unit_blueprints.push(newBlueprint.url);
            selectedRow.scheduling_unit_blueprints_ids.push(newBlueprint.id);
        }));
        this.setState({scheduleunit: suList});
    }

    /**
     * Check and delete the selected Scheduling Unit(s)
     */
    checkAndDeleteSchedulingUnit(e) {
        e.preventDefault();
        this.pageUpdated = false;
        this.suDraftsList = [];
        this.suBlueprintList = [];
        this.deletableDraftWithBlueprint = [];
        this.deletableSUForDialogContent = [];
        let tmpTotalSUBList = [];
        let hasInvalidSUD = false;
        if(this.selectedRows.length === 0) {
            appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Select Scheduling Unit Draft/Blueprint to delete.'});
        }   else {
            //Filter SUB
            this.suBlueprintList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.type.toLowerCase() === "blueprint" });
            //Filter SUD
            if (this.suBlueprintList && this.suBlueprintList.length > 0) {
                this.suDraftsList = _.difference(this.selectedRows, this.suBlueprintList);
            }   else {
                this.suDraftsList = this.selectedRows;
            }
            //Find Deletable SU Drafts
            if (this.suDraftsList && this.suDraftsList.length > 0) {
                this.suDraftsList.map(sud => {
                    if (sud.scheduling_unit_blueprints_ids && sud.scheduling_unit_blueprints_ids.length === 0) {
                        this.deletableDraftWithBlueprint.push(sud);
                        this.deletableSUForDialogContent.push(sud);
                    }   else if (this.suBlueprintList && this.suBlueprintList.length > 0) {
                        let tmpSUBList = _.filter(this.suBlueprintList, (sub => { return sub.draft_id === sud.id}));
                        tmpTotalSUBList = (tmpSUBList && tmpSUBList.length > 0)?[...tmpTotalSUBList, ...tmpSUBList]: tmpTotalSUBList;
                        if (sud.scheduling_unit_blueprints_ids && tmpSUBList && tmpSUBList.length === sud.scheduling_unit_blueprints_ids.length) {
                            this.deletableDraftWithBlueprint.push(sud);
                            this.deletableSUForDialogContent.push(sud);
                            this.deletableSUForDialogContent = [...this.deletableSUForDialogContent, ...tmpSUBList];
                        }   else {
                            hasInvalidSUD = true;
                            this.deletableSUForDialogContent = [...this.deletableSUForDialogContent, ...tmpSUBList];
                        }
                    }   else {
                        hasInvalidSUD = true;
                    }
                    return sud;
                });
            }
           // Find SUB which is not blongs to the selected SUD
            if (this.suBlueprintList && this.suBlueprintList.length !== tmpTotalSUBList.length) {
                this.deletableDraftWithBlueprint = [...this.deletableDraftWithBlueprint, ..._.difference(this.suBlueprintList, tmpTotalSUBList)];
                this.deletableSUForDialogContent = [...this.deletableSUForDialogContent, ..._.difference(this.suBlueprintList, tmpTotalSUBList)];
            }           
            if (this.deletableDraftWithBlueprint.length === 0 && this.deletableSUForDialogContent.length === 0) {
                appGrowl.show({severity: 'info', summary: 'Blueprint Exists', detail: "Blueprint(s) exist(s) for the selected Scheduling Unit Draft(s) and can not be deleted."});
            }   else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header= "Confirm to Delete Scheduling Unit(s)";
                if (hasInvalidSUD) {
                    dialog.detail = "One or more selected Scheduling Unit Draft(s) having Blueprint(s) cannot be deleted. Do you want to ignore them and delete others?";
                }   else {
                    dialog.detail = "Do you want to delete the selected Scheduling Unit Draft/Blueprint?";
                }
                dialog.content = this.getSchedulingDialogContent;
                dialog.actions = [{id: 'yes', title: hasInvalidSUD? 'Delete others':'Delete', callback: this.deleteSchedulingUnit, className:(this.props.project)?"dialog-btn": "", style:{"background": "#e91224","border": "1px solid #e91224"}},
                {id: 'no', title: 'Cancel', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
                dialog.onSubmit = this.deleteSchedulingUnit;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({dialog: dialog, dialogVisible: true});
            }
        }
    }
    
    /**
     * Prepare Scheduling Unit(s) details to show on confirmation dialog
     */
    getSchedulingDialogContent() {
        let selectedSchedulingUnits = [];
        let unselectedSchedulingUnits = [];
        for(const su of this.deletableSUForDialogContent) {
            selectedSchedulingUnits.push({suId: su.id, suName: su.name, 
                suType: su.type,
                sudbid: su.type.toLowerCase() === 'draft'? su.scheduling_unit_blueprints_ids.join(', '): su.draft_id});
        }
        let unselectedSUList = _.difference(this.selectedRows, this.deletableSUForDialogContent);
        for(const su of unselectedSUList) {
            unselectedSchedulingUnits.push({suId: su.id, suName: su.name, 
                suType: su.type,
                sudbid: su.type.toLowerCase() === 'draft'? su.scheduling_unit_blueprints_ids.join(', '): su.draft_id});
        }
        return  <> 
                     {selectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) that can be deleted</b>
                            <DataTable value={selectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suType" header="Type"></Column>
                                <Column field="sudbid" header="Draft/Blueprint ID(s)"></Column>
                            </DataTable>
                        </div>
                    }
                    {unselectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) that will be ignored</b>
                            <DataTable value={unselectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suType" header="Type"></Column>
                                <Column field="sudbid" header="Draft/Blueprint ID(s)"></Column>
                            </DataTable>
                        </div>
                    }
                    
                </>
    }

    /**
     * Prepare Scheduling Unit(s) details to show status of deletion
     */
    getSUDeleteStatusContent() {
        let schedulingUnits = this.deletableDraftWithBlueprint;
        return <>
            <DataTable value={schedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suid" header="Scheduling Unit Id"></Column>
                <Column field="name" header="Scheduling Unit Name"></Column>
                <Column field="type" header="Type"></Column>
                <Column field="deleteStatus" header="Status"></Column>
            </DataTable>
        </>
    }
    
    /**
     * Delete selected Scheduling Unit(s)
     */
    async deleteSchedulingUnit() {
        this.suDraftsWithBlueprintList = [];
        let stateSUs = this.state.scheduleunit;
        for(const schedulingUnit of this.deletableDraftWithBlueprint) {
            const deleteStatus = await ScheduleService.deleteSchedulingUnit(schedulingUnit.type, schedulingUnit.id);
            schedulingUnit.deleteStatus = deleteStatus?"Success":"Failed";
            if (deleteStatus) {
                _.remove(stateSUs, (stateSU) => { return stateSU.id === schedulingUnit.id});
            }
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Delete Scheduling Unit(s) Status";
        dialog.actions = [{ id: 'no', title: 'Close', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel" }];
        dialog.detail = ""
        dialog.content = this.getSUDeleteStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ dialog: dialog, dialogVisible: true, scheduleunit: stateSUs });
        if (_.filter(this.deletableDraftWithBlueprint, ["deleteStatus", "Success"]).length > 0) {
            this.pageUpdated = true;   
            this.getSchedulingUnitList(false, this.changesutype(), this.filterQry, this.orderBy, this.limit, this.offset);
        }
    }

    /**
     * Prepare Scheduling Unit(s) details to show on confirmation dialog before cancelling
     */
    getSUCancelConfirmContent() {
        let selectedSUs = [], ignoredSUs = [];
        for (const obj of this.selectedSURows) {
            if (obj.type  === "Blueprint" && this.SU_END_STATUSES.indexOf(obj.status) < 0) {
                selectedSUs.push({
                    suId: obj.id, suName: obj.name,
                    suType: obj.type, status: obj.status
                });
            }   else {
                ignoredSUs.push({
                    suId: obj.id, suName: obj.name,
                    suType: obj.type, status: obj.status
                });
            }
        }
        return <>
            <div style={{marginTop: '1em'}}>
                <b>Scheduling Unit(s) that can be cancelled</b>
                <DataTable value={selectedSUs} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>
                    <Column field="suType" header="Type"></Column>
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>
            {ignoredSUs.length > 0 &&
            <div style={{marginTop: '1em'}}>
                <b>Scheduling Unit(s) that will be ignored</b>
                <DataTable value={ignoredSUs} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>
                    <Column field="suType" header="Type"></Column>
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>
            }
        </>
    }

    /**
     * Prepare Scheduling Unit(s) details to show status of cancellationn
     */
    getSUCancelStatusContent() {
        let cancelledSchedulingUnits = this.state.cancelledSchedulingUnits;
        return <>
            <DataTable value={cancelledSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="suName" header="Scheduling Unit Name"></Column>
                <Column field="type" header="Type"></Column>
                <Column field="status" header="Status"></Column>
            </DataTable>
        </>
    }

    async changeProject(projectName) {
        const projectSchedSets = await ScheduleService.getSchedulingSets(`project=${encodeURIComponent(projectName)}`);
        let copySUParams = this.state.copySUParams;
        copySUParams.project = projectName;
        copySUParams.scheduling_set_id = null;
        this.projectSchedSets = projectSchedSets;
        _.map(this.projectSchedSets, suSet => suSet.nameId = `${suSet.name} (${suSet.id})`);
        this.setState({copySUParams: copySUParams, projectSchedSets: projectSchedSets});
    }

    setSchedUnitParams(key, value) {
        let copySUParams = this.state.copySUParams;
        copySUParams[key] = value;
        this.setState({copySUParams: copySUParams});
    }

    getSUCopyStatusContent() {
        let selectedSUs = [];
        this.selectedRows.forEach(function(obj) {
                selectedSUs.push({
                    suId: obj.id, suName: obj.name,
                    suType: obj.type
                });
         })
        return <>
            <div style={{marginTop: '1em'}}>
                <b>Scheduling Unit(s) that will be copied</b>
                <DataTable value={selectedSUs} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>                                  
                </DataTable>
            </div>
        </>
    }

    /**
     * Provides form with fields to update while or after copying the Scheduling Unit draft
     * @returns Component
     */
    getAdvancedSUCopyContent() {
        let selectedSUs = [];
        this.selectedRows.forEach(function(obj) {
                selectedSUs.push({
                    suId: obj.id, suName: obj.name,
                    suType: obj.type
                });
         })
        return <>
            <div style={{marginTop: '1em'}}>
                {this.selectedRows.length === 1 &&
                <div className="p-field p-grid">
                    <label htmlFor="schedUnitName" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{color:'red'}}>*</span></label>
                    <div className="col-lg-4 col-md-4 col-sm-12" data-testid="namediv">
                        <InputText id="name" data-testid="name" style={{width:"100%"}}
                                    tooltip="Enter name of the Scheduling Unit" maxLength="128"
                                    ref={input => {this.nameInput = input;}}
                                    value={this.state.copySUParams.name}
                                    onChange={(e) => this.setSchedUnitParams('name', e.target.value)}
                                    onBlur={(e) => this.setSchedUnitParams('name', e.target.value)}
                                    />
                    </div>
                    <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{color:'red'}}>*</span></label>
                    <div className="col-lg-4 col-md-4 col-sm-3">
                        <InputTextarea rows={3} cols={30} style={{width:"100%"}}
                                    tooltip="Longer description of the scheduling unit" maxLength="128"
                                    data-testid="description" value={this.state.copySUParams.description} 
                                    onChange={(e) => this.setSchedUnitParams('description', e.target.value)}
                                    onBlur={(e) => this.setSchedUnitParams('description', e.target.value)}
                                    />
                    </div>
                    
                </div>}
                <div className="p-field p-grid">
                    <label htmlFor="project" className="col-lg-2 col-md-2 col-sm-12">Project </label>
                    <div className="col-lg-4 col-md-4 col-sm-12" data-testid="project" >
                        <Dropdown inputId="project" optionLabel="name" optionValue="name" 
                                tooltip="Project" style={{width: "100%"}}
                                value={this.state.copySUParams.project}
                                options={this.projects} 
                                placeholder="Select Project"
                                onChange={(e) => {this.changeProject(e.value)}} 
                                />
                    </div>
                    <label htmlFor="schedSet" className="col-lg-2 col-md-2 col-sm-12">Scheduling Set <span style={{color:'red'}}>*</span></label>
                    <div className="col-lg-4 col-md-4 col-sm-12">
                        <Dropdown data-testid="schedSet" id="schedSet" optionLabel="nameId" optionValue="id" 
                                tooltip="Scheduling set of the project" style={{width: "100%"}}
                                value={this.state.copySUParams.scheduling_set_id} 
                                options={this.projectSchedSets}
                                placeholder="Select Scheduling Set" 
                                onChange={(e) => {this.setSchedUnitParams('scheduling_set_id',e.value)}} 
                                />
                    </div>
                </div>
                <div className="p-field p-grid">
                    <label htmlFor="priority_queue_value" className="col-lg-2 col-md-2 col-sm-12">Priority Queue</label>
                    <div className="col-lg-4 col-md-4 col-sm-10">
                        <Dropdown data-testid="priority_queue_value" id="priority_queue_value" optionLabel="value" optionValue="url" 
                            tooltip="Priority queue of this scheduling unit. Queues provide a strict ordering between scheduling units."
                            value={this.state.copySUParams.priority_queue} 
                            options={this.priorityQueueTypes} 
                            onChange={(e) => {this.setSchedUnitParams('priority_queue',e.value)}} 
                            placeholder="Select Priority Queue" style={{width:"100%"}}
                            />     
                    </div>
                </div>
                {/* <div style={{marginLeft: "10px", marginTop: "20px"}}><b>Scheduling Unit(s) that will be copied</b></div>
                <DataTable value={selectedSUs} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>                                  
                </DataTable> */}
            </div>
        </>
    }

    //Confirmation to Copy Scheduling Unit
    confirmCopyingSU(e) {
        e.preventDefault();
        if (this.selectedRows.length > 0) {
            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header = "Confirm to Copy Scheduling Unit(s)";
            if (this.state.suType === 'Draft') {
                dialog.detail = "Do you want to copy the selected Scheduling Unit " + this.state.suType + "?";
            } else {
                dialog.detail = "Copying Scheduling Unit Blueprint(s) will create new Scheduling Unit Draft(s). Do you want to proceed?"
            }
            dialog.content = this.getSUCopyStatusContent;
            dialog.actions = [{
                    id: 'yes',
                    title: 'Copy',
                    callback: this.copySchedulingUnit
                },
                {
                    id: 'no',
                    title: 'Cancel', 
                    className:'act-btn-cancel',
                    callback: this.closeDialog
                }
            ];
            dialog.onSubmit = this.copySchedulingUnit;
            dialog.width = '35vw';
            dialog.showIcon = false;
            this.setState({
                dialog: dialog,
                dialogVisible: true
            });
        } else {
            appGrowl.show({
                severity: 'info',
                summary: 'Select Draft/Blueprint',
                detail: 'Select at least one Scheduling Unit Draft/Blueprint to copy.'
            });
        }
    }
    
    //Function to Copy SU Draft and Blueprint 
    async copySchedulingUnit() {
        this.successSchedulingUnits = [];
        this.failedSchedulingUnits = [];
        for (const item of this.selectedRows) {
            try {
                let response = null;
                let param = {};
                if (this.state.suType === 'Draft') {
                    response = await ScheduleService.copySchedulingUnitDraft(item.id, param);
                } else {
                    response = await ScheduleService.copySchedulingUnitBluePrint(item.id, param);
                }             
                if (response.status === 201) {
                    this.successSchedulingUnits.push({
                        suId: item.id,
                        suName: item.name,
                        status: 'Success'
                    });
                } else {
                    this.failedSchedulingUnits.push({
                        suId: item.id,
                        suName: item.name,
                        status: 'Failed'                        
                    });
                }
            } catch (error) {
                this.failedSchedulingUnits.push({
                    suId: item.id,
                    suName: item.name,
                    status: 'Failed'
                });
            }
        //    this.reloadData()
        }
        this.tableReloadFunction(this.tableState);
        let dialog = this.state.dialog;
        dialog.type = "success";
        dialog.header = "Copy Scheduling Unit Status";
        dialog.detail = "";
        dialog.content = this.getStatusContent;
        dialog.actions = [{
            id: 'no',
            title: 'Close',
            callback: this.closeDialog,
            className: (this.props.project) ? "dialog-btn" : ""
        }];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({
            dialog: dialog,
            dialogVisible: true
        });
    }
    
    /**
     * Function to show the advanced copy dialog with fields to specify
     */
    async showAdvancedCopyDialog(e) {
        e.preventDefault();
        if (this.selectedRows.length > 0) {
            let copySUParams = {};
            if (this.selectedRows.length === 1) { // Set the default values from the selected SU if only one row is selected
                copySUParams.name = this.selectedRows[0].name;
                copySUParams.description = this.selectedRows[0].description;
                copySUParams.project = this.selectedRows[0].project;
                copySUParams.priority_queue = (_.find(this.priorityQueueTypes, {'value': this.selectedRows[0].priority_queue})).url;
                await this.changeProject(this.selectedRows[0].project);
                copySUParams.scheduling_set_id = (_.find(this.state.projectSchedSets, {'name': this.selectedRows[0].scheduling_set})).id;
            }   else { // If more than one row selected, set the values common in all selected SUs
                let selectedUniqProjectSUs = _.uniqBy(this.selectedRows, 'project');
                let selectedProject = null, selectedSUSet = null;
                if (selectedUniqProjectSUs.length === 1) {
                    selectedProject = selectedUniqProjectSUs[0].project;
                    let selectedUniqSetSUs = _.uniqBy(_.filter(this.selectedRows, {'project': selectedProject}), 'scheduling_set');
                    selectedSUSet = selectedUniqSetSUs.length === 1?selectedUniqSetSUs[0].scheduling_set:null;
                }
                if (selectedProject) {
                    await this.changeProject(selectedProject);
                    copySUParams = this.state.copySUParams;
                    if (selectedSUSet) {
                        copySUParams.scheduling_set_id = (_.find(this.state.projectSchedSets, {'name': selectedSUSet})).id;
                    }
                }
                let selectedUniqPrioritySUs = _.uniqBy(this.selectedRows, 'priority_queue');
                if (selectedUniqPrioritySUs.length === 1) {
                    copySUParams.priority_queue = (_.find(this.priorityQueueTypes, {'value': selectedUniqPrioritySUs[0].priority_queue})).url;
                }
            }
            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header = "Advanced Copy Scheduling Unit";
            if (this.state.suType === 'Draft') {
                dialog.detail = "Select the Project and Scheduling set to which you want to copy this Scheduling Unit draft(s)";
            } else {
                dialog.detail = "Copying Scheduling Unit Blueprint(s) will create new Scheduling Unit Draft(s). Select the Project and Scheduling set to which you want to copy this Scheduling Unit blueprint(s)"
            }
            dialog.content = this.getAdvancedSUCopyContent;
            dialog.actions = [{
                    id: 'yes',
                    title: 'Copy',
                    callback: this.copyAdvancedSchedulingUnit
                },
                {
                    id: 'no',
                    title: 'Cancel',
                    className: 'act-btn-cancel',
                    callback: this.closeDialog
                }
            ];
            dialog.onSubmit = this.copyAdvancedSchedulingUnit;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({
                dialog: dialog,
                dialogVisible: true,
                copySUParams: copySUParams
            });
        } else {
            appGrowl.show({
                severity: 'info',
                summary: 'Select Draft/Blueprint',
                detail: 'Select at least one Scheduling Unit Draft/Blueprint to copy.'
            });
        }
    }
    
    /**
     * To copy Scheduling Unit(s) to another Project/Scheduling Set
     * @returns 
     */
    async copyAdvancedSchedulingUnit() {
        this.successSchedulingUnits = [];
        this.failedSchedulingUnits = [];
        if (!this.state.copySUParams.scheduling_set_id || (this.selectedRows.length === 1 && 
                (this.state.copySUParams.name === "" || this.state.copySUParams.description === ""))) {
            appGrowl.show({ severity: 'info', summary: 'Missing Information', detail: 'Please enter required fields' });
            return;
        }   else {
            this.setState({showSpinner: true, spinnerPercent: 0});
        }
        let param = { scheduling_set_id: this.state.copySUParams.scheduling_set_id, 
                      scheduling_set: (_.find(this.projectSchedSets, {id: this.state.copySUParams.scheduling_set_id})).url };
        const totalToCopy = this.selectedRows.length;
        let copied = 0;
        for (const item of this.selectedRows) {
            try {
                let response = null;
                if (this.state.suType === 'Draft') {
                    response = await ScheduleService.copySchedulingUnitDraft(item.id, param);
                } else {
                    response = await ScheduleService.copySchedulingUnitBluePrint(item.id, param);
                }
                let copiedSUD = response.data;
                copiedSUD.task_drafts = _.map(copiedSUD.task_drafts, 'url');
                copiedSUD.scheduling_set_id = param.scheduling_set_id;
                copiedSUD.scheduling_set = param.scheduling_set;
                copiedSUD.name = this.state.copySUParams.name || copiedSUD.name;
                copiedSUD.description = this.state.copySUParams.description || copiedSUD.description;
                copiedSUD.priority_queue = this.state.copySUParams.priority_queue || copiedSUD.priority_queue;
                await ScheduleService.updateSchedulingUnit('draft', copiedSUD);
                if (response.status === 201) {
                    this.successSchedulingUnits.push({
                        suId: item.id,
                        suName: item.name,
                        status: 'Success'
                    });
                    // this.reloadData()    //TODO: should remove this function if not required anymore.
                } else {
                    this.failedSchedulingUnits.push({
                        suId: item.id,
                        suName: item.name,
                        status: 'Failed'                        
                    });
                }
            } catch (error) {
                this.failedSchedulingUnits.push({
                    suId: item.id,
                    suName: item.name,
                    status: 'Failed'
                });
            }
            copied++;
            this.setState({spinnerPercent: (copied/totalToCopy*100).toFixed(0)});
        }
        this.tableReloadFunction(this.tableState);
        let dialog = this.state.dialog;
        dialog.type = "success";
        dialog.header = "Copy Scheduling Unit Status";
        dialog.detail = "";
        dialog.content = this.getStatusContent;
        dialog.actions = [{
            id: 'no',
            title: 'Close',
            callback: this.closeDialog,
            className: (this.props.project) ? "dialog-btn" : "act-btn-cancel"
        }];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({
            dialog: dialog,
            dialogVisible: true,
            showSpinner: false,
            spinnerPercent: undefined,
            copySUParams: {}
        });
    }
  
  
    /**
     * Function to get confirmation before cancelling all selected scheduling unit blueprints if the status is 
     * not one of the end statuses. 
     * If no selected scheduling unit is cancellable, show info to select a cancellable scheduling unit.
     * 
     */
    confirmCancelSchedulingUnit(e) {
        e.preventDefault();
        this.pageUpdated = false;
        let selectedBlueprints = this.selectedRows.filter(schedulingUnit => {
                                        return schedulingUnit.type === 'Blueprint' && 
                                        this.SU_END_STATUSES.indexOf(schedulingUnit.status)<0});
        if (selectedBlueprints.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Select at least one cancellable Scheduling Unit Blueprint to cancel.' });
        } else {
            if(!this.state.dialogVisible) {
                this.selectedSURows = _.cloneDeep(this.selectedRows);
            }            
            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header = "Confirm to Cancel Scheduling Unit(s)";
            dialog.actions = [{ id: 'yes', title: 'Cancel SU', callback: this.cancelSchedulingUnit, className:(this.props.project)?"dialog-btn": "", style:{"background": "#e91224","border": "1px solid #e91224"} },
                                { id: 'no', title: 'No', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel" }];
            dialog.detail = "Cancelling the scheduling unit means it will no longer be executed / will be aborted. This action cannot be undone. Already finished/cancelled scheduling unit(s) will be ignored. Do you want to proceed?";
            dialog.content = this.getSUCancelConfirmContent;
            dialog.submit = this.cancelSchedulingUnit;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({ dialog: dialog, dialogVisible: true });
        }
    }
    
    /**
     * Function to cancel all selected Scheduling Unit blueprints if its status is not one of the end statuses
     * and update their status on successful cancellation.
     */
    async cancelSchedulingUnit() {
        let schedulingUnits = this.state.scheduleunit;
        let selectedBlueprints = this.selectedSURows.filter(su => {return su.type === 'Blueprint'});
        let cancelledSchedulingUnits = []
        for (const selectedSU of selectedBlueprints) {
            if (this.SU_END_STATUSES.indexOf(selectedSU.status) < 0) {
                const cancelledSU = await ScheduleService.cancelSchedulingUnit(selectedSU.id);
                let schedulingUnit = _.find(schedulingUnits, {'id': selectedSU.id, type: 'Blueprint'});
                if (cancelledSU) {
                    schedulingUnit.status = cancelledSU.status;
                }
                cancelledSchedulingUnits.push({
                    suId: schedulingUnit.id, suName: schedulingUnit.name, type: schedulingUnit.type,
                    status: schedulingUnit.status.toLowerCase()==='cancelled'?'Cancelled': 'Error Occured'
                });
            }
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Cancel Scheduling Unit(s) Status";
        dialog.actions = [{ id: 'no', title: 'Close', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel" }];
        dialog.detail = ""
        dialog.content = this.getSUCancelStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ scheduleunit: schedulingUnits, cancelledSchedulingUnits: cancelledSchedulingUnits, dialog: dialog, dialogVisible: true });
    }
    
    /**
     * Callback function to close the dialog prompted.
     */
    closeDialog() {
        this.setState({dialogVisible: false, copySUParams: {}});
    }

    /**
     * Switch the SU type and fetch the data from the server
     * @param {String} type - SU Type
     */
    async changeScheduleUnitType(type) {
        let params = type;
        if(type==='Constraints'){
            params='Blueprint'
        }
        await this.getFilterColumns(type.toLowerCase())
        this.lsKeySortColumn = "SchedulingUnit_"+params+"_SortData";
        this.setToggleBySorting();
        this.pageUpdated = true;
        this.filterQry = '';
        this.orderBy = '';
        this.setState({suType: type, scheduleunit: [], loadingStatus:true});
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if (sortData) {
            let columnDetails = _.find(this.state.columnMap, {displayName:sortData[0]?sortData[0].id:sortData.id});
            if(columnDetails) {
                this.orderBy = 'ordering='+((sortData[0]?sortData[0].desc:sortData.desc)?'-':'')+columnDetails.orgField;
            }
        }
        this.getSchedulingUnitList(false,this.changesutype(type),this.filterQry, this.orderBy, this.limit, this.offset);
        this.storeUIAttr("listType", type.replaceAll(" ","_"));
    }

    /**
     * Fetch data from server side - while doing pagination, filtering, sorting
     * @param {Table State} Table props state 
     * @returns 
     */
    async fetchTableData(state) {
        this.filterQry = '';
        this.orderBy = '';
        this.pageUpdated = true;
        this.setState({loadingStatus:true});
        let filters = UtilService.localStore({ type: 'get', key: "scheduleunit_list_"+this.changesutype() });
        if(filters.length > 0 ) {
            this.idFilterValues = [];
            for( const filter of filters) {
                if (filter.id === 'Observation Start Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'on_sky_start_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'on_sky_start_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }   else if (filter.id === 'Observation End Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'on_sky_stop_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'on_sky_stop_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }   else if (filter.id === 'Process Start Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'process_start_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'process_start_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }   else if (filter.id === 'Process End Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'process_stop_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'process_stop_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }   else if ((filter.id === 'Scheduling Unit ID' || filter.id === 'Linked Draft ID') && filter.value !== '') {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    this.idFilterValues = UnitConverter.getSubbandOutput(filter.value);
                    this.filterQry +=  columnDetails.orgField+'='+this.idFilterValues.toString()+"&";
                }   else if (filter.id === 'Linked Blueprint ID' && filter.value !== '') {
                    const values = UnitConverter.getSubbandOutput(filter.value);
                    for ( const value of values) {
                        this.filterQry += 'scheduling_unit_blueprints='+value+"&";
                    }
                }   else if (filter.id === 'Rank' && filter.value !== '') {
                    const values = _.split(filter.value, ",");
                    if (values[0] && values[0] !== '' && values[0]>0) {
                        this.filterQry += "rank_min="+parseFloat(values[0])+'&';
                    }
                    if (values[1] && values[1] !== '' && values[1]>0) {
                        this.filterQry += "rank_max="+parseFloat(values[1])+'&';
                    }
                }   else if ((filter.id === 'Created_At' || filter.id === 'Updated_At') && filter.value !== '') {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    const values = _.split(filter.value, ",");
                    if (values.length>2){
                        continue;
                    }
                    if((values[0] && values[0] !== '' && values[0] !== 'null') && (values[1] && values[1] !== '' && values[1] !== 'null')) {
                        this.filterQry += columnDetails.orgField+'_after='+ moment(new Date(values[0])).format("YYYY-MM-DDT00:00:00")+".000Z&";
                        this.filterQry += columnDetails.orgField+'_before='+moment(new Date(values[1])).format("YYYY-MM-DDT23:59:59")+".000Z&";
                    }
                }   else if (filter.id === 'Duration (HH:mm:ss)' && filter.value !== '') {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    const values = _.split(filter.value, ",");
                    if (values[0].includes(":")) {
                        this.filterQry += columnDetails.orgField + "_min=PT" + UnitConverter.getHHmmssToSecs(values[0]) + "S&";
                    }
                    if (values[1].includes(":")) {
                        this.filterQry += columnDetails.orgField + "_max=PT" + UnitConverter.getHHmmssToSecs(values[1]) + "S&";
                    }
                    
                }   else if (filter.id === 'Template ID' && filter.value !== '') {
                    // Since it is ModelChoiceFilter, it will allow single parameter, so use last parameter if contains multi parameter
                    const values = _.split(filter.value, ",");
                    this.filterQry += 'observation_strategy_template='+values[values.length-1]+"&";
                }   else if (filter.id.toLowerCase() === 'status') {
                    for ( const value of filter.value) {
                        this.filterQry += 'status='+value+"&";
                    }
                }   else {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    if(columnDetails) {
                        this.filterQry += columnDetails.orgField +'='+encodeURIComponent(filter.value)+'&'
                    }
                }
            }
        }

        let sortBy = state.sortBy?state.sortBy[0]:null;
        if (sortBy) {
            this.defaultSortColumn = sortBy;
            UtilService.localStore({ type: 'set', key: this.lsKeySortColumn ,value:sortBy});
            let columnDetails = _.find(this.state.columnMap, {displayName:sortBy.id});
            if(columnDetails) {
                this.orderBy = 'ordering='+((sortBy.desc)?'-':'')+columnDetails.orgField;
            }
        }
        this.filterQry = this.filterQry.substring(0,this.filterQry.length-1);
        await this.getSchedulingUnitList(false, this.changesutype(), this.filterQry, this.orderBy, state.pageSize, (state.pageIndex*state.pageSize));
        return [this.state.scheduleunit, this.totalPage];
    }

    /**
     * Callback function passed to ViewTable component to set the reloader function to be called from 
     * parent component if server side pagination is implemented.
     * @param {Function} tableReloadFunction - function of the ViewTable component that calls the table reload 
     * fucntion to fetch the data with passed state parameters.
     */
    setTableReloadFunction(tableReloadFunction) {
        this.tableReloadFunction = tableReloadFunction;
    }

    /**
     * Callback function passed to ViewTable component to get the ViewTable component state object.
     * @param {Object} tableState - state object of the ViewTable component. This state object is passed as 
     * the parameter when the tableReloadFunction is called to keep the current page, filter and other state
     * parameters retained.
     */
    setTableState(tableState) {
        this.tableState = tableState;
    }
    
    /**
     * Get Option-list values for Select Dropdown filter in 'Viewtable'
     * @param {String} id : Column id
     * @returns 
     */
    getFilterOptions(id) {
        let options = null;
        if(id && id === 'Status') {
            options = this.statusList;
        }   else if (id === 'Linked Draft ID') {
            options = _.sortBy(this.draftIds);
        }   else if (id === 'Priority Queue') {
            options = UIConstants.SU_PRIORITY_QUEUE;
        }
        return options;
    }

    /**
    * Function to invoke form UI to create cleanup task
    */
    async cleanUpSUTask(e) {
        e.preventDefault();
        this.pageUpdated = false; 
       if (this.selectedRows.length > 0) {
           this.suBlueprintList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.type.toLowerCase() === "blueprint"});
           if (this.suBlueprintList.length === 0) {
               appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Blueprint(s)'});
           } else {
               let dialog = this.state.dialog;
               dialog.type = "confirmation";
               dialog.header= "Confirm to Create Clean-up Task(s)";
               dialog.detail = "Do you want to create clean-up task(s) for selected Scheduling Unit Blueprint(s)?";
               dialog.content = this.getCleanUpDialogContent;
               dialog.actions = [{id: 'yes', title: 'Create Clean-up Task', callback: this.createSUCleanupTask, className:(this.props.project)?"dialog-btn": ""},
               {id: 'no', title: 'Cancel', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
               dialog.onSubmit = this.createSUCleanupTask;
               dialog.width = '55vw';
               dialog.showIcon = false;
               this.setState({dialog: dialog, dialogVisible: true});
           }
       }   else {
           appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Blueprint(s)'});
       }
   }
   
   /**
    * Prepare dialog content with SU List
    * @returns : dialog content
    */
    getCleanUpDialogContent() {
       let selectedSchedulingUnits = [];
       for(const su of this.suBlueprintList) {
           selectedSchedulingUnits.push({suId: su.id, suName: su.name});
       }
       return  <> 
                   {selectedSchedulingUnits.length > 0 &&
                       <div style={{marginTop: '1em'}}>
                           <b>Scheduling Unit(s) list to create Clean-up task(s)</b>
                           <DataTable value={selectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                               <Column field="suId" header="Id"></Column>
                               <Column field="suName" header="Name"></Column>
                           </DataTable>
                       </div>
                   }
                   
               </>
   }

   /**
    * Function to create clean-up task for each selected SU blueprint
    */
    async createSUCleanupTask() {
       this.setState({dialogVisible: false});
       this.successSchedulingUnits = [];
       this.failedSchedulingUnits = [];
       for(const su of this.suBlueprintList) {
           try{
               let response = null;
               response = await ScheduleService.createSUCleanupTask(su);
               if(response.message === 'Success') {
                   this.successSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Success'});
               }   else {
                   this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
               }
           } catch(error) {
               this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
           }
       }
      
       let dialog = this.state.dialog;
       dialog.type = "success";
       dialog.header= "Create Clean-up Task(s) Status";
       dialog.detail = "";
       dialog.content = this.getStatusContent;
       dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
       dialog.onSubmit = this.closeDialog;
       dialog.width = '55vw';
       dialog.showIcon = false;
       this.setState({dialog: dialog, dialogVisible: true});
   }

   /**
    * Prepare dialog content for after create Clean-up task
    * @returns 
    */
    getStatusContent() {
       const suList = _.concat(this.successSchedulingUnits, this.failedSchedulingUnits);
       return  <> 
                   {suList.length > 0 &&
                       <div style={{marginTop: '1em'}}>
                           <DataTable value={suList} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                               <Column field="suId" header="Id"></Column>
                               <Column field="suName" header="Name"></Column>
                               <Column field="status" header="Action Status"></Column>
                           </DataTable>
                       </div>
                   }
               </>
    }
    
    /**
     * Function to call the SUBCreator component's function to check and create SUBs
     */
    checkAndCreateSUB(e) {
        e.preventDefault();
        if (this.selectedRows.length > 0) {
            const suBlueprintList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.type.toLowerCase() === "blueprint"});
            const suDraftsList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.type.toLowerCase().startsWith("draft")});
            const hasDrafts = suDraftsList.length > 0 ? true : false;
            const hasBlueprint = suBlueprintList.length > 0 ? true : false;
            if (hasBlueprint && !hasDrafts) {
                appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Draft(s)'});
            }   else if (hasBlueprint && hasDrafts) {
                this.subCreator.checkBlueprint(this, true);
            }   else {
                this.subCreator.checkBlueprint(this, false);
            }
        }   else {
            appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Draft(s)'});
        }
    }

    /**
     * @returns Prepare confirmation dialog content for Pin/Unpin data after ingest
     */
    getAutoDeletionDialogContent() {
        const unPinnedSUList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.output_pinned === false});
        const pinnedSUList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.output_pinned === true});
        const noActionSUList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.output_pinned === null});
        return  <> 
                    {unPinnedSUList.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) that will be marked to Pin data after ingest</b>
                            <DataTable value={unPinnedSUList} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="id" header="Id"></Column>
                                <Column field="name" header="Name"></Column>
                            </DataTable>
                        </div>
                    }
                    {pinnedSUList.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) that will be marked to Unpin data after ingest</b>
                            <DataTable value={pinnedSUList} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="id" header="Id"></Column>
                                <Column field="name" header="Name"></Column>
                            </DataTable>
                        </div>
                    }
                    {noActionSUList.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) that cannot be marked to Pin/Unpin here as some of the tasks are marked to Pin and others to Unpin. Please update them from the details page.</b>
                            <DataTable value={noActionSUList} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="id" header="Id"></Column>
                                <Column field="name" header="Name"></Column>
                            </DataTable>
                        </div>
                    }
                </>
    }

    /**
     * Confirm to Pin/Unpin data after ingest for selected SU draft(s)/blueprint(s)
     */
    async confirmAutoDeletion(e) {
        e.preventDefault();
        if (this.selectedRows.length > 0) {
            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header= "Confirm Pin/Unpin data after ingest";
            dialog.detail = "Do you want to Pin/Unpin data after ingest for selected Scheduling Units?";
            dialog.content = this.getAutoDeletionDialogContent;
            dialog.actions = [{id: 'yes', title: 'Pin/Unpin data', callback: this.setAutoDeletion, className:(this.props.project)?"dialog-btn": ""},
            {id: 'no', title: 'Cancel', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
            dialog.onSubmit = this.setAutoDeletion;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({dialog: dialog, dialogVisible: true});
        }   else {
            appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit(s)'});
        }        
    }

    /**
     * Prevent/Allow automatic deletion for selected SU(s)
     */
    async setAutoDeletion(){
        this.successSchedulingUnits = [];
        this.failedSchedulingUnits = [];
        const unPinnedSUList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.output_pinned === false});
        const pinnedSUList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.output_pinned === true});
        this.successSchedulingUnits = await this.updateAutoDeletion(unPinnedSUList);
        this.failedSchedulingUnits = await this.updateAutoDeletion(pinnedSUList);
        this.tableReloadFunction(this.tableState);
        let dialog = this.state.dialog;
        dialog.type = "success";
        dialog.header= "Pin/Unpin data Status";
        dialog.detail = "";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Call service to set Pin/Unpin data
     * @param {Array} suList : - selected scheduling unit list
     * @returns 
     */
    async updateAutoDeletion(suList) {
        let result = [];
        for(const scheduleunit of suList) {
            let tasks = [];
            if (this.changesutype() === 'draft') {
                for (const task of scheduleunit.task_drafts) {
                    tasks.push({id: task.id, 'output_pinned': !scheduleunit.output_pinned});
                }
            }
            
            let resSu = await ScheduleService.updateSchedulingUnitValue(this.changesutype(), {id: scheduleunit.id, 'output_pinned': !scheduleunit.output_pinned}, tasks);
            if (resSu) {
                scheduleunit['status']= 'Success';
            }   else {
                scheduleunit['status']= 'Failed';
                appGrowl.show({ severity: 'error', summary: 'Failed', detail: '[SU Name : '+scheduleunit.name+'] - Unable to update the Automatic Deletion state.' });
            }
            scheduleunit['suId'] = scheduleunit.id;
            scheduleunit['suName'] =  scheduleunit.name;
            result.push(scheduleunit);
        }
        return result;
    }

    /**
    * Function to invoke form UI to create Rerun Pipeline Task(s)
    */
      async confirmCopyFailedTasks(e) {
        e.preventDefault();
        this.pageUpdated = false; 
        if (this.selectedRows.length > 0) {
           this.suBlueprintList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.status === "error" 
           || schedulingUnit.status === "cancelled"});
           if (this.suBlueprintList.length === 0) {
               appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Blueprint(s) with status as "Error/Cancelled"'});
           } else {
               let dialog = this.state.dialog;
               dialog.type = "confirmation";
               dialog.header= "Confirm to re-run all failed tasks";
               dialog.detail = "Do you want to re-run all failed tasks of the selected Scheduling Unit Blueprint(s)?";
               dialog.content = this.getCopyFailedTasksDlgContent;
               dialog.actions = [{id: 'yes', title: 'Re-run', callback: this.copyFailedTasks, className:(this.props.project)?"dialog-btn": ""},
               {id: 'no', title: 'Cancel', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
               dialog.onSubmit = this.copyFailedTasks;
               dialog.width = '55vw';
               dialog.showIcon = false;
               this.setState({dialog: dialog, dialogVisible: true});
           }
       }   else {
           appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Blueprint(s) with status as "Error/Cancelled"'});
       }
   }

   /**
    * Prepare dialog content with SU List for create Rerun Pipeline Task(s)
    * @returns : dialog content
    */
    getCopyFailedTasksDlgContent() {
        let selectedSchedulingUnits = [];
        let unselectedSchedulingUnits = [];
        for(const su of this.suBlueprintList) {
            selectedSchedulingUnits.push({suId: su.id, suName: su.name, suStatus: su.status});
        }
        let ignoreSUbList = _.difference(this.selectedRows, this.suBlueprintList);
        for(const su of ignoreSUbList) {
            unselectedSchedulingUnits.push({suId: su.id, suName: su.name, suStatus: su.status});
        }
        return  <> 
                    {selectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Failed tasks will be created and executed for these Scheduling Unit(s) with status &apos;Error&apos; / &apos;Cancelled&apos;</b>
                            <DataTable value={selectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suStatus" header="Status"></Column>
                            </DataTable>
                        </div>
                    }
                    {unselectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) with status other than &apos;Error&apos; / &apos;Cancelled&apos; will be ignored</b>
                            <DataTable value={unselectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suStatus" header="Status"></Column>
                            </DataTable>
                        </div>
                    }
                </>
    }

    /**
    * Function to create Rerun Pipeline Task(s) for each selected SU blueprint
    */
     async copyFailedTasks() {
        this.setState({dialogVisible: false, showSpinner: true});
        this.successSchedulingUnits = [];
        this.failedSchedulingUnits = [];
        for(const su of this.suBlueprintList) {
            try{
                let response = null;
                response = await ScheduleService.copySUFailedTasks(su);
                if(response.message === 'Success') {
                    this.successSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Success'});
                }   else {
                    this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
                }
            } catch(error) {
                this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
            }
        }
        this.setState({showSpinner: false});
        let dialog = this.state.dialog;
        dialog.type = "success";
        dialog.header= "Re-run all failed tasks - Status";
        dialog.detail = "";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
    * Function to invoke form UI to update SU Blueprint(s)
    */
      async confirmUpdateSUBlueprint(e) {
        e.preventDefault();
        this.pageUpdated = false; 
        if (this.selectedRows.length > 0) {
           this.suBlueprintList = this.selectedRows;
           let dialog = this.state.dialog;
           dialog.type = "confirmation";
           dialog.header= "Confirm to update blueprints from draft";
           dialog.detail = "Do you want to update selected scheduling unit blueprint(s) according to the changed and added task drafts?";
           dialog.content = this.getUpdateSUBDialogContent;
           dialog.actions = [{id: 'yes', title: 'Update Blueprint(s)', callback: this.updateSUB, className:(this.props.project)?"dialog-btn": ""},
           {id: 'no', title: 'Cancel', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
           dialog.onSubmit = this.updateSUB;
           dialog.width = '55vw';
           dialog.showIcon = false;
           this.setState({dialog: dialog, dialogVisible: true});
        }   else {
            appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Blueprint(s)'});
        }
   }

   /**
    * Prepare dialog content with SU List for Blueprint(s)
    * @returns : dialog content
    */
    getUpdateSUBDialogContent() {
        let selectedSchedulingUnits = [];
        let unselectedSchedulingUnits = [];
        for(const su of this.suBlueprintList) {
            selectedSchedulingUnits.push({suId: su.id, suName: su.name, suStatus: su.status});
        }
        let ignoreSUbList = _.difference(this.selectedRows, this.suBlueprintList);
        for(const su of ignoreSUbList) {
            unselectedSchedulingUnits.push({suId: su.id, suName: su.name, suStatus: su.status});
        }
        return  <> 
                    {selectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Blueprints of these scheduling unit(s) with status &apos;Error&apos; / &apos;Cancelled&apos; will be updated</b>
                            <DataTable value={selectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suStatus" header="Status"></Column>
                            </DataTable>
                        </div>
                    }
                    {unselectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) with status other than &apos;Error&apos; / &apos;Cancelled&apos; will be ignored to update blueprints from task drafts</b>
                            <DataTable value={unselectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suStatus" header="Status"></Column>
                            </DataTable>
                        </div>
                    }
                </>
    }

    /**
    * Function to update selected SU blueprint
    */
     async updateSUB() {
        this.setState({dialogVisible: false, showSpinner: true});
        this.successSchedulingUnits = [];
        this.failedSchedulingUnits = [];
        for(const su of this.suBlueprintList) {
            try{
                let response = null;
                response = await ScheduleService.updateSUBTasks(su);
                if(response.message === 'Success') {
                    this.successSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Success'});
                }   else {
                    this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
                }
            } catch(error) {
                this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
                console.log('Error while update Scheduling Unit Blueprint & Subtasks Graph from Task Drafts', error);
            }
        }
        this.setState({showSpinner: false});
        let dialog = this.state.dialog;
        dialog.type = "success";
        dialog.header= "Update blueprints from draft - Status";
        dialog.detail = "";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Function to show confirmation before copy specification doc for failed task
     */
     async confirmCopySpecAndFailedTasks() {
        if (this.selectedRows.length > 0) {
            this.suBlueprintList = _.filter(this.selectedRows, (schedulingUnit) => { return schedulingUnit.status === "error" 
            || schedulingUnit.status === "cancelled"});
            if (this.suBlueprintList.length === 0) {
                appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Blueprint(s) with status "Error/Cancelled"'});
            } else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header= "Confirm to create draft copy for failed tasks";
                dialog.detail = "Do you want to create draft copy for failed tasks of the selected Scheduling Unit Blueprint(s)?";
                dialog.content = this.getCopySpecDlgContent;
                dialog.actions = [{id: 'yes', title: 'Create draft tasks', callback: this.copySpecAndFailedTasks},
                {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
                dialog.onSubmit = this.copySpecAndFailedTasks;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({dialog: dialog, dialogVisible: true});
            }
        }   else {
            appGrowl.show({severity: 'info', summary: 'Select Row', detail: 'Please select one or more Scheduling Unit Blueprint(s) with status Error/Cancelled'});
        }
    }

    /**
    * Prepare dialog content to show the list of SUs with status 'Error/Cancelled' that will be considered to create draft copy
    * @returns : dialog content
    */
    getCopySpecDlgContent() {
        let selectedSchedulingUnits = [];
        let unselectedSchedulingUnits = [];
        for(const su of this.suBlueprintList) {
            selectedSchedulingUnits.push({suId: su.id, suName: su.name, suStatus: su.status});
        }
        let ignoreSUbList = _.difference(this.selectedRows, this.suBlueprintList);
        for(const su of ignoreSUbList) {
            unselectedSchedulingUnits.push({suId: su.id, suName: su.name, suStatus: su.status});
        }
        return  <> 
                    {selectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) with status &apos;Error&apos; / &apos;Cancelled&apos; will be considered</b>
                            <DataTable value={selectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suStatus" header="Status"></Column>
                            </DataTable>
                        </div>
                    }
                    {unselectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <b>Scheduling Unit(s) with status other than &apos;Error&apos; / &apos;Cancelled&apos; will be ignored</b>
                            <DataTable value={unselectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suStatus" header="Status"></Column>
                            </DataTable>
                        </div>
                    }
                </>
    }

    /**
     * Function to create Rerun Pipeline
     */
    async copySpecAndFailedTasks() {
        this.setState({dialogVisible: false, showSpinner: true});
        this.successSchedulingUnits = [];
        this.failedSchedulingUnits = [];
        for(const su of this.suBlueprintList) {
            try{
                let response = null;
                response = await ScheduleService.copySUSpecAndFailedTasks(su);
                if(response.message === 'Success') {
                    this.successSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Success'});
                }   else {
                    this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
                }
            } catch(error) {
                this.failedSchedulingUnits.push({suId: su.id, suName: su.name, status: 'Failed'});
            }
        }
        this.setState({showSpinner: false});
        let dialog = this.state.dialog;
        dialog.type = "success";
        dialog.header= "Create copy for failed tasks - Status";
        dialog.detail = "";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog, className:(this.props.project)?"dialog-btn": "act-btn-cancel"}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Function to call wnen websocket is connected
     */
    onConnect() {
        try{
            const userDets = localStorage.getItem("user");
            if (userDets) {
                this.websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }



    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    async handleData(data) {
        if (data) {
            console.log('received websocket data:', data)
            const jsonData = JSON.parse(data);
            await this.refreshSUList(jsonData); 
        }
    }

    /**
     * Refresh the SU list
     * @param {*} jsonData 
     */
    async refreshSUList(jsonData) {
        switch (jsonData.object_type) {
            case 'scheduling_unit_draft': {
                let suList = this.state.scheduleunit;
                if (this.state.suType === 'Draft') {
                    if (jsonData.action === 'delete') {
                        _.remove(suList, function (su) { return su.id === jsonData.object_details.id});
                    }   else {
                        suList = await this.refreshSchedulingUnitList('draft', jsonData);
                    }
                    this.setState({scheduleunit: suList}); 
                }
                break;
            }
            case 'scheduling_unit_blueprint': {
                if (this.state.suType === 'Blueprint' || this.state.suType === 'Constraints') {
                    let suList = this.state.scheduleunit;
                    if (jsonData.action === 'delete') {
                        _.remove(suList, function (su) { return su.id === jsonData.object_details.id});
                    }   else {
                        suList = await this.refreshSchedulingUnitList('blueprint', jsonData);                        
                    }
                    this.setState({scheduleunit: suList});
                }
                break;
            }
            default: { break; }
        }
    }
 
    /**
     * Refresh the SU list
     * @param {*} suType 
     * @param {*} jsonData 
     * @returns 
     */
    async refreshSchedulingUnitList(suType, jsonData) {
        let isSearchable = false;
        let suList = this.state.scheduleunit;
        let filterQry = '';
        let updatedSU = {};
        if (this.idFilterValues.length === 0) {
            filterQry = 'id='+jsonData.object_details.id;  
            filterQry += this.filterQry === ''? '': '&'+this.filterQry;          
            isSearchable = true;                        
        }   else if (_.includes(this.idFilterValues, jsonData.object_details.id)) {
            let idFilter = 'id='+this.idFilterValues.toString();
            filterQry = _.replace(this.filterQry, idFilter, ('id='+jsonData.object_details.id));
            isSearchable = true;
        }
        
        if (this.props.project) {
            filterQry += (filterQry === ''?'project=':'&project=')+this.props.project;
        }
        if (isSearchable) {
            let expand = '';
            let fields = '';
            if(suType === 'draft'){
                expand=this.SU_DRAFT_EXPAND;
                fields=this.SU_DRAFT_FIELDS;              
            }else if(suType === 'blueprint'){
                expand=this.SU_BLUEPRINT_EXPAND;
                fields=this.SU_BLUEPRINT_FIELDS;   
            }
            
            let response = await ScheduleService.getSchedulingUnitsExpandWithFilter(suType, expand, filterQry, this.orderBy, this.limit, this.offset,  fields.join(","));
            if (response && response.data.count>0) {
                let constraintColumns = [];
                if (suType === 'blueprint') {
                   if (this.state.suType === 'Constraints') {
                    updatedSU = await this.updateSchedulingUnitConstraint(constraintColumns, null, response.data.results[0]);                    
                   }    else {
                    updatedSU = await this.updateSchedulingUnitBlueprint(null, response.data.results[0]);
                   }
                }   else {
                    updatedSU = await this.updateSchedulingUnitDraft(response.data.results[0]);
                }
                updatedSU = await this.addTargetColumns(updatedSU);
                if (this.state.suType === 'Constraints') {
                    // Sort the constraint column keys and add to default columns
                    let defaultcolumns = this.state.defaultcolumns[0];
                    this.additionalConstColumnOrders = [];
                    for (const constraintColName of _.sortBy(_.keys(constraintColumns))) {
                        defaultcolumns[constraintColName] = constraintColumns[constraintColName];
                        this.additionalConstColumnOrders.push(constraintColumns[constraintColName].name);
                    }
                }
                if (jsonData.action === 'create') {
                    suList.push(updatedSU);
                    this.totalPage++;
                }   else if (jsonData.action === 'update') {
                    _.remove(suList, function (su) { return su.id === jsonData.object_details.id});
                    suList.push(updatedSU);
                }
            }   else {
                _.remove(suList, function (su) { return su.id === jsonData.object_details.id});
            }
        }
        return suList;
    }

    async updateSchedulingUnitBlueprint(workflows, scheduleunit) {
        if (workflows === null) {            
            workflows = await this.timelineCommonUtils.getWorkflowsAndTasks([scheduleunit.id])
        }
        scheduleunit['project'] = scheduleunit.draft.scheduling_set.project.name;
        scheduleunit['project_rank'] = scheduleunit.draft.scheduling_set.project.rank;
        scheduleunit['links'] = ['Project', 'id'];
        scheduleunit['linksURL'] = {
                                    'Project': `/project/view/${scheduleunit.draft.scheduling_set.project.name}`,
                                    'id': `/schedulingunit/view/blueprint/${scheduleunit.id}`
                                };
        scheduleunit['scheduling_set'] = scheduleunit.draft.scheduling_set.name;
        const workflowDetails = this.timelineCommonUtils.getWorkflowDetails(workflows, scheduleunit.id);
        scheduleunit['workflowStatus'] = workflowDetails.status;
        scheduleunit.duration = moment.utc((scheduleunit.duration || 0)*1000).format('HH:mm:ss');
        scheduleunit.type= 'Blueprint'; 
        scheduleunit['actionpath'] ='/schedulingunit/view/blueprint/'+scheduleunit.id;
        scheduleunit['task_content'] = this.getTaskTypeGroupCounts(scheduleunit['task_blueprints']);
        scheduleunit['observation_strategy_template_name'] = scheduleunit.draft.observation_strategy_template? scheduleunit.draft.observation_strategy_template.name : '';
        scheduleunit['observation_strategy_template_description'] = scheduleunit.draft.observation_strategy_template? scheduleunit.draft.observation_strategy_template.description : '';
        scheduleunit['observation_strategy_template_id'] = scheduleunit.draft.observation_strategy_template? scheduleunit.draft.observation_strategy_template.id : '';
        scheduleunit['station_group'] = this.getStationGroup(scheduleunit).counts;
        scheduleunit['draft'] = this.getLinksList([scheduleunit.draft.id], 'draft'); //key 'drafts' used in filters, so use after process or change variable while fetching
        scheduleunit.canSelect = true;
        scheduleunit['suid'] =  scheduleunit.id;
        return scheduleunit;
    }

    async updateSchedulingUnitDraft(scheduleunit) {
        scheduleunit['status'] = null;
        scheduleunit['workflowStatus'] = null;
        scheduleunit['results_accepted'] = null;
        scheduleunit['observation_strategy_template_name'] = scheduleunit.observation_strategy_template?scheduleunit.observation_strategy_template.name:null;
        scheduleunit['observation_strategy_template_description'] = scheduleunit.observation_strategy_template?scheduleunit.observation_strategy_template.description:null;
        scheduleunit['draft'] = this.getLinksList(scheduleunit.scheduling_unit_blueprints_ids, 'blueprint');
        scheduleunit['task_content'] = this.getTaskTypeGroupCounts(scheduleunit['task_drafts']);
        scheduleunit['station_group'] = this.getStationGroup(scheduleunit).counts;
        scheduleunit['actionpath']='/schedulingunit/view/draft/'+scheduleunit.id;
        scheduleunit['type'] = 'Draft';
        scheduleunit['duration'] = moment.utc((scheduleunit.duration || 0)*1000).format('HH:mm:ss');
        scheduleunit['observation_strategy_template_id'] = scheduleunit.observation_strategy_template? scheduleunit.observation_strategy_template.id : '';
        scheduleunit.project = scheduleunit.scheduling_set?scheduleunit.scheduling_set.project.name:'';
        scheduleunit.project_rank = scheduleunit.scheduling_set?scheduleunit.scheduling_set.project.rank:'';
        scheduleunit.canSelect = true;
        scheduleunit.links = ['Project', 'id'];
        scheduleunit.linksURL = {
            'Project': `/project/view/${scheduleunit.scheduling_set?scheduleunit.scheduling_set.project.name:''}`,
            'id': `/schedulingunit/view/draft/${scheduleunit.id}`
        };
        scheduleunit['suid'] =  scheduleunit.id;
        scheduleunit.scheduling_set = scheduleunit.scheduling_set?scheduleunit.scheduling_set.name:'';
        const totalTasks = scheduleunit.task_drafts?scheduleunit.task_drafts.length:0;
        const pinnedTasks = scheduleunit.task_drafts?_.filter(scheduleunit.task_drafts, {'output_pinned': true}):'';
        scheduleunit['output_pinned'] = pinnedTasks.length === 0?false:(totalTasks === pinnedTasks.length || null);
        return scheduleunit;
    }

    /**
     * Group the target values into 4 columns
     * @param {*} schedulingUnits 
     */
    addTargetColumns(scheduleunit) {
        scheduleunit['priority_queue'] = scheduleunit.priority_queue_value;
        scheduleunit.taskDetails = scheduleunit.type?.startsWith("Draft")?scheduleunit.task_drafts:scheduleunit.task_blueprints;
        const targetObserv =  scheduleunit.taskDetails ? scheduleunit.taskDetails.find(task => task.specifications_template.type_value==='observation' && task.specifications_doc.station_configuration?.SAPs) : null;
        scheduleunit['priority_queue'] = scheduleunit.priority_queue_value;
        if (targetObserv) {
            this.scheduleunitTypeColumnToBeRemove = [];
            let targetNames = [];
            let targetAngle1s = [];
            let targetAngle2s = [];
            let targetRefs = [];   
            scheduleunit['target_observation_sap'] = targetObserv.specifications_doc.station_configuration?.SAPs.length;
            let SAPs = targetObserv.specifications_doc.station_configuration?.SAPs
            for (const target of SAPs) {
                targetNames.push(target.digital_pointing.target) ;
                targetAngle1s.push(UnitConverter.getAngleInput(target.digital_pointing.angle1));
                targetAngle2s.push(UnitConverter.getAngleInput(target.digital_pointing.angle2, true));
                targetRefs.push(target.digital_pointing.direction_type);
            }
            scheduleunit['targetName'] = this.formatListToNewLineText(targetNames);
            scheduleunit['targetAngle1'] = this.formatListToNewLineText(targetAngle1s);
            scheduleunit['targetAngle2'] = this.formatListToNewLineText(targetAngle2s);
            scheduleunit['targetRef'] = this.formatListToNewLineText(targetRefs);
        }   else {
            scheduleunit['target_observation_sap'] = 0;
        }
        return scheduleunit;
    }

    async updateSchedulingUnitConstraint(constraintColumns, workflows, scheduleunit) {
        if (workflows === null) {            
            workflows = await this.timelineCommonUtils.getWorkflowsAndTasks([scheduleunit.id])
        }
        scheduleunit['project'] = scheduleunit.draft.scheduling_set.project.name;
        scheduleunit['project_rank'] = scheduleunit.draft.scheduling_set.project.rank;
        scheduleunit['links'] = ['Project', 'id'];
        scheduleunit['linksURL'] = {
                                    'Project': `/project/view/${scheduleunit.draft.scheduling_set.project.name}`,
                                    'id': `/schedulingunit/view/blueprint/${scheduleunit.id}`
                                };
        scheduleunit['scheduling_set'] = scheduleunit.draft.scheduling_set.name;
        const workflowDetails = this.timelineCommonUtils.getWorkflowDetails(workflows, scheduleunit.id);
        scheduleunit['workflowStatus'] = workflowDetails.status;
        scheduleunit.duration = moment.utc((scheduleunit.duration || 0)*1000).format('HH:mm:ss');
        scheduleunit['type'] = "Blueprint"; 
        scheduleunit['priority'] =  scheduleunit.priority || "";
        scheduleunit['actionpath'] ='/schedulingunit/view/blueprint/'+scheduleunit.id;
        scheduleunit['station_group'] = this.getStationGroup(scheduleunit).counts;
        scheduleunit['suid'] =  scheduleunit.id;
        //prepare Constraint details and get constraint columns for only one time in the loop
        if (constraintColumns.length === 0) {
            constraintColumns = this.scheduleConstraintsArray(scheduleunit.scheduling_constraints_doc, undefined,constraintColumns);
        }
        this.constraintColumns = [];                 
        for (const key in derviedscheduleconstraints) {
            if (key !== '$schema') {
                this.constraintColumns.push(key);
                scheduleunit[key] = derviedscheduleconstraints[key];
            }
        }
        return scheduleunit;     
    }

    render(){
        if (this.state.isLoading) {
            return <AppLoader/>
        }
        return(
            <>
                <div className="p-select " style={{position: 'relative'}}>
                    <div className="p-field p-grid" style={{position: 'absolute', marginLeft: '-11em', top: '-2em'}}>
                        <label>Show </label>
                        <span className="p-float-label">
                            <Dropdown inputId="sutype" optionLabel="name" optionValue="name" 
                                tooltip="Scheduling Unit Type/Constraints" tooltipOptions={this.tooltipOptions}
                                value={this.state.suType}                      
                                options={this.suTypeList} 
                                onChange={(e) => {this.changeScheduleUnitType( e.value)}} 
                                style={{width: '18em', marginLeft: '0.5em'}}
                            />
                        </span>                           
                    </div>
                </div>
               {/** 
                Call View table to show table data, the parameters are,
                data - Pass API data
                defaultcolumns - These columns will be populate by default in table with header mentioned
                optionalcolumns - These columns will not appear in the table by default, but user can toggle the columns using toggle menu
                showaction - {true/false} -> to show the action column
                keyaccessor - This is id column for Action item
                paths - specify the path for navigation - Table will set "id" value for each row in action button                    
               */}
               <div className="delete-option">
                    <div >
                        <span className="p-float-label">
                            <>
                            {this.state.scheduleunit && this.state.scheduleunit.length > 0 &&
                                <>
                                {this.state.suType === 'Draft' &&
                                    <>
                                        <a href="#" style={{marginLeft: "5px"}} onClick={(e)=> this.checkAndCreateSUB(e)} 
                                            title={this.state.suPermissions.createSub ? "Create Blueprint(s)":`${this.access_denied_message} to create Blueprint(s)`} >
                                            <i className= {this.state.suPermissions.createSub ?"fa fa-stamp":"fa fa-disabled fa-stamp"} 
                                            aria-hidden="true" ></i>
                                        </a>
                                    </>
                                }
                                {this.state.suType === 'Blueprint' && 
                                    <>
                                        <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.confirmCopyFailedTasks(e)}
                                            title={this.state.suPermissions.createRerun ?"Re-run all failed tasks" :`${this.access_denied_message}  to re-run for all failed tasks`} >
                                            <i className={this.state.suPermissions.createRerun?"fa fa-redo":"fa fa-disabled fa-redo"}
                                            aria-hidden="true" ></i>
                                        </a>
                                        <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.confirmCopySpecAndFailedTasks(e)}
                                            title={this.state.suPermissions.copyFailedTasks ? "Create draft copy for failed tasks" :`${this.access_denied_message} to create draft copy for failed tasks`} >
                                            <i className={this.state.suPermissions.copyFailedTasks?"fas fa-code-branch":"fas fa-disabled fa-code-branch"}
                                            aria-hidden="true" ></i>
                                        </a>
                                        <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.confirmUpdateSUBlueprint(e)}
                                            title={this.state.suPermissions.updateSub ? "Update blueprints from draft" :`${this.access_denied_message} to update blueprint from draft`} >
                                            <i className={this.state.suPermissions.updateSub?"fas fa-save":"fas fa-disabled fa-save"}
                                            aria-hidden="true" ></i>
                                        </a>
                                        <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.cleanUpSUTask(e)}
                                            title={this.state.suPermissions.cleanupTask ? "Create Clean-up Task(s)":`${this.access_denied_message} to create Clean-up Task(s)`} > 
                                            <i className={this.state.suPermissions.cleanupTask?"fa fa-recycle":"fa fa-disabled fa-recycle"}
                                            aria-hidden="true" ></i>
                                        </a>
                                        <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.confirmCancelSchedulingUnit(e)}
                                            title={this.state.suPermissions.cancelSU ? "Cancel selected Scheduling Unit(s)":`${this.access_denied_message} to cancel Scheduling Unit(s)`} > 
                                            <i className={this.state.suPermissions.cancelSU?"fa fa-ban":"fa fa-disabled fa-ban"}
                                            aria-hidden="true" ></i>
                                        </a>
                                    </>
                                }
                                {this.state.suType!== 'Constraints' &&
                                <>
                                <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.confirmAutoDeletion(e)} 
                                    title={this.state.suPermissions.autoDeletion ? "Pin/Unpin data after ingest":`${this.access_denied_message} to update the Pin/Unpin data after ingest`} > 
                                    <i className={this.state.suPermissions.autoDeletion?"fa fa-thumbtack":"fa fa-disabled fa-thumbtack"}
                                    aria-hidden="true" ></i>
                                </a>
                                <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.confirmCopyingSU(e)}
                                    title={this.state.suPermissions.copySU ? "Copy Scheduling Unit(s) Draft/Blueprint":`${this.access_denied_message} to copy Scheduling Unit(s)`} > 
                                    <i className={this.state.suPermissions.copySU?"fa fa-copy":"fa fa-disabled fa-copy"} ></i>
                                </a>                                
                                <a href="#" style={{marginLeft: "5px"}} onClick={(e) => this.showAdvancedCopyDialog(e)}
                                    title={this.state.suPermissions.copySU ? "Advanced Copy Scheduling Unit(s) Draft/Blueprint":`${this.access_denied_message} to copy Scheduling Unit(s)`} > 
                                    <i className={this.state.suPermissions.copySU?"fa fa-window-restore":"fa fa-disabled fa-window-restore"} ></i>
                                </a>
                                {
                                    this.props.project &&
                                    <>
                                    <a href="#" style={{marginLeft: "5px"}} onClick={()=> this.confirmChangeSUSet()} 
                                        title= "Move to another Scheduling Set" >
                                        <i className= {"fas fa-arrows-alt"} 
                                        aria-hidden="true" ></i>
                                    </a>
                                    </>
                                }
                                <a href="#"  style={{marginLeft: "5px"}} onClick={(e) => this.checkAndDeleteSchedulingUnit(e)} 
                                    title={this.state.suPermissions.delete ? "Delete selected Scheduling Unit(s)":`${this.access_denied_message} to delete Scheduling Unit(s)`} > 
                                    <i className={this.state.suPermissions.delete?"fa fa-trash":"fa fa-disabled fa fa-trash"}
                                    aria-hidden="true" ></i>
                                </a>
                                </>
                               }
                            </>
                            }
                            {this.props.project &&
                                    <>
                                        <Link style={{marginLeft: "5px"}} to={`${this.props.project?"/project/"+this.props.project:""}/schedulingunit/create`}
                                            title={this.state.userrole && this.state.userrole.userRolePermission.scheduleunit && this.state.userrole.userRolePermission.scheduleunit.create ? "Add New Scheduling Unit to this Project":`${this.access_denied_message} to add new Scheduling Unit`} > 
                                            <i className={this.state.userrole && this.state.userrole.userRolePermission.scheduleunit && this.state.userrole.userRolePermission.scheduleunit.create?"fa fa-plus":"fa fa-disabled fa fa-plus"}
                                            ></i></Link>
                                        <Link style={{marginLeft: "5px"}} to={`${this.props.project?"/project/"+this.props.project:""}/schedulingset/schedulingunit/create`}
                                            title={this.state.userrole && this.state.userrole.userRolePermission.scheduleunit && this.state.userrole.userRolePermission.scheduleunit.excelview ? "Add Scheduling Set to this Project":`${this.access_denied_message} to add/view Scheduling Set`} > 
                                            <i className={this.state.userrole && this.state.userrole.userRolePermission.scheduleunit && this.state.userrole.userRolePermission.scheduleunit.excelview?"fa fa-table":"fa fa-disabled fa fa-table"}
                                            ></i></Link>
                                    </>
                                }
                            </>
                        </span>
                    </div>                           
                </div>
                {/* {   (this.state.scheduleunit && this.state.scheduleunit.length>0)? */}                
                { this.state.scheduleunit?
                    <ViewTable 
                        data={this.state.scheduleunit} 
                        totalPage={this.totalPage}
                        defaultcolumns={this.state.tmpDefaulcolumns ? this.state.tmpDefaulcolumns : this.state.defaultcolumns} 
                        optionalcolumns={this.state.tmpOptionalcolumns ? this.state.tmpOptionalcolumns : this.state.optionalcolumns}
                        columnclassname={this.state.columnclassname}
                        columnOrders={this.state.tmpColumnOrders}
                        defaultSortColumn={this.defaultSortColumn}
                        showaction={true}
                        keyaccessor="id"
                        paths={this.state.paths}
                        unittest={this.state.unittest}
                        tablename={"scheduleunit_list_"+this.changesutype()}
                        allowRowSelection={this.state.suType.toLowerCase() === 'constraints'? false:this.props.allowRowSelection}
                        onRowSelection = {this.onRowSelection}
                        toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                        lsKeySortColumn={this.lsKeySortColumn}   
                        callBackFunction={this.fetchTableData}   
                        showGlobalFilter={false} showTopPagination={true}
                        loadingStatus={this.state.loadingStatus}
                        showFilterOption={this.getFilterOptions} //Callback function to provide inputs for option-list in Select Dropdown filter
                        pageUpdated={this.pageUpdated}
                        storeFilter={true}
                        setTableReloadFunction={this.setTableReloadFunction}
                        setTableState={this.setTableState}
                    />
                    :<div>No Scheduling Unit found</div>
                 }  
                {/* Component that has functions to create Scheduling unit blueprint */}
                <SUBCreator ref={subCreator => {this.subCreator = subCreator}} project={this.props.project} />
                <CustomDialog type="confirmation" visible={this.state.dialogVisible}
                    header={this.state.dialog.header} message={this.state.dialog.detail} actions={this.state.dialog.actions}
                    content={this.state.dialog.content} width={this.state.dialog.width} showIcon={this.state.dialog.showIcon}
                    onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.state.dialog.onSubmit}/>
                <CustomPageSpinner visible={this.state.showSpinner} percentage={this.state.spinnerPercent} />
                {!this.state.isLoading &&
                    <Websocket ref={websocket => this.websocket = websocket} url={process.env.REACT_APP_WEBSOCKET_URL} 
                        onOpen={this.onConnect} onMessage={this.handleData}  />}
            </>
        )
    }
}

SchedulingUnitList.propTypes = {
  hideProjectColumn: PropTypes.any,
  project: PropTypes.any,
  allowRowSelection: PropTypes.any
}

export default SchedulingUnitList;