import PropTypes from 'prop-types';
import 'leaflet/dist/leaflet.css'
import { Circle, FeatureGroup, LayerGroup, LayersControl, MapContainer, Marker, Popup, Rectangle, TileLayer,Icon, Polyline } from 'react-leaflet'
import L from 'leaflet'
import { useMap } from 'react-leaflet'
import coordinates from './data/bundlecoordinates.json'


delete L.Icon.Default.prototype._getIconUrl;


L.Icon.Default.mergeOptions({
  iconRetinaUrl: require('leaflet/dist/images/marker-icon-2x.png'),
  iconUrl: require('leaflet/dist/images/marker-icon.png'),
  shadowUrl: require('leaflet/dist/images/marker-shadow.png')
});

function GetMapComponent(props) {
  const {
    mapRef
  } = props
  const map = useMap()
  setTimeout(() => { mapRef(map) }, 0);
  return null
}

function StationGeoView(props) {
  const {
    stations,
    mapRef
  } = props




  /*
                "Station names": s,
                "Array statistics": x,
                "HBA": h,
                "LBA": l,
                "Core": c,
                "Remote": r,
                "International": i,
                "Antennas": a,
                "Cabinets": c,
                "Network": n,
                "Outliers": f,
                */

  return (
    <div className="GeoBlock">

      <MapContainer
        className="full-screen-map"
        center={[52.91512333474063, 6.869839092349391]}
        zoom={6}
        minZoom={3}
        maxZoom={19}
        maxBounds={[[-85.06, -180], [85.06, 180]]}
        scrollWheelZoom={true}>
        
        <LayersControl position="topright">
      
        <LayersControl.BaseLayer checked name="Satelite">
          <LayerGroup>
            <TileLayer name="ArcGis" 
          url="http://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}"
        />
        </LayerGroup>
        </LayersControl.BaseLayer>

        
        <LayersControl.BaseLayer checked name="Openstreet">
          <LayerGroup>
        <TileLayer           url="https://tile.openstreetmap.org/{z}/{x}/{y}.png"    />    
        </LayerGroup>
        </LayersControl.BaseLayer>
          <LayersControl.Overlay name="Antennas">
          <LayerGroup>
            {coordinates.a.map((f) => {
              return <FeatureGroup pathOptions={{ color: 'purple' }} key={f.Id}>
                  <Popup>{f.Name}</Popup>
                  <Circle center={[f.Latitude, f.Longitude]} radius={1.2}  />
                  </FeatureGroup>
            })}
            </LayerGroup>
          </LayersControl.Overlay>


          <LayersControl.Overlay name="Outliers">
          <LayerGroup>
            {coordinates.f.map((f) => {
                            return <FeatureGroup pathOptions={{ color: 'yellow' }} key={f.Id}>
                            <Popup>{f.Name}</Popup>
                            <Circle center={[f.Latitude, f.Longitude]} radius={1.2} key={f.Id} color="yellow" />
                            </FeatureGroup>
            })}
            </LayerGroup>
          </LayersControl.Overlay>

          <LayersControl.Overlay name="HBA">
          <LayerGroup>
            {coordinates.h.map((f) => {
                            return <FeatureGroup pathOptions={{ color: 'yellow' }} key={f.Id}>
                            <Popup>{f.Name}</Popup>
                            <Circle center={[f.Latitude, f.Longitude]} radius={1.2} key={f.Id} color="yellow" />
                            </FeatureGroup>
            })}
            </LayerGroup>
          </LayersControl.Overlay>


          <LayersControl.Overlay name="LBA">
          <LayerGroup>
            {coordinates.l.map((f) => {
                            return <FeatureGroup pathOptions={{ color: 'yellow' }} key={f.Id}>
                            <Popup>{f.Name}</Popup>
                            <Circle center={[f.Latitude, f.Longitude]} radius={1.2} key={f.Id} color="yellow" />
                            </FeatureGroup>
            })}
            </LayerGroup>
          </LayersControl.Overlay>

          <LayersControl.Overlay name="Remote">
          <LayerGroup>
            {coordinates.r.map((f) => {
                            return <FeatureGroup pathOptions={{ color: 'orange' }} key={f.Id}>
                            <Popup>{f.Name}</Popup>
                            <Circle center={[f.Latitude, f.Longitude]} radius={1.2} key={f.Id} color="orange" />
                            </FeatureGroup>
            })}
            </LayerGroup>
          </LayersControl.Overlay>

        <LayersControl.Overlay name="International">
          <LayerGroup>
            {coordinates.r.map((f) => {
                            return <FeatureGroup pathOptions={{ color: 'blue' }} key={f.Id}>
                            <Popup>{f.Name}</Popup>
                            <Circle center={[f.Latitude, f.Longitude]} radius={1.2} key={f.Id} color="blue" />
                            </FeatureGroup>
            })}
            </LayerGroup>
          </LayersControl.Overlay>

      


          <LayersControl.Overlay name="Core">
          <LayerGroup>
            {coordinates.c.map((c) => {
                            return <FeatureGroup pathOptions={{ color: 'red' }} key={c.Id}>
                            <Popup>{c.Name}</Popup>
                            <Circle center={[c.Latitude, c.Longitude]} radius={1.2} color="red" />
                            </FeatureGroup>
          

            })}
            </LayerGroup>
          </LayersControl.Overlay>
          
          <LayersControl.Overlay name="Network">
          <LayerGroup>
            {coordinates.n.map((h) => {
              if (h.Icon=="supericon") {
                return <Circle center={[h.Latitude, h.Longitude]} radius={1.2} key={h.Id} color="black" />
 
              }
              else
              {
                return <Polyline positions={h.Points}  key={h.Id} color="black" />
              }
            })}


            </LayerGroup>

          </LayersControl.Overlay>

          <LayersControl.Overlay name="Stations Names" checked>
            <LayerGroup>
              {stations.map((station) => {
                const markerPosition = [
                  station.geometry.coordinates[1],
                  station.geometry.coordinates[0],
                ];

                const customIcon = new L.divIcon({
                  className: 'custom-marker-icon',
                  html: `
            <div >
              <div class="marker-circle ${station.properties.className}"></div>
              <div class="marker-text">${station.properties.name}</div>
            </div>
          `,
                });

                return (
                  <FeatureGroup pathOptions={{ color: 'purple' }} key={"fg" + station.id}>
                    <Popup>Station {station.id}</Popup>
                    <Marker
                      key={station.id}
                      position={markerPosition}
                      icon={customIcon}>

                    </Marker>
                  </FeatureGroup>
                );
              })}

            </LayerGroup>
      
          </LayersControl.Overlay>
        </LayersControl>

        <GetMapComponent mapRef={mapRef} />
      </MapContainer>
    </div>
  );
}

StationGeoView.propTypes = {
  stations: PropTypes.array,
  mapRef: PropTypes.any
}

export default StationGeoView;
