import PropTypes from 'prop-types';
import React, { Component } from 'react';
import _ from 'lodash';
import moment from 'moment';
import "flatpickr/dist/flatpickr.css";
import { Link } from 'react-router-dom';
import AppLoader from "../../layout/components/AppLoader";
import ViewTable from '../../components/ViewTable/ViewTable';
import PageHeader from '../../layout/components/PageHeader';
import UIConstants from '../../utils/ui.constants';
import UnitConverter from '../../utils/unit.converter';
import SystemEventService from '../../services/system.event.service';
import UtilService from '../../services/util.service';
import AuthStore from '../../authenticate/auth.store';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import { PickList } from 'primereact/picklist';
import { Button } from 'primereact/button';
import { Dialog } from 'primereact/dialog';
import { appGrowl } from '../../layout/components/AppGrowl';


const LabelSpanComponent = ({ label, value }) => {
    return (
        <>
            <label className="col-lg-2 col-md-2 col-sm-12">{label}</label>
            <span className="col-lg-4 col-md-4 col-sm-12">{value}</span>
        </>
    );
}

LabelSpanComponent.propTypes = {
  label: PropTypes.any,
  value: PropTypes.any
}

export class ExistingSystemEventList extends Component{
    SYSTEM_EVENT_LIST_TABLE_NAME = 'ext_system_event_list';
    lsKeySortColumn='systemeventSortData';
    constructor(props){
        super(props);
        this.state = {
            testShowDialog: false,
            paths: [{
                "View": "/systemevent/view",
            }],
            systemEventList: null,
            defaultcolumns: [{
                id: {name:"System Event Id"},
                created_at: {
                    name: "Created",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                name: {name:"Name"},
                description: {name:"Description"},
                start: {
                    name: "Start Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                stop: {
                    name: "End Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                affected_tasks_ids: {
                    name:"Affected Tasks",
                },
                actionpath: "actionpath"
            }],
            optionalcolumns:  [{}],
            columnclassname: [{
                "System Event Id":"filter-input-50",
            }],
            isLoading: true,
            userrole: AuthStore.getState(),
            doSearchSEBySUB: false,
            isPickListUpdated: false
        }
        this.selectedRows = [];
        this.defaultSortColumn = [];
        this.filterQry = '';
        this.orderBy = '';
        this.limit = 10;
        this.offset =  0;
        this.currentPageSize = 10;
        this.systemEvents= [];
        this.totalPage = 0;
        this.pageUpdated = true;
        this.seIssueTypeList = [];
        this.seIssueSubTypeList = [];
        this.seSeverityList = [];
        this.seStatusList = [];
        this.seHardwareStationsList = [];
        this.affectedTasksIds = [];

        this.fetchTableData = this.fetchTableData.bind(this);
        this.createSEFilterBySUBTime = this.createSEFilterBySUBTime.bind(this);
        this.closeList = this.closeList.bind(this);
        this.itemTemplate = this.itemTemplate.bind(this);
        this.onChangePickList = this.onChangePickList.bind(this);
        this.closeAddTaskDialog = this.closeAddTaskDialog.bind(this);
        this.addTaskToSystemEvent = this.addTaskToSystemEvent.bind(this);
        this.onRowSelection = this.onRowSelection.bind(this);
    }

    async componentDidMount() {
        UtilService.localStore({ type: 'set', key: this.SYSTEM_EVENT_LIST_TABLE_NAME, value: []});
        this.setState({doSearchSEBySUB: true});
        this.setToggleBySorting();
        this.getFilterColumns();
        if (this.props.schedulingUnit) {
            this.props.schedulingUnit.task_blueprints_ids.forEach(id => {
                const tmpVar = {name: id, value: id};
                this.affectedTasksIds.push(tmpVar);
            });
            let systemLists = await this.fetchTableData();
            if (systemLists && systemLists[0].length>0) {
                this.setState({systemEventList: systemLists[0]});
            }
        }        
    }

    toggleBySorting(sortData) { 
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [{...sortData}] });
    }

    setToggleBySorting() {      
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if(sortData){
            if(Object.prototype.toString.call(sortData) === '[object Array]'){
                this.defaultSortColumn = sortData;
            }
            else{
                this.defaultSortColumn = [{...sortData}];
            }
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] });       
    }

    /**
     * Get server side filter column details form API
     */
    async getFilterColumns() {
        const apiFilters = await SystemEventService.getSystemEventFilterDefinition();
        this.columnMap = [];
        let tmpDefaulColumns = _.cloneDeep(this.state.defaultcolumns[0]);
        let tmpOptionalColumns = _.cloneDeep(this.state.optionalcolumns[0]);
        let tmpColumnOrders = _.cloneDeep(this.state.columnOrders);
        if(apiFilters) {
            tmpDefaulColumns = this.getAPIFilter(apiFilters, tmpDefaulColumns);
            tmpOptionalColumns = this.getAPIFilter(apiFilters, tmpOptionalColumns);
            this.setState({tmpDefaulcolumns: [tmpDefaulColumns], tmpOptionalcolumns:[tmpOptionalColumns], tmpColumnOrders: tmpColumnOrders, columnMap: this.columnMap})
        }
    }

    /**
     * Prepare API FIlter column to view table component
     * @param {*} apiFilters 
     * @param {*} columnDef 
     * @returns 
     */
    getAPIFilter(apiFilters, columnDef) {
        const defaultColKeys = Object.keys(columnDef);
        defaultColKeys.forEach(key => {
            let tmpColMap = {};
            let tempKey = key;
            tmpColMap['orgField'] = tempKey;
            tmpColMap['tmpField'] = tempKey;
            if(columnDef[key]) {
                tmpColMap['displayName'] = columnDef[key]['name'];
            }
            this.columnMap.push(tmpColMap);
            //Set Enable/Disable the Filter & SortBy in each column
            if(apiFilters.data.filters[tempKey]) {
                columnDef[key]['disableSortBy'] = !_.includes(apiFilters.data.ordering, tempKey);
                columnDef[key]['disableFilters'] = false;
                if( (tempKey !== 'start' && tempKey !== 'stop') && UIConstants.FILTER_MAP[apiFilters.data.filters[tempKey].type]) {
                    columnDef[key]['filter'] = UIConstants.FILTER_MAP[apiFilters.data.filters[tempKey].type];
                }
            }   else if (key === 'affected_tasks_ids' && apiFilters.data.filters['affected_tasks']) {
                columnDef[key]['disableSortBy'] = !_.includes(apiFilters.data.ordering, 'affected_tasks');
                columnDef[key]['disableFilters'] = false;
            }
        });
        return columnDef;
    }
    
    /**
     * Create default System Event (SE) filter for last 24 Hrs using SUB on sky time
     */
    async createSEFilterBySUBTime() {
        let filterQry = '';
        if (this.props.schedulingUnit.on_sky_stop_time) {
            let  startTime = new Date(this.props.schedulingUnit.on_sky_stop_time);
            filterQry = 'start_before='+ moment(startTime).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
            let stopTime = new Date(this.props.schedulingUnit.on_sky_start_time);
            stopTime.setDate(stopTime.getDate()-1);
            filterQry += 'stop_rangeornull_after='+ moment(stopTime).format("YYYY-MM-DDTHH:mm:ss")+".000Z";
        }
        return filterQry;
    }

    /**
     * Fetch data from server side - while doing pagination, filtering, sorting
     * @param {Table State} Table props state 
     * @returns 
     */
    async fetchTableData(state) {
        this.filterQry = '';
        this.orderBy = '';
        this.pageUpdated = true;
        this.setState({loadingStatus:true});
        let seFilters = '';
        this.currentPageSize = (state && state.pageSize) ? state.pageSize : this.currentPageSize;
        let offset = (state && state.pageIndex) ? state.pageIndex*this.currentPageSize : 0;
        const sortByValue = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn});
        let filters = UtilService.localStore({ type: 'get', key: this.SYSTEM_EVENT_LIST_TABLE_NAME});
        if (!state || state.filters.length === 0) {
            seFilters = await this.createSEFilterBySUBTime();            
        }
        if (seFilters.length>0) {
            this.filterQry = seFilters;
        }   else {
            filters = state?state.filters:[];
            if(filters.length > 0 ) {
                for( const filter of filters) {
                if (filter.id === 'Created') {
                    const values = filter.value;
                    if (values[0] && values[0] !== '') {
                        this.filterQry += 'created_at_after='+ moment(new Date(values[0])).format("YYYY-MM-DD HH:mm:SS&");
                    }
                    if (values[1] && values[1] !== '') {
                        this.filterQry += 'created_at_before='+moment(new Date(values[1])).format("YYYY-MM-DD HH:mm:ss&");
                    }
                }   else if (filter.id === 'Start Time') {
                    const values = filter.value;
                    if (values[0] && values[0] !== '') {
                        this.filterQry += 'start_after='+ moment(new Date(values[0])).format("YYYY-MM-DD HH:mm:SS&");
                    }
                    if (values[1] && values[1] !== '') {
                        this.filterQry += 'start_before='+moment(new Date(values[1])).format("YYYY-MM-DD HH:mm:ss&");
                    }
                }   else if (filter.id === 'End Time') {
                    const values = filter.value;
                    if (values[0] && values[0] !== '') {
                        this.filterQry += 'stop_rangeornull_after='+ moment(new Date(values[0])).format("YYYY-MM-DD 00:00:00&");
                    }
                    if (values[1] && values[1] !== '') {
                        this.filterQry += 'stop_rangeornull_before='+moment(new Date(values[1])).format("YYYY-MM-DD 23:59:59&");
                    }
                }   else if (filter.id === 'Affected Tasks') {
                    let values = [];
                    if (Array.isArray(filter.value)) {
                        values = filter.value;
                    }   else {
                        values = UnitConverter.getSubbandOutput(filter.value);
                    }
                    for ( const value of values) {
                        this.filterQry += 'affected_tasks='+value+'&';
                    }
                }   else if (filter.id === 'System Event Id') {
                    const values = UnitConverter.getSubbandOutput(filter.value);
                    if (values && values.length>0) {
                        this.filterQry += 'id_min='+values[0]+'&';
                        this.filterQry += 'id_max='+values[values.length-1]+'&';
                    }
                }   else {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    if(columnDetails) {
                        this.filterQry += columnDetails.orgField +'='+ encodeURIComponent(filter.value) +'&'
                    }
                    }  
                }
            }
        }
 
        let sortBy = state && state.sortBy?state.sortBy[0]:(sortByValue)?sortByValue:null;
        if (sortBy) {
            this.defaultSortColumn = sortBy;
            UtilService.localStore({ type: 'set', key: this.lsKeySortColumn ,value:sortBy});
            let columnDetails = _.find(this.state.columnMap, {displayName:sortBy.id});
            if(columnDetails) {
                this.orderBy = 'ordering='+((sortBy.desc)?'-':'')+columnDetails.orgField;
            }
        }
        this.filterQry = this.filterQry.substring(0,this.filterQry.length-1);
        await this.getSystemEventList(this.filterQry, this.orderBy, this.currentPageSize, offset);
        return [this.state.systemEventList, this.totalPage];
     }


    /**
     * Get system events for given paramter
     * @param {*} filterQry 
     * @param {*} orderBy 
     * @param {*} limit 
     * @param {*} offset 
     */
    async getSystemEventList(filterQry, orderBy, limit, offset) {
        const promises = [  
            AuthUtil.getUserRolePermission(),
            SystemEventService.getSystemEventWithFilter(filterQry, orderBy, limit, offset),
        ];
        await Promise.all(promises).then(responses => {
            let systemEvent = {};
            this.setState({userrole: responses[0]});
            this.totalPage = responses[1] && responses[1].data?responses[1].data.count:1;
            this.systemEvents = [];
            if (responses[1] && responses[1].data) {               
                for( const response  of responses[1].data.results){
                    systemEvent = response;
                    if(systemEvent.stop === null || systemEvent.stop === ''){
                        systemEvent['stop'] = 'Unknown';
                    } else {
                        systemEvent['stop'] = moment(systemEvent['stop']).format(UIConstants.CALENDAR_DATETIME_FORMAT);
                    }
                    systemEvent['start']= moment(systemEvent['start']).format(UIConstants.CALENDAR_DATETIME_FORMAT);
                    systemEvent['actionpath'] = `/systemevent/view/${systemEvent.id}`;
                    systemEvent['tmp_affected_tasks_ids'] = systemEvent.affected_tasks_ids;
                    systemEvent['affected_tasks_ids'] = this.getLinksList(systemEvent.affected_tasks_ids);
                    systemEvent['jira_url']= <a href={systemEvent.jira_url} target="_blank" rel="noreferrer">{systemEvent.jira_url}</a>;
                    this.systemEvents.push(systemEvent);
                }
            }
            this.pageUpdated = true;
            this.setState({
                isLoading: false,
                systemEventList: this.systemEvents,
            });
        });
    }

    /**
     * Function to get a component with list of links to a list of ids
     * @param {Array} linkedItems - list of ids
     */
    getLinksList = (linkedItems) => {
        return (
            <>
                {linkedItems.length>0 && linkedItems.map((item) => (
                    <Link key={item} style={{paddingRight: '3px'}} to={`/task/view/blueprint/${item}`} >{item}</Link>
                ))}
            </>
        );                    
    }

    closeList(){
        this.props.history.length>1?this.props.history.goBack():this.props.history.push(`/su/timelineview/week`);
    }

    closeAddTaskDialog() {
        this.setState({ showAddTasksDialog: false, affectedTasks: [], currentTasks: [] });
    }

    /**
     * Update the source and target value for picklist
     * @param {object} event 
     */
    onChangePickList(event) {
        this.setState({
            isPickListUpdated: true,
            currentTasks: event.source,
            affectedTasks: event.target,
            isDirty: true
        });
    }

    /**
     * Update the System Event
     */
    async addTaskToSystemEvent() {
        let systemEvent = _.cloneDeep(this.state.systemevent);
        let affected_tasks = systemEvent['affected_tasks'];
        //remove related tasks of current SUB from system Event
        let tasks = this.props.schedulingUnit.task_blueprints;
        for (const task of tasks) {
            _.remove(affected_tasks,  function(affectedTask){
                return affectedTask === task.url;
            });
        }
        for (const affectedTask of this.state.affectedTasks) {
            affected_tasks.push(affectedTask.url);
        }
        systemEvent['stop'] = (systemEvent['stop'] && systemEvent['stop'] !== 'Invalid date' && systemEvent['stop'] !== 'Unknown') ? moment(systemEvent['stop']).format(UIConstants.CALENDAR_DATETIME_FORMAT) : null;
        systemEvent['affected_tasks'] = affected_tasks;
        systemEvent['jira_url'] = systemEvent['jira_url'].props['href'];
        systemEvent = await SystemEventService.updateSystemEvent(systemEvent);
        if (systemEvent && systemEvent.id) {
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'System Event updated successfully.' });
            this.props.refreshSystemList();
        } else {
            appGrowl.show({ severity: 'error', summary: 'Error Occured', detail: 'Unable to update System Event'});
        }
        this.setState({ showAddTasksDialog: false, affectedTasks:[] });
        this.fetchTableData();
    }

    /**
     * Update Picklist option value
     * @param {Object} item 
     * @returns 
     */
    itemTemplate(item) {
        const displayName = `${item.name} - ${item.shortDescription} [${item.controlId}]`;
        return (
            <div className="task-category">
                <span className="task-item">{displayName}</span>
            </div>
        );
    }

    /**
     * Set selected row details
     * @param {Object} selectedRows 
     */
    onRowSelection(selectedRows) {
        this.selectedRows = selectedRows;
        if (this.props.onRowSelection) {
            this.props.onRowSelection(selectedRows) ;
        }
        
    }

    /**
     * Prepare the affected task details and show in pick list
    */
    showAddTasksDialog() {
        if (this.selectedRows.length === 0) {
            appGrowl.show({severity: 'info', summary: 'Info', detail: 'Select an existing System Event to add one or more tasks from this Scheduling Unit'});
        }   else if (this.selectedRows.length>1) {  
            appGrowl.show({severity: 'info', summary: 'Info', detail: 'Select one System Event to add Affected Task(s)'});
        }   else {
            let tmp_affected_tasks_ids = this.selectedRows[0]['tmp_affected_tasks_ids'];
            tmp_affected_tasks_ids = tmp_affected_tasks_ids.length>0 ?tmp_affected_tasks_ids:[];
            let affectedTasks = [];
            let currentTasks = [];
            this.props.schedulingUnit.task_blueprints.forEach(task => {
                if (task.task_type.toLowerCase() === 'observation') {
                    const controlTask = _.find(task.subtasks, subtask => { return subtask.primary === true});
                    currentTasks.push({"id": task.id, "name": task.name+' ['+task.id+']', 
                                        "taskName": task.name, 'url':task.url,
                                        "shortDescription": task.short_description,
                                        "controlId": controlTask?controlTask.id:""
                                    });
                }
            });
            _.sortBy(currentTasks);
            let affectedTaskList = currentTasks;
            currentTasks = [];
            for ( const affectedTask of affectedTaskList) {
                if (_.includes(tmp_affected_tasks_ids, affectedTask.id)){
                    affectedTasks.push(affectedTask);
                }   else {
                    currentTasks.push(affectedTask);
                }
            }
            this.setState({showAddTasksDialog: true, systemevent: this.selectedRows[0], affectedTasks: affectedTasks, currentTasks: currentTasks});
        }
    }

    RenderLabelSpan =(Label,FieldValue) =>{
        return  <><label className="col-lg-2 col-md-2 col-sm-12">{Label}</label>
        <span className="col-lg-4 col-md-4 col-sm-12">{FieldValue}</span></>
    }


    render() {
        const permissions = this.state.userrole.userRolePermission.systemevent;
        return(
            <React.Fragment>
                {!this.props.schedulingUnit &&
                    <PageHeader location={this.props.location} title={'System Issues - List'} 
                                actions={[{icon: 'fa-plus-square', 
                                    title:permissions.create?'Add System Event': "Don't have permission to add new System Event",
                                    disabled: permissions.create? !permissions.create: true,
                                    props : { pathname: `/systemevent/create`}},
                                    {icon: 'fa-window-close', title:'Click to close System Issues list', type: 'button',  
                                    actOn: 'click', props:{ callback: this.closeList }}]}/>
                }
                {this.state.isLoading? <AppLoader /> : (permissions.list) ?
                <>
                {this.state.systemEventList?
                    <>
                        <p>By default, the list shows the events occured in the last 24hrs of the Scheduling unit observation time. Apply filters for Start Time and End Time to view others also. </p>
                         <ViewTable 
                            data={this.state.systemEventList} 
                            defaultcolumns={this.state.tmpDefaulcolumns ? this.state.tmpDefaulcolumns : this.state.defaultcolumns} 
                            optionalcolumns={this.state.tmpOptionalcolumns ? this.state.tmpOptionalcolumns : this.state.optionalcolumns}
                            columnclassname={this.state.columnclassname}
                            defaultSortColumn={this.defaultSortColumn}
                            isOnlySingleRowAllowed={true}
                            showaction={true}
                            showActionInNewTab={this.props.schedulingUnit?true:false}
                            paths={this.state.paths}
                            tablename={this.SYSTEM_EVENT_LIST_TABLE_NAME}
                            showCSV={this.props.schedulingUnit?false:true}
                            allowRowSelection={true}
                            onRowSelection={this.onRowSelection}
                            toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                            lsKeySortColumn={this.lsKeySortColumn}
                            pageUpdated={this.pageUpdated}
                            storeFilter={this.props.schedulingUnit?false:true}
                            callBackFunction={this.fetchTableData}
                            totalPage={this.totalPage}
                            showTopTotal={this.props.schedulingUnit?false:true}
                            showGlobalFilter={false}
                            showTopComponents={false}
                        />
                       </>: <div>No System Event found </div>}
                        <Dialog header={"Add Task(s) to this System Event"} style={{width: '50vw', height: '50vw'}} visible={this.state.showAddTasksDialog} 
                            maximizable maximized={false} onHide={this.closeAddTaskDialog} inputId="confirm_dialog"
                            footer={<div>
                                    <Button  label="Save" icon="pi pi-check" title='Save the System Event with updated Task' onClick={() => {this.addTaskToSystemEvent()}} style={{width: '9em'}} disabled={!this.state.isPickListUpdated}/>
                                    <Button className="act-btn-cancel" icon="pi pi-times" label="Cancel" onClick={this.closeAddTaskDialog} />
                                </div>
                            } 
                            >
                            <div className="ag-theme-balham" style={{ width: '100%'}}>
                                {this.state.systemevent && 
                                    <div className="main-content" style={{width: '100%'}}>
                                        <div className="p-grid">
                                            <LabelSpanComponent label="Name" value={this.state.systemevent.name}></LabelSpanComponent>
                                            <LabelSpanComponent label="Description" value={this.state.systemevent.description}></LabelSpanComponent>
                                        </div>
                                        <div className="p-grid">
                                            <LabelSpanComponent label="Issue Type" value={this.state.systemevent.issue_type_value}></LabelSpanComponent>
                                            <LabelSpanComponent label="Issue Subtype" value={this.state.systemevent.issue_subtype_value}></LabelSpanComponent>
                                        </div>
                                        <div className="p-grid">
                                            <LabelSpanComponent label="Jira URL" value={this.state.systemevent.jira_url}></LabelSpanComponent>
                                            <LabelSpanComponent label="Notes" value={this.state.systemevent.notes}></LabelSpanComponent>
                                        </div>
                                        <div className="p-grid">
                                            <LabelSpanComponent label="Severity" value={this.state.systemevent.severity_value}></LabelSpanComponent>
                                            <LabelSpanComponent label="Status" value={this.state.systemevent.status_value}></LabelSpanComponent>
                                        </div>
                                        <div className="p-grid">
                                            <LabelSpanComponent label="Start Time" value={moment.utc(this.state.systemevent.start).format(UIConstants.CALENDAR_DATETIME_FORMAT)}></LabelSpanComponent>
                                            <LabelSpanComponent label="End Time" value={(this.state.systemevent.stop && this.state.systemevent.stop !== 'Unknown')?moment.utc(this.state.systemevent.stop).format(UIConstants.CALENDAR_DATETIME_FORMAT): 'Unknown'}></LabelSpanComponent>
                                        </div>
                                    </div>
                                }
                                <div className="p-fluid" style={{width: '60%'}}>
                                    <div className="p-grid">
                                        <div className="p-col-12" style={{width: "100%"}}>
                                            <div className="picklist-demo">
                                                <div className="card">
                                                    <PickList source={this.state.currentTasks} target={this.state.affectedTasks} itemTemplate={this.itemTemplate} sourceHeader="Task(s)" targetHeader="Affected Task(s)"
                                                        sourceStyle={{ height: '14em', width: '20em' }} targetStyle={{ height: '14em', width: '20em' }} onChange={this.onChangePickList} />
                                                </div>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </Dialog>
                    </>
                : <AccessDenied/>
                }
            </React.Fragment>
        );
    }
}

ExistingSystemEventList.propTypes = {
  schedulingUnit: PropTypes.shape({
    task_blueprints_ids: PropTypes.shape({
      forEach: PropTypes.func
    }),
    on_sky_stop_time: PropTypes.any,
    on_sky_start_time: PropTypes.any,
    task_blueprints: PropTypes.shape({
      forEach: PropTypes.func
    })
  }),
  history: PropTypes.object,
  refreshSystemList: PropTypes.func,
  onRowSelection: PropTypes.func,
  location: PropTypes.oneOfType([PropTypes.object, PropTypes.string])
}