
import { BrowserRouter as Router } from 'react-router-dom';
import 'regenerator-runtime/runtime'
//import { MemoryRouter } from 'react-router-dom';
import { render,  cleanup } from '@testing-library/react';
import mockConsole from 'jest-mock-console';
import {DataProduct} from './dataproduct';
import DataProductServiceMock from '../../__mocks__/dataproduct.service.data';
import { act} from "react-dom/test-utils";
import TaskService from '../../services/task.service';
import TaskServiceMock from '../../__mocks__/task.service.data';
import DataProductService from '../../services/data.product.service';
import _ from 'lodash';

let TaskDetailsSpy, SubtaskSpy, InputDataProductSpy, OutputDataProductSpy;
let container = null;
let restoreConsole;

beforeEach(() => { 
    restoreConsole = mockConsole();
    container = document.createElement("div");
    document.body.appendChild(container);
    setMockSpy();
  });

  afterEach(() => {
    restoreConsole();
    // cleanup on exiting
    clearMockSpy();
    cleanup();
    container.remove();
    container = null;
  });

    const setMockSpy = (async() => {   
        TaskDetailsSpy = jest.spyOn(TaskService, 'getTaskDetails');
        TaskDetailsSpy.mockImplementation(() =>{
            return Promise.resolve(TaskServiceMock.taskDetails);
        });
        SubtaskSpy = jest.spyOn(DataProductService, 'getSubtask');
        SubtaskSpy.mockImplementation((id) =>{
            let subtask = {};
            subtask.data =_.filter(DataProductServiceMock.subtasks, {'id': id})[0]
           return Promise.resolve(subtask)
        });
        InputDataProductSpy = jest.spyOn(DataProductService, 'getSubtaskInputDataproduct');
        InputDataProductSpy.mockImplementation((id) =>{
            let inputDataProduct = {};
            inputDataProduct.data = _.filter(DataProductServiceMock.inputDataProdouct, {'id': id})
            return Promise.resolve(inputDataProduct ? inputDataProduct: []);
        });
        OutputDataProductSpy = jest.spyOn(DataProductService, 'getSubtaskOutputDataproduct');
        OutputDataProductSpy.mockImplementation((id) =>{
            let outputDataProduct = {}; 
            let outputData = _.filter(DataProductServiceMock.outputDataProdouct, {'id': id})[0]
            outputDataProduct.data = outputData ? outputData.output : [];
            return Promise.resolve(outputDataProduct);
        })
    });

    const clearMockSpy = (async() => {
        TaskDetailsSpy.mockRestore();
        SubtaskSpy.mockRestore();
        OutputDataProductSpy.mockRestore();
        InputDataProductSpy.mockRestore();
    })

    it("renders without crashing", async() =>{
        act(() => {
        render(<Router><DataProduct match={{params: {id: 1}}} /></Router>, container);
        });
    })

    it('render table in the dataproduct list', async () => {
        let content;
        await act(async () => {
            content = render(<Router><DataProduct match={{params: {id: 1}}} /></Router>, container);
        });

        const element = content.queryByTestId('viewtable');
        expect(element).toBeInTheDocument();
        expect(content.queryAllByTestId('tablerow').length).toBe(244);
        expect(content.queryAllByRole('tablehead').length).toBe(7);
        
    })

    it('render File name - checking if data load', async () => {
        let content;
        await act(async () => {
            content = render(<Router><DataProduct match={{params: {id: 1}}} /></Router>, container);
        });
        expect(content.queryAllByTestId('tablerow')[1].innerHTML.includes('L2000000_SAP000_SB242_uv.MS')).toBeTruthy();
    });
