import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { Redirect } from 'react-router-dom'
import moment from 'moment';
import { Dialog } from 'primereact/dialog';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import { Dropdown } from 'primereact/dropdown';
import _ from 'lodash';
import TaskService from '../../services/task.service';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import ViewTable from '../../components/ViewTable/ViewTable';
import UIConstants from '../../utils/ui.constants';
import TaskStatusLogs from './state_logs';
import { appGrowl } from '../../layout/components/AppGrowl';
import { CustomDialog } from '../../layout/components/CustomDialog';
import UnitConverter from '../../utils/unit.converter';
import UtilService from '../../services/util.service';
import { Link } from 'react-router-dom';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import Websocket from 'react-websocket';

export class TaskList extends Component {
    
    // The following values should be lower case
    ignoreSorting = ['status logs'];
    TASK_END_STATUSES = ['finished', 'error', 'cancelled'];
    TASK_DRAFT_EXPAND = 'task_blueprints,task_blueprints,scheduling_unit_draft,scheduling_unit_draft.scheduling_set';
    TASK_DRAFT_FIELDS = ['id', 'name', 'description', 'short_description', 'task_type', 'url', 'duration', 'relative_start_time', 'relative_stop_time', 
                            'created_at', 'updated_at', 'task_blueprints.name', 'task_blueprints.id', 'task_blueprints_ids', 'tags', 
                            'specifications_template_id',
                            'scheduling_unit_draft_id', 'scheduling_unit_draft.id', 'scheduling_unit_draft.name',
                            'scheduling_unit_draft.scheduling_set.project',  'scheduling_unit_draft.scheduling_set.project_id','scheduling_unit_draft.scheduling_set.name'
                        ]
    TASK_BLUEPRINT_EXPAND = 'subtasks,draft,scheduling_unit_blueprint,draft.scheduling_unit_draft,draft.scheduling_unit_draft.scheduling_set';
    TASK_BLUEPRINT_FIELDS = ['id', 'name', 'description', 'short_description', 'status', 'unschedulable_reason', 'task_type', 'url', 'on_sky_start_time', 'on_sky_stop_time', 
                                'process_start_time', 'process_stop_time', 'duration', 'relative_start_time', 'relative_stop_time', 
                                'obsolete_since', 'created_at', 'updated_at', 'draft_id', 'tags', 'specifications_template_id',
                                'scheduling_unit_blueprint_id', 'scheduling_unit_blueprint.id', 'scheduling_unit_blueprint.name',
                                'draft.scheduling_unit_draft.scheduling_set.project',  'draft.scheduling_unit_draft.scheduling_set.project_id','draft.scheduling_unit_draft.scheduling_set.name',
                                'subtasks.id', 'subtasks.primary', 'subtasks.total_nr_of_dataproducts', 'subtasks.total_size_of_dataproducts', 'subtasks.total_size_of_dataproducts_on_filesysem'
                            ]
    COMMA_SEPARATE_TOOLTIP ='Enter Id and press ‘Enter’ key to search. For multiple values enter comma separated values. For range, provide input like 1..6';

    constructor(props) {
        super(props);
        this.getUIAttr();
        this.state = {
            taskType: this.taskUIAttr['listType'] || 'Draft',
            isLoading: true,
            tasks: [],
            obsoleteTasks: [],
            paths: [{
                "View": "/task",
            }],
            userrole: {
                userRolePermission: {}
            },
            columnOrders: [
                "Status Logs",
                "Status",
                "Project",
                "Scheduling Unit ID",
                "Scheduling Unit Name",
                "Scheduling Set",
                "ID",
                "Control ID",
                "Name",
                "Task Type",
                "Description",
                "Short Description",
                "Observation Start Time",
                "Observation End Time",
                "Process Start Time",
                "Process End Time",
                "Duration (HH:mm:ss)",
                "Relative Start Time (HH:mm:ss)",
                "Relative End Time (HH:mm:ss)",
                "#Dataproducts",
                "Data size [TB]",
                "Data size on Disk [TB]",
                "Subtask Content",
                "Tags",
                "Linked Draft ID",
                "API URL",
                "Cancelled",
                "Obsolete",
                "Created at",
                "Updated at"
            ],
            dialog: {},
            defaultcolumns: [{
                status_logs: {
                    name:"Status Logs"
                },
                status: {
                    name: "Status",
                    filter: "multiselect-filter",
                    showTag: true,
                    tagClass: 'su-$dynamic$' //$dynamic$ will be replace with cell value
                },
                unschedulable_reason: {
                    name:"Unschedulable Reason",    
                },
                obsolete: {
                    name: "Obsolete",
                    filter: "switch"                    
                },
                scheduling_unit_draft: {
                    name:"Scheduling Unit ID",
                    tooltip: this.COMMA_SEPARATE_TOOLTIP
                },
                scheduling_unit_blueprint: {
                    name:"Scheduling Unit ID",
                    tooltip: this.COMMA_SEPARATE_TOOLTIP
                },
                schedulingUnitName: {
                    name:"Scheduling Unit Name",
                },
                scheduling_set_name: {
                    name:"Scheduling Set",
                },
                project: {
                    name: "Project"
                },
                id: {
                    name:"ID",
                    tooltip: this.COMMA_SEPARATE_TOOLTIP
                },
                subTaskID: {
                    name:"Control ID",
                    tooltip: this.COMMA_SEPARATE_TOOLTIP
                },
                name: {
                    name:"Name"
                },
                description: {
                    name:"Description"
                },
                short_description: {
                    name:"Short Description"
                },
                on_sky_start_time: {
                    name: "Observation Start Time",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                on_sky_stop_time: {
                    name: "Observation End Time",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                process_start_time: {
                    name: "Process Start Time",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                process_stop_time: {
                    name: "Process End Time",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                duration: {
                    name:"Duration (HH:mm:ss)",
                    filter: "durationMinMax",
                },
                relative_start_time: {
                    name:"Relative Start Time (HH:mm:ss)",
                    filter: "durationMinMax",
                },
                relative_stop_time: {
                    name:"Relative End Time (HH:mm:ss)",
                    filter: "durationMinMax",
                },
                noOfOutputProducts: {
                    name:"#Dataproducts"
                },
                do_cancel: {
                    name: "Cancelled",
                    filter: "switch"
                },
            }],
            optionalcolumns: [{
                task_type: {
                    name: "Task Type",
                    filter: "multiselect-filter"
                },
                size: {
                    name:"Data size [TB]"
                },
                dataSizeOnDisk: {
                    name:"Data size on Disk [TB]"
                },
                subtaskContent: {
                    name:"Subtask Content"
                },
                tags: {
                    name:"Tags"
                },
                draft: {
                    name:"Linked Draft ID",
                    tooltip: this.COMMA_SEPARATE_TOOLTIP
                },
                url: {
                    name:"API URL"
                },
                created_at: {
                    name: "Created at",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                updated_at: {
                    name: "Updated at",
                    filter: "flatpickrDateRange",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                actionpath: "actionpath"
            }],
            columnclassname: [{
                "Status Logs": "filter-input-0",
                "Type": "filter-input-75",
                "Scheduling Unit ID": "filter-input-50",
                "Scheduling Unit Name": "filter-input-100",
                "Project":"filter-input-100",
                "Scheduling Set":"filter-input-100",
                "ID": "filter-input-50",
                "Control ID": "filter-input-75",
                "Cancelled": "filter-input-50",
                "Duration (HH:mm:ss)": "filter-input-75",
                "Template ID": "filter-input-50",
                "Linked Draft ID": "filter-input-75",
                "Relative Start Time (HH:mm:ss)": "filter-input-100",
                "Relative End Time (HH:mm:ss)": "filter-input-100",
                "Observation Start Time": "filter-input-150",
                "Observation End Time": "filter-input-150",
                "Process Start Time": "filter-input-150",
                "Process End Time": "filter-input-150",
                "Status": "filter-input-125",
                "#Dataproducts": "filter-input-75",
                "Data size [TB]": "filter-input-50",
                "Data size on Disk [TB]": "filter-input-50",
                "Subtask Content": "filter-input-75",
                "Linked BluePrint ID": "filter-input-75",
                "API URL":"filter-input-175",
                "Created at":"filter-input-150",
                "Updated at":"filter-input-150",
                
            }],
            selectedOnHoldRows: [],
            actions: [],
            selectedTaskWithOnhold: []
        };
        this.selectedTaskRows = [];
        this.idFilterValues = [];
        this.statusList = [];
        this.blueprintTaskTypeList = [];
        this.access_denied_message = "Don't have permission";
        this.pageUpdated = true;
        this.taskTypeList = [{name: 'Blueprint'}, {name: 'Draft'}];
        this.filterQry = '';
        this.orderBy = '';
        this.limit = 10;
        this.offset =  0;
        this.totalPage = 0;
        this.lsKeySortColumn = "Task_"+this.state.taskType+"_SortData";
        this.selectedRows = [];
        this.subtaskTemplates = [];
        this.getObsTaskConfirmDlgContent= this.getObsTaskConfirmDlgContent.bind(this);
        this.markObsoleteTasks = this.markObsoleteTasks.bind(this);
        this.getTaskobsStatusContent =this.getTaskobsStatusContent.bind(this);  
        this.confirmDeleteTasks = this.confirmDeleteTasks.bind(this);
        this.confirmCancelTasks = this.confirmCancelTasks.bind(this);
        this.confirmObsoleteTasks = this.confirmObsoleteTasks.bind(this);
        this.onRowSelection = this.onRowSelection.bind(this);
        this.deleteTasks = this.deleteTasks.bind(this);
        this.cancelTasks = this.cancelTasks.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
        this.getTaskDeleteDialogContent = this.getTaskDeleteDialogContent.bind(this);
        this.getTaskCancelConfirmContent = this.getTaskCancelConfirmContent.bind(this);
        this.getCancelTaskStatusContent = this.getCancelTaskStatusContent.bind(this);
        this.changeTaskType = this.changeTaskType.bind(this);
        this.fetchTableData = this.fetchTableData.bind(this);
        this.getFilterOptions = this.getFilterOptions.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.handleData = this.handleData.bind(this);
        this.refreshTaskList = this.refreshTaskList.bind(this);
        this.getFormattedTaskDraft = this.getFormattedTaskDraft.bind(this);
        this.getFormattedTaskBlueprint = this.getFormattedTaskBlueprint.bind(this);
        this.getLinksList = this.getLinksList.bind(this);
        this.showTaskRestartConfirmation = this.showTaskRestartConfirmation.bind(this);
        this.getTaskRestartConfirmContent = this.getTaskRestartConfirmContent.bind(this);
        this.restartTask = this.restartTask.bind(this);
        this.getTaskRestartStatusContent = this.getTaskRestartStatusContent.bind(this);

    }

    subtaskComponent = (task) => {
        return (
            <button className="p-link" onClick={() => { this.setState({ showStatusLogs: true, task: task }) }}>
                <i className="fa fa-history"></i>
            </button>
        );
    };


    /**
     * Formatting the task_blueprints in blueprint view to pass to the ViewTable component
     * @param {Object} schedulingUnit - scheduling_unit_blueprint object from extended API call loaded with tasks(blueprint) along with their template and subtasks
     */
    getFormattedTaskBlueprints(tasks) {
        let taskBlueprintsList = [];
        for (const taskBlueprint of tasks) {           
            taskBlueprintsList.push(this.getFormattedTaskBlueprint(taskBlueprint));
        }
        return taskBlueprintsList;
    }

    getFormattedTaskBlueprint(taskBlueprint) {
        taskBlueprint['project'] = taskBlueprint.draft.scheduling_unit_draft.scheduling_set.project_id;
        taskBlueprint['scheduling_set_name'] = taskBlueprint.draft.scheduling_unit_draft.scheduling_set.name;
        taskBlueprint['status_logs'] = this.subtaskComponent(taskBlueprint);
        taskBlueprint['obsolete'] = taskBlueprint.obsolete_since!==null;
        taskBlueprint['tasktype'] = 'Blueprint';
        taskBlueprint['actionpath'] = '/task/view/blueprint/' + taskBlueprint['id'];
        taskBlueprint['blueprint_draft'] = this.getLinksList([taskBlueprint['draft_id']],'draft');
        taskBlueprint['relative_start_time'] = 0;
        taskBlueprint['relative_stop_time'] = 0;          
        taskBlueprint.duration = moment.utc((taskBlueprint.duration && taskBlueprint.duration>0?taskBlueprint.duration:0) * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
        taskBlueprint.schedulingUnitName = taskBlueprint.scheduling_unit_blueprint.name;
        for (const subtask of taskBlueprint.subtasks) {
            subtask.subTaskTemplate = _.find(this.subtaskTemplates, ['id', subtask.specifications_template_id]);
        }
        taskBlueprint.schedulingUnitId = taskBlueprint.scheduling_unit_blueprint_id;
        taskBlueprint.subTasks = taskBlueprint.subtasks;
        taskBlueprint['links'] = ['API URL'];
        taskBlueprint['linksURL'] = {
            'API URL': taskBlueprint.url
        }
        if (taskBlueprint['task_type']) {
            taskBlueprint['rowColor'] = 'task-'+taskBlueprint['task_type'];
        }
        return taskBlueprint;
    }

    /**
     * Formatting the task_drafts and task_blueprints in draft view to pass to the ViewTable component
     * @param {Object} schedulingUnit - scheduling_unit_draft object from extended API call loaded with tasks(draft & blueprint) along with their template and subtasks
     */
    async getFormattedTaskDrafts(tasks) {
        let scheduletasklist = [];
        // Common keys for Task and Blueprint
        //let commonkeys = ['id', 'created_at', 'description', 'short_description' , 'name', 'tags', 'updated_at', 'url', 'do_cancel', 'relative_start_time', 'relative_stop_time', 'on_sky_start_time', 'on_sky_stop_time', 'process_start_time', 'process_stop_time', 'duration', 'status',"obsolete"];
        for (const task of tasks) {
            let scheduletask = this.getFormattedTaskDraft(task);
            //Add Task Draft details to array
            scheduletasklist.push(scheduletask);
        }
        return scheduletasklist;
    }

    getFormattedTaskDraft(task) {
        // Common keys for Task and Blueprint
        let commonkeys = ['id', 'created_at', 'description', 'short_description' , 'name', 'tags', 'updated_at', 'url', 'do_cancel', 'relative_start_time', 'relative_stop_time', 'on_sky_start_time', 'on_sky_stop_time', 'process_start_time', 'process_stop_time', 'duration', 'status','obsolete', 'task_type'];
        let scheduletask = {};
        scheduletask['tasktype'] = 'Draft';
        scheduletask['actionpath'] = '/task/view/draft/' + task['id'];
        scheduletask['hasBluePrints'] = task.task_blueprints ? task.task_blueprints.length>0 : false;
        scheduletask['blueprint_draft'] = this.getLinksList(task['task_blueprints_ids'], 'blueprint');
        scheduletask['status'] = task['status'];   
        scheduletask['project'] = task.scheduling_unit_draft?task.scheduling_unit_draft.scheduling_set.project_id:'';
        scheduletask['scheduling_set_name'] = task.scheduling_unit_draft?task.scheduling_unit_draft.scheduling_set.name:'';
        
        //fetch task draft details
        for (const key of commonkeys) {
            scheduletask[key] = task[key];
        }
        if (scheduletask['task_type']) {
            scheduletask['rowColor'] = 'task-'+scheduletask['task_type'];
        }
        scheduletask.duration = moment.utc((scheduletask.duration&&scheduletask.duration>0?scheduletask.duration:0) * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
        scheduletask.relative_start_time = moment.utc(scheduletask.relative_start_time * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
        scheduletask.relative_stop_time = moment.utc(scheduletask.relative_stop_time * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
        scheduletask.produced_by = task.produced_by;
        scheduletask.produced_by_ids = task.produced_by_ids;
        scheduletask.schedulingUnitId = task.scheduling_unit_draft_id;
        scheduletask.schedulingUnitName = task.scheduling_unit_draft?.name; 
        scheduletask['links'] = ['API URL'];
        scheduletask['linksURL'] = {
            'API URL': task.url
        }
        return scheduletask;
    }

    async formatDataProduct(tasks) {
        await Promise.all(tasks.map(async task => {
            task.status_logs = task.tasktype === "Blueprint" ? this.subtaskComponent(task) : "";
            //Displaying SubTask ID of the 'control' Task
            const subTaskIds = task.subTasks ? task.subTasks.filter(sTask => sTask.primary === true) : [];
            // TODO:Update when aggregated fields for dataproducts, datasize in subtasks added in the backend.
            task.size = 0;
            task.dataSizeOnDisk = 0;
            task.noOfOutputProducts = 0;
            task.canSelect = task.tasktype.toLowerCase() === 'blueprint' ? true : !task.hasBluePrints;
            if (task.subTasks?.length) {
                task.noOfOutputProducts = _.sumBy(task.subTasks, 'total_nr_of_dataproducts');
                task.size = _.sumBy(task.subTasks, 'total_size_of_dataproducts')
                task.dataSizeOnDisk = _.sumBy(task.subTasks, 'total_size_of_dataproducts_on_filesysem')
                task.size = UnitConverter.getUIResourceUnit('bytes', (task.size));
                task.dataSizeOnDisk = UnitConverter.getUIResourceUnit('bytes', (task.dataSizeOnDisk));
            }
            task.subTaskID = subTaskIds.length ? subTaskIds[0].id : '';
            task.draft = task['blueprint_draft'];
            task['scheduling_unit_draft'] = task.schedulingUnitId;
            task['scheduling_unit_blueprint'] = task.schedulingUnitId;
            return task;
        }));
        return tasks;
    }

    async componentDidMount() {
        await this.getFilterColumns(this.state.taskType.toLowerCase());
        this.setToggleBySorting();
        const permission = await AuthUtil.getUserRolePermission();
        const {task, task_blueprint, task_draft} = permission.userRolePermission
        this.subtaskTemplates = await TaskService.getSubtaskTemplates();
        let actions = [];
        if(this.state.taskType === 'Draft'){
            actions = [
                        {icon: 'fa fa-trash', 
                         title: task_draft?.delete?'Delete Task(s)':`${this.access_denied_message} to delete Task(s)`,
                         disabled: task_draft?.delete? !task_draft?.delete: true,
                         type: 'button', actOn: 'click', props: { callback: this.confirmDeleteTasks }}
                        ];
        }
        else {
            actions = [ {icon: 'fa fa-times-circle', 
                            title: task.edit?'Mark as Obsolete': `${this.access_denied_message}`,
                            disabled: task.edit? !task.edit: true,
                            type: 'button', actOn: 'click', props: { callback: this.confirmObsoleteTasks }},
                        {icon: 'fa fa-ban', 
                            title: task_blueprint.canceltask?'Cancel Task(s)': `${this.access_denied_message} to cancel Task(s)`,
                            disabled: task_blueprint.canceltask? !task_blueprint.canceltask: true,
                            type: 'button', actOn: 'click', props: { callback: this.confirmCancelTasks }},
                        {icon: 'fa fa-trash', 
                            title: task_blueprint.delete?'Delete Task(s)':`${this.access_denied_message} to delete Task(s)`,
                            disabled: task_blueprint.delete? !task_blueprint.delete: true,
                            type: 'button', actOn: 'click', props: { callback: this.confirmDeleteTasks }},
                        {icon: 'fa fa-rotate', 
                            title: `Restart Task (s)`,
                            disabled: false,
                            type: 'button', actOn: 'click', props: { callback: this.showTaskRestartConfirmation }}
                        
                        ]
        }
        this.setState({ tasks: [], isLoading: false, actions: actions, loadingStatus: false, userrole: permission });
    }

    /**
     * Function to get the UI attributes from local storage that are stored in the previous view
     */
    getUIAttr() {
        this.taskUIAttr = UtilService.localStore({ type: 'get', key: 'TASK_LIST_UI_ATTR' }) || {};
    }

    /**
     * Function that stores the preferred values in local storage to restore the same in the next view
     * @param {string} key 
     * @param {Object} value 
     */
    storeUIAttr(key, value) {
        this.taskUIAttr[key] = value;
        UtilService.localStore({ type: 'set', key: 'TASK_LIST_UI_ATTR', value: this.taskUIAttr }); 
    }

    toggleBySorting = (sortData) => {
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: sortData });
    }
    setToggleBySorting() {     
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if (sortData) {
            if (Object.prototype.toString.call(sortData) === '[object Array]') {
                this.defaultSortColumn = sortData;
            }
            else {
                this.defaultSortColumn = [{ ...sortData }];
            }
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] });
    }

    /**
     * Prepare Task(s) details to show on confirmation dialog before deleting
     */
    getTaskDeleteDialogContent() {
        let selectedTasks = [];
        for (const obj of this.selectedRows) {
            selectedTasks.push({
                id: obj.id, suId: obj.schedulingUnitId, suName: obj.schedulingUnitName,
                taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
            });
        }
        return <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="taskId" header="Task Id"></Column>
                <Column field="taskName" header="Task Name"></Column>
                <Column field="status" header="Status"></Column>
            </DataTable>
    }

    confirmDeleteTasks() {
        this.pageUpdated = false;
        if (this.selectedRows.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', detail: 'Select Task to delete.' });
        } else {
            let dialog = {};
            dialog.type = "confirmation";
            dialog.header = "Confirm to Delete Task(s)";
            dialog.detail = "Do you want to delete the selected Task(s)?";
            dialog.content = this.getTaskDeleteDialogContent;
            dialog.actions = [{ id: 'yes', title: 'Delete Tasks', callback: this.deleteTasks, className:'act-btn-dispose' },
            { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
            dialog.onSubmit = this.deleteTasks;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({ dialog: dialog, dialogVisible: true });
        }
    }

    getTaskRestartConfirmContent() {
        const taskWithOnhold = this.state.selectedOnHoldRows;
        const taskWithoutOnhold = _.difference(this.selectedRows, taskWithOnhold);
        let selectedTaskWithOnhold = [];
        let selectedTasksWithoutOnhold = [];
        for (const task of taskWithOnhold) {  
            selectedTaskWithOnhold.push({
                    id: task.id, type: task.task_type, status: task.status
                });           
        }
        for (const task of taskWithoutOnhold) {           
            selectedTasksWithoutOnhold.push({
                    id: task.id, type: task.task_type, status: task.status
                });           
        }
        this.selectedRows = [];
        return             <div style={{marginTop: '1em'}}>
                {selectedTaskWithOnhold.length>0 &&
                    <>
                        <b>Task(s) that will be queued to restart</b>
                        <DataTable value={selectedTaskWithOnhold} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Task Id"></Column>
                            <Column field="type" header="Type"></Column>                  
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>} 
                {selectedTasksWithoutOnhold.length>0 &&
                    <>
                        <b>Task(s) with status other than &apos;on hold&apos; will be ignored</b>
                        <DataTable value={selectedTasksWithoutOnhold} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Task Id"></Column>
                            <Column field="type" header="Type"></Column>                  
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
                  
            </div>
    }

    
    showTaskRestartConfirmation() {
        this.pageUpdated = false;
        let selectedOnHoldRows = this.selectedRows.filter(task => {
            return task.status === 'on hold'});
        if(this.selectedRows.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Please select minimum one task to restart' });
        } else {
            if (selectedOnHoldRows.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Select at least one task with status on hold', 
                            detail: 'Selected task(s) cannot be restarted' });
            }
            else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header = "Confirm to restart Task(s)";
                dialog.actions = [{ id: 'yes', title: 'Restart', className:'act-btn-dispose', callback: this.restartTask },
                                    { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
                dialog.detail = "Below tasks will be queued to restart";
                dialog.content = this.getTaskRestartConfirmContent;
                dialog.submit = this.restartTask;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({ dialog: dialog, dialogVisible: true,  selectedOnHoldRows: selectedOnHoldRows });   
            }
                          
        }
    }

    async restartTask() {
        let tasks = this.state.tasks;
        let restartTasks = []
        for (const selectedTask of _.cloneDeep(this.state.selectedOnHoldRows)) {          
                const restartTaskResponse = await TaskService.restartTaskOnHold(selectedTask.id);
                let task = _.find(tasks, {'id': selectedTask.id});
                restartTasks.push({
                    id: task.id,
                    type: task.type, 
                    actionStatus:restartTaskResponse && restartTaskResponse.status === 200 ?'Success':'Failed'
                });            
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Restart Task (s) - Status";
        dialog.actions = [{ id: 'no', title: 'Close', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getTaskRestartStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ restartTasks: restartTasks, dialog: dialog, confirmDialogVisible: true });        
    }

    getTaskRestartStatusContent() {
        this.selectedRows = []; 
        return  <div style={{marginTop: '1em'}}>
                <DataTable value={this.state.restartTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="id" header="Task Id"></Column>    
                    <Column field="actionStatus" header="Action Status"></Column>
                </DataTable>
            </div>
            
    }
    /**
     * Delete Task(s)
     */
    async deleteTasks() {
        let hasError = false;
        for (const task of this.selectedRows) {
            if (!await TaskService.deleteTask(task.tasktype, task.id)) {
                hasError = true;
            }
        }
        if (hasError) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while deleting Task(s)' });
            this.setState({ dialogVisible: false });
        } else {
            this.selectedRows = [];
            this.setState({ dialogVisible: false });
            this.componentDidMount();
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Task(s) deleted successfully' });
            await this.fetchTableData(this.state.tableState);
        }
    }

    /**
     * Content for confirmation dialog before marking task as obsolete
     */
     getObsTaskConfirmDlgContent(){
        const blueprintsWithObsolete = this.state.blueprintsWithObsolete;
        const blueprintWithoutObsolete = _.difference(this.selectedRows, blueprintsWithObsolete);
        let selectedTasksWithObsolete = [];
        let selectedTasksWithoutObsolete = [];
        for (const obj of blueprintsWithObsolete) {           
                selectedTasksWithObsolete.push({
                    id: obj.id, suId: obj.schedulingUnitId, suName: obj.schedulingUnitName,
                    taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
                });       
        }

        for (const obj of blueprintWithoutObsolete) {           
            selectedTasksWithoutObsolete.push({
                id: obj.id, suId: obj.schedulingUnitId, suName: obj.schedulingUnitName,
                taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
            });       
        }

        return   <div style={{marginTop: '1em'}}>
                {selectedTasksWithoutObsolete.length>0 &&
                    <>
                        <b>Task(s) that can be marked as obsolete</b>
                        <DataTable value={selectedTasksWithoutObsolete} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="suId" header="Scheduling Unit Id"></Column>
                            <Column field="suName" header="Scheduling Unit Name"></Column>
                            <Column field="taskId" header="Task Id"></Column>
                            <Column field="controlId" header="Control Id"></Column>
                            <Column field="taskName" header="Task Name"></Column>
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
                {selectedTasksWithObsolete.length>0 &&
                    <>
                        <b>Task(s) already marked as obsolete will be ignored</b>
                        <DataTable value={selectedTasksWithObsolete} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="suId" header="Scheduling Unit Id"></Column>
                            <Column field="suName" header="Scheduling Unit Name"></Column>
                            <Column field="taskId" header="Task Id"></Column>
                            <Column field="controlId" header="Control Id"></Column>
                            <Column field="taskName" header="Task Name"></Column>
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
            </div>
    }

    /**
     * Prepare Task(s) details to show on confirmation dialog before cancelling
     */
    getTaskCancelConfirmContent() {
        let selectedTasks = [], ignoredTasks = [];
        for (const obj of this.selectedTaskRows) {
            if (this.TASK_END_STATUSES.indexOf(obj.status) < 0) {
                selectedTasks.push({
                    id: obj.id, suId: obj.schedulingUnitId, suName: obj.schedulingUnitName,
                    taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
                });
            }   else {
                ignoredTasks.push({
                    id: obj.id, suId: obj.schedulingUnitId, suName: obj.schedulingUnitName,
                    taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
                });
            }
        }
        return <>
            <div style={{marginTop: '1em'}}>
                <b>Task(s) that can be cancelled</b>
                <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>
                    <Column field="taskId" header="Task Id"></Column>
                    <Column field="controlId" header="Control Id"></Column>
                    <Column field="taskName" header="Task Name"></Column>
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>
            {ignoredTasks.length > 0 &&
            <div style={{marginTop: '1em'}}>
                <b>Task(s) that will be ignored</b>
                <DataTable value={ignoredTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>
                    <Column field="taskId" header="Task Id"></Column>
                    <Column field="controlId" header="Control Id"></Column>
                    <Column field="taskName" header="Task Name"></Column>
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>
            }
        </>
    }

    /**
     * Prepare Task(s) details to show status of Task cancellationn
     */
     getCancelTaskStatusContent() {
        let cancelledTasks = this.state.cancelledTasks;
        return  <DataTable value={cancelledTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="suName" header="Scheduling Unit Name"></Column>
                <Column field="taskId" header="Task Id"></Column>
                <Column field="controlId" header="Control Id"></Column>
                <Column field="taskName" header="Task Name"></Column>
                <Column field="status" header="Status"></Column>
            </DataTable>
    }
    /**
     * Content to show status after marking task as obsolete
     */
    getTaskobsStatusContent() {
        let obsoleteTasks = this.state.obsoleteTasks;
        return     <DataTable value={obsoleteTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="suName" header="Scheduling Unit Name"></Column>
                <Column field="taskId" header="Task Id"></Column>
                <Column field="controlId" header="Control Id"></Column>
                <Column field="taskName" header="Task Name"></Column>
                <Column field="status" header="Status"></Column>
                <Column field='actionstatus' header= "Action Status"></Column>
            </DataTable>
    }

    /**
     * Function to get confirmation before cancelling all selected task blueprints if the task status is 
     * not one of the end statuses. If no selected task is cancellable, show info to select a cancellable task.
     * 
     */
    confirmCancelTasks() {
        this.pageUpdated = false;
        let selectedBlueprints = this.selectedRows.filter(task => {
                                        return task.tasktype === 'Blueprint' && 
                                        this.TASK_END_STATUSES.indexOf(task.status)<0});
        if (this.selectedRows.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Please select minimum one task to cancel' });
        } else {
            if(selectedBlueprints.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Already Cancelled', 
                            detail: 'Selected task(s) are already cancelled' });
            }
            else {
                if(!this.state.dialogVisible) {
                    this.selectedTaskRows = _.cloneDeep(this.selectedRows);
                }
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header = "Confirm to Cancel Task(s)";
                dialog.actions = [{ id: 'yes', title: 'Cancel Tasks', callback: this.cancelTasks, className:'act-btn-dispose' },
                                    { id: 'no', title: 'No', className:'act-btn-cancel', callback: this.closeDialog }];
                dialog.detail = "Warning: Cancelling the task means, it will no longer be executed / will be aborted. This action cannot be undone. Already finished/cancelled task(s) will be ignored. Do you want to proceed?";
                dialog.content = this.getTaskCancelConfirmContent;
                dialog.submit = this.cancelTasks;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({ dialog: dialog, dialogVisible: true });
            }
            
        }
    }

    /**
     * Function to show confirmation dialog before marking task as obsolete
     */
    confirmObsoleteTasks() {
        this.pageUpdated = false;
        let selectedBlueprints = this.selectedRows.filter(task => {
                                        return task.tasktype === 'Blueprint'
                                        });
        const  blueprintsWithObsolete = this.checkObsolete(selectedBlueprints)
        const blueprintWithoutObsolete = _.difference(selectedBlueprints, blueprintsWithObsolete);
        if (selectedBlueprints.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Please select minimum one task to mark as obsolete' });
        } else {
            if (blueprintWithoutObsolete.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Already marked as obsolete', 
                            detail: 'Selected task(s) are already marked as obsolete' });
            }
            else {
            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header = "Confirm to mark task(s) as obsolete";
            dialog.actions = [{ id: 'yes', title: 'Mark as obsolete', className:'act-btn-dispose', callback: this.markObsoleteTasks },
                                { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
            dialog.detail = "Warning: Obsolete task(s) will not be copied and dataproducts will not be used for pipelines and ingest. Are you sure you want to make the task(s) obsolete?";
            dialog.content = this.getObsTaskConfirmDlgContent;
            dialog.submit = this.markObsoleteTasks;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({ dialog: dialog, dialogVisible: true, blueprintsWithObsolete: blueprintsWithObsolete});
            }
        }
    }

    /**
     * Function to cancel all selected task blueprints if the task status is not one of the end statuses
     * and update their status on successful cancellation.
     */
    async cancelTasks() {
        let tasks = this.state.tasks;
        let selectedBlueprints = this.selectedTaskRows.filter(task => {return task.tasktype === 'Blueprint'});
        let cancelledTasks = []
        for (const selectedTask of selectedBlueprints) {
            if (this.TASK_END_STATUSES.indexOf(selectedTask.status) < 0) {
                const cancelledTask = await TaskService.cancelTask(selectedTask.id);
                let task = _.find(tasks, {'id': selectedTask.id, tasktype: 'Blueprint'});
                if (cancelledTask) {
                    task.status = cancelledTask.status;
                }
                cancelledTasks.push({
                    id: task.id, suId: task.schedulingUnitId, suName: task.schedulingUnitName,
                    taskId: task.id, controlId: task.subTaskID, taskName: task.name, 
                    status: task.status.toLowerCase()==='cancelled'?'Cancelled': 'Error Occured'
                });
            }
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Cancel Task(s) Status";
        dialog.actions = [{ id: 'no', title: 'Close', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getCancelTaskStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ tasks: tasks, cancelledTasks: cancelledTasks, dialog: dialog, dialogVisible: true });
    }
    /**
     * Function to show status after marking task as osolete
     */
    async markObsoleteTasks() {
        let tasks = this.state.tasks;
        const blueprintWithoutObsolete = _.difference(this.selectedRows, this.state.blueprintsWithObsolete);
        let obsoleteTasks = []
        for (const selectedTask of blueprintWithoutObsolete) {       
                const obsTask = await TaskService.markTaskObsolete(selectedTask.id);
                let task = _.find(tasks, {'id': selectedTask.id, tasktype: 'Blueprint'});     
                if (obsTask) {
                    task.status = obsTask.status;
                    task.obsolete_since = obsTask.obsolete_since ;
                    task.obsolete = obsTask.obsolete_since!==null;
                }               
                obsoleteTasks.push({
                    id: task.id, suId: task.schedulingUnitId, suName: task.schedulingUnitName,
                    taskId: task.id, controlId: task.subTaskID, taskName: task.name, 
                    status: task.status.toLowerCase()==='obsolete'?'Obsolete':  task.status,
                    actionstatus: obsTask?'Success': 'Failed'
                });
         
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Mark task(s) as obsolete - Status";
        dialog.actions = [{ id: 'no', title: 'Ok', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getTaskobsStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ tasks: tasks, obsoleteTasks: obsoleteTasks, dialog: dialog, dialogVisible: true });
    }
    /**
     * Function to check if task is marked as obsolete or not
     * @param {Array} selectedBlueprints 
     * @returns Array - List of tasks which are marked as obsolete
     */
    checkObsolete(selectedBlueprints) {
        const blueprintsWithObsolete = selectedBlueprints.filter(task => {
            return task.obsolete_since !== null}); 
        return blueprintsWithObsolete;
    }

    /**
     * Callback function to close the dialog prompted.
     */
    closeDialog() {
        this.setState({ dialogVisible: false, cancelledTasks: [], obsoleteTasks:[] });
    }

    onRowSelection(selectedRows) {
        this.selectedRows = selectedRows;
    }

    /**
     * Change Task type in UI
     * @param {String} type 
     */
    async changeTaskType(type) {
        this.filterQry = '';
        this.orderBy = '';
        this.pageUpdated = true;
        this.lsKeySortColumn = "Task_"+type+"_SortData";
        this.setToggleBySorting();
        this.setState({taskType: type, tasks: [], loadingStatus:true});
       
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if (sortData) {
            let columnDetails = _.find(this.state.columnMap, {displayName:sortData[0]?sortData[0].id:sortData.id});
            if(columnDetails) {
                this.orderBy = 'ordering='+((sortData[0]?sortData[0].desc:sortData.desc)?'-':'')+columnDetails.orgField;
            }
        }
        this.storeUIAttr('listType', type)
    }

    /**
     * Get filter columns form API
     * @param {String} type 
     */
    async getFilterColumns(type) {
        const taskFilters = await TaskService.getTaskFilterDefinition(type);

        const columnOrderToRemove = this.state.taskType === 'Blueprint' ?[] :['Status', 'Unschedulable Reason','Status Logs', 'Observation Start Time', 'Observation End Time', 'Process Start Time', 'Process End Time', 'Control ID','#Dataproducts', 'Data Size [TB]', 'Data Size on Disk [TB]', 'Subtask Content', 'Obsolete'];
        const columnDefinitionToRemove = this.state.taskType === 'Blueprint' ?['scheduling_unit_draft']:['scheduling_unit_blueprint', 'status', 'unschedulable_reason','status_logs', 'on_sky_start_time', 'on_sky_stop_time', 'process_start_time', 'process_stop_time','subTaskID','noOfOutputProducts','size','dataSizeOnDisk','subtaskContent', 'obsolete', 'task_type'];
        this.columnMap = [];
        let tmpDefaulColumns = _.cloneDeep(this.state.defaultcolumns[0]);
        let tmpOptionalColumns = _.cloneDeep(this.state.optionalcolumns[0]);
        let tmpColumnOrders = _.cloneDeep(this.state.columnOrders);
        if (this.state.taskType === 'Blueprint') {
            tmpOptionalColumns['draft'].name = 'Linked Draft ID';
            let index = tmpColumnOrders.indexOf('Linked Blueprint ID') ;
            if (index !== -1) {
                tmpColumnOrders[index] = 'Linked Draft ID';
            }            
        }   else {
            tmpOptionalColumns['draft'].name = 'Linked Blueprint ID';
            let index = tmpColumnOrders.indexOf('Linked Draft ID') ;
            if (index !== -1) {
                tmpColumnOrders[index] = 'Linked Blueprint ID';
            }
        }
        tmpColumnOrders = _.differenceWith(tmpColumnOrders, columnOrderToRemove, _.isEqual);
         this.removeColumns(tmpDefaulColumns,columnDefinitionToRemove);
         this.removeColumns(tmpOptionalColumns,columnDefinitionToRemove);
        if(taskFilters) {
            this.getStatusList(taskFilters);
            this.getBlueprintTaskTypeList(taskFilters);
            tmpDefaulColumns = this.getAPIFilter(taskFilters, tmpDefaulColumns);
            tmpOptionalColumns = this.getAPIFilter(taskFilters, tmpOptionalColumns);
            this.setState({tmpDefaulcolumns: [tmpDefaulColumns], tmpOptionalcolumns:[tmpOptionalColumns], tmpColumnOrders: tmpColumnOrders, columnMap: this.columnMap})
        }
    }

    removeColumns(arrayValue, keys) {
        keys.forEach(key => {
            delete arrayValue[key];
        });
    }

    /**
     * Get Status list for filter
     * @param {Array} suFilters 
     */
    getStatusList(taskFilters) {
        if (taskFilters?.data?.filters?.['status']) {
            this.statusList = [];
            taskFilters.data.filters['status'].choices.forEach(choice => {
                const tmpVar = {name:  _.startCase(choice.value), value: choice.value};
                this.statusList.push(tmpVar);
            });
        }
    }

    /**
     * Get Task type list for filter
     * @param {Array} suFilters 
     */
    getBlueprintTaskTypeList(taskFilters) {
        if (taskFilters?.data?.filters?.['task_type']) {
            this.blueprintTaskTypeList = [];
            taskFilters.data.filters['task_type'].choices.forEach(choice => {
                const tmpVar = {name:  _.startCase(choice.value), value: choice.value};
                this.blueprintTaskTypeList.push(tmpVar);
            });
        }
    }

    getAPIFilter(taskFilters, columnDef) {
        const defaultColKeys = Object.keys(columnDef);
        defaultColKeys.forEach(key => {
            let tmpColMap = {};
            let tempKey = key;
            tmpColMap['orgField'] = tempKey;
            tmpColMap['tmpField'] = tempKey;
            if(columnDef[key]) {
                tmpColMap['displayName'] = columnDef[key]['name'];
            }
            this.columnMap.push(tmpColMap);
            //Set Enable/Disable the Filter & SortBy in each column
            if(taskFilters?.data?.filters?.[tempKey]) {
                columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, tempKey);
                columnDef[key]['disableFilters'] = false;
                if(UIConstants.FILTER_MAP[taskFilters.data.filters[tempKey].type]) {
                
                    columnDef[key]['filter'] = UIConstants.FILTER_MAP[taskFilters.data.filters[tempKey].type];
                }
            }   else if (key === 'subTaskID' && taskFilters.data.filters['subtasks']) {
                    columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, 'subtasks');
                    columnDef[key]['disableFilters'] = false;
            }   else if (tempKey === 'schedulingUnitName' && 
                    (taskFilters?.data?.filters?.['scheduling_unit_draft_name'] || taskFilters?.data?.filters?.['scheduling_unit_blueprint_name'])) {
                if (this.state.taskType === 'Draft') {
                    columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, 'scheduling_unit_draft_name');
                    columnDef[key]['disableFilters'] = false;
                }   else {
                    columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, 'scheduling_unit_blueprint_name');
                    columnDef[key]['disableFilters'] = false;
                }
            }   else if (key === 'duration' && taskFilters?.data?.filters?.['duration_min']) {
                columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, 'duration_min');
                columnDef[key]['disableFilters'] = false;
            }   else if (key === 'relative_start_time' && taskFilters?.data?.filters?.['relative_start_time_min']) {
                columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, 'relative_start_time_min');
                columnDef[key]['disableFilters'] = false;
            }   else if (key === 'relative_stop_time' && taskFilters?.data?.filters?.['relative_stop_time_min']) {
                columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, 'relative_stop_time_min');
                columnDef[key]['disableFilters'] = false;
            }   else if (key === 'draft' && taskFilters?.data?.filters?.['task_blueprints']) {
                columnDef[key]['disableSortBy'] = !_.includes(taskFilters.data.ordering, 'task_blueprints');
                columnDef[key]['disableFilters'] = false;
            }   else if (columnDef[key]['name']) {
                columnDef[key]['disableSortBy'] = true;
                columnDef[key]['disableFilters'] = true;
            }
        });
        return columnDef;
    }

    getLinksList = (linkedItems, type) => {
        return (
            <>
                {linkedItems && linkedItems.length>0 && linkedItems.map((item) => (
                    <Link key={item} style={{paddingRight: '3px'}} to={`/task/view/${type}/${item}`}>{item}</Link>
                ))}
            </>
        );                    
    }
    
    /**
     * 
     * @param {String} taskType : Task type Blueprint/Draft
     * @param {String} filterQry : Filter query
     * @param {String} orderBy : Order by cilumn name
     * @param {String} limit : Rows limit per request
     * @param {String} offset : Offset value
     */
    async getTaskLists(taskType, filterQry, orderBy, limit, offset) {
        let expand = taskType.toLowerCase() === 'draft' ? this.TASK_DRAFT_EXPAND: this.TASK_BLUEPRINT_EXPAND;
        let fields = taskType.toLowerCase() === 'draft' ? this.TASK_DRAFT_FIELDS: this.TASK_BLUEPRINT_FIELDS;
        let response = await TaskService.getExpandedTasksWithFilter(taskType.toLowerCase(), expand, fields.join(","), filterQry, orderBy, limit, offset);
        const {task_draft, task_blueprint} = this.state.userrole.userRolePermission
        if (response?.data) {
            this.totalPage = response.data.count;
            let tasks = taskType.toLowerCase() === 'draft' ? (await this.getFormattedTaskDrafts(response.data.results)) : await this.getFormattedTaskBlueprints(response.data.results);
            tasks = await this.formatDataProduct(tasks);
            let actions = [];
            let projectTask = _.uniqBy(_.map(tasks, function(task) {
                                        return {
                                            project: task.project,
                                            id: task.id
                                        }
                                    }),'project');
            let taskPermissions = {
                cancel : true,
                obsolete : true,
                delete : true
            }
            for (const task of projectTask) {
                let taskWithProjectPermission = await AuthUtil.getUserPermissionByModuleId(taskType.toLowerCase() === 'draft'? 'task_draft' : 'task_blueprint', task.id);
                if(!taskWithProjectPermission[task.id].cancel) {
                    taskPermissions.cancel = false;
                }
                if(!taskWithProjectPermission[task.id].obsolete) {
                    taskPermissions.obsolete = false;
                }
                if((!taskWithProjectPermission[task.id].delete)) {
                    taskPermissions.delete = false;
                }
            }
            this.setState({taskPermissions : taskPermissions})
            if(task_blueprint && task_draft){
            if(taskType === 'Draft'){
                actions = [
                            {icon: 'fa fa-trash', 
                                title: this.state.taskPermissions?.delete?'Delete Task(s)':`${this.access_denied_message} to delete Task(s)`,
                                disabled: !this.state.taskPermissions?.delete,
                                type: 'button', actOn: 'click', props: { callback: this.confirmDeleteTasks }}
                            ];
            }
            else {
                actions = [ {icon: 'fa fa-sync',
                            title: `Restart Task(s)`,
                            disabled: false,
                            type: 'button', actOn: 'click', props: { callback: this.showTaskRestartConfirmation }},
                            {icon: 'fa fa-ban', 
                            title: this.state.taskPermissions?.cancel?'Cancel Task(s)': `${this.access_denied_message} to cancel Task(s)`,
                            disabled: this.state.taskPermissions?.cancel? !this.state.taskPermissions?.cancel: true,
                            type: 'button', actOn: 'click', props: { callback: this.confirmCancelTasks }},
                            {icon: 'fa-times-circle', iconType:'far',
                                title: this.state.taskPermissions?.obsolete?'Mark as Obsolete': `${this.access_denied_message} to mark obsolete`,
                                disabled: this.state.taskPermissions?.obsolete? !this.state.taskPermissions?.obsolete: true,
                                type: 'button', actOn: 'click', props: { callback: this.confirmObsoleteTasks }},
                            {icon: 'fa fa-trash', 
                                title: this.state.taskPermissions?.delete?'Delete Task(s)':`${this.access_denied_message} to delete Task(s)`,
                                disabled: this.state.taskPermissions?.delete? !this.state.taskPermissions?.delete: true,
                                type: 'button', actOn: 'click', props: { callback: this.confirmDeleteTasks }},
                            ];
            }
        }
            this.setState({ tasks: tasks, isLoading: false, actions: actions, loadingStatus: false });
        }   else {
            appGrowl?.show({severity: 'error', summary: 'Error', detail: 'Unable to fetch records'});
        }
    }


    /**
     * Fetch data based on request
     * @param {state} state 
     * @returns 
     */
    async fetchTableData(state) {
        this.filterQry = '';
        this.orderBy = '';
        this.pageUpdated = true;
        this.setState({loadingStatus:true, tableState: state});
        await this.getFilterColumns(this.state.taskType.toLowerCase());
        let filters = UtilService.localStore({ type: 'get', key: "su_task_list_"+this.state.taskType });
        if(filters.length > 0 ) {
            this.idFilterValues = [];
            for( const filter of filters) {
                if (filter.id === 'Observation Start Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'on_sky_start_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'on_sky_start_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }   else if (filter.id === 'Observation End Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'on_sky_stop_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'on_sky_stop_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }  else if (filter.id === 'Process Start Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'process_start_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'process_start_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }   else if (filter.id === 'Process End Time') {
                    const values = filter.value;
                    if (values[0]) {
                        this.filterQry += 'process_stop_time_after='+ moment(new Date(values[0])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                    if (values[1]) {
                        this.filterQry += 'process_stop_time_before='+moment(new Date(values[1])).format("YYYY-MM-DDTHH:mm:ss")+".000Z&";
                    }
                }   else if (filter.id === 'Scheduling Unit Name') {
                    if (this.state.taskType === 'Draft') {
                        this.filterQry += 'scheduling_unit_draft_name='+encodeURIComponent(filter.value)+'&';
                    }   else {
                        this.filterQry += 'scheduling_unit_blueprint_name='+encodeURIComponent(filter.value)+'&';
                    }
                }   else if ((filter.id === 'Duration (HH:mm:ss)' || filter.id === 'Relative Start Time (HH:mm:ss)' || filter.id === 'Relative End Time (HH:mm:ss)') && filter.value !== '') {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    const values = _.split(filter.value, ",");
                    if (values[0].includes(":")) {
                        this.filterQry += columnDetails.orgField+"_min=PT"+UnitConverter.getHHmmssToSecs(values[0])+"S&";
                    }
                    if (values[1].includes(":")) {
                        this.filterQry += columnDetails.orgField+"_max=PT"+UnitConverter.getHHmmssToSecs(values[1])+"S&";
                    }
                }   else if (filter.id === 'Linked Blueprint ID' && filter.value !== '') {
                    const values = UnitConverter.getSubbandOutput(filter.value)
                    _.split(values, ",").forEach(value =>{
                        if(value && _.trim(value) !== '') {
                        this.filterQry += 'task_blueprints='+_.trim(value)+'&';
                        }
                    });
                }   else if ((filter.id === 'Created at' || filter.id === 'Updated at') && filter.value !== '') {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    const values = _.split(filter.value, ",");
                    if (values.length>2){
                        continue;
                    }
                    if((values[0] && values[0] !== '' && values[0] !== 'null') && (values[1] && values[1] !== '' && values[1] !== 'null')) {
                        this.filterQry += columnDetails.orgField+'_after='+ moment(new Date(values[0])).format("YYYY-MM-DDT00:00:00")+".000Z&";
                        this.filterQry += columnDetails.orgField+'_before='+moment(new Date(values[1])).format("YYYY-MM-DDT23:59:59")+".000Z&";
                    }
                }   else if ((filter.id === 'Control ID') && filter.value !== '') {
                    const values = UnitConverter.getSubbandOutput(filter.value);
                    _.split(values, ",").forEach(value =>{
                        if(value && _.trim(value) !== '') {
                        this.filterQry += 'subtasks='+_.trim(value)+'&';
                        }
                    });
                }   else if ((filter.id === 'Scheduling Unit ID' || filter.id === 'Linked Draft ID'|| filter.id === 'ID') && filter.value !== '') {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    const values = UnitConverter.getSubbandOutput(filter.value);
                    this.idFilterValues = filter.id === 'ID' ?values:[];
                    this.filterQry +=  columnDetails.orgField+'='+values.toString()+"&";
                }   else if (filter.id.toLowerCase() === 'status') {
                    for ( const value of filter.value) {
                        this.filterQry += 'status='+value+"&";
                    }
                }   else if (filter.id === 'Task Type') {
                    for ( const value of filter.value) {
                        this.filterQry += 'task_type='+value+"&";
                    }
                }    else {
                    let columnDetails = _.find(this.state.columnMap, {displayName:filter.id});
                    if(columnDetails) {
                        this.filterQry += columnDetails.orgField +'='+_.trim(encodeURIComponent(filter.value))+'&'
                    }
                }
            }
        }
        let sortBy = this.state.tableState.sortBy?this.state.tableState.sortBy[0]:null;
        if (sortBy) {
            this.setState({defaultSortColumn: [sortBy]}); 
            UtilService.localStore({ type: 'set', key: this.lsKeySortColumn ,value:sortBy});
            let columnDetails = _.find(this.state.columnMap, {displayName:sortBy.id});
            if(columnDetails) {
                this.orderBy = 'ordering='+((sortBy.desc)?'-':'')+columnDetails.orgField;
            }
        }
        this.filterQry = this.filterQry.substring(0,this.filterQry.length-1);
        await this.getTaskLists(this.state.taskType, this.filterQry, this.orderBy, this.state.tableState.pageSize, (this.state.tableState.pageIndex*this.state.tableState.pageSize));
        return [this.state.tasks, this.totalPage]; 
     }

     /**
     * Get Option-list values for Select Dropdown filter in 'Viewtable'
     * @param {String} id : Column id
     * @returns 
     */
    getFilterOptions(id) {
        let options = null;
        if(id && id === 'Status') {
            options = this.statusList;
        }   else if(id && id === 'Task Type') {
            options = this.blueprintTaskTypeList;
        }
        return options;
    }

    /**
     * Function to call wnen websocket is connected
     */
    onConnect() {
        try{
            const userDets = localStorage.getItem("user");
            if (userDets) {
                this.websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }



    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    handleData(data) {
        if (data) {
            console.log('received websocket data:', data)
            const jsonData = JSON.parse(data);
            this.refreshTaskList(jsonData); 
        }
    }

    /**
     * Refresh the Task list
     * @param {*} jsonData 
     */
    async refreshTaskList(jsonData) {
        switch (jsonData.object_type) {
            case 'task_draft': {
                if (this.state.taskType === 'Draft') {
                    let taskList = this.state.tasks;
                    if (jsonData.action === 'delete') {
                        _.remove(taskList, function (task) { return task.id === jsonData.object_details.id});
                    }   else {
                        let isSearchable = false;
                        let filterQry = '';
                        if (this.idFilterValues.length === 0) {
                            filterQry = 'id='+jsonData.object_details.id;
                            filterQry += this.filterQry !== ''? '&'+this.filterQry:'';
                            isSearchable = true;                        
                        }   else if (_.includes(this.idFilterValues, jsonData.object_details.id)) {
                            let idFilter = 'id='+this.idFilterValues.toString();
                            filterQry = _.replace(this.filterQry, idFilter, ('id='+jsonData.object_details.id));
                            isSearchable = true;
                        }                    
                        if (isSearchable) {
                            let updatedTask = {};
                            TaskService.getExpandedTasksWithFilter('draft', this.TASK_DRAFT_EXPAND, this.TASK_DRAFT_FIELDS.join(","), filterQry, this.orderBy, this.state.tableState.pageSize, (this.state.tableState.pageIndex*this.state.tableState.pageSize))
                            .then(async(response) => {
                                if (response.data.count>0) {
                                    updatedTask = this.getFormattedTaskDraft(response.data.results[0]);
                                    if (jsonData.action === 'create') {
                                        taskList.push(updatedTask);
                                        this.totalPage++;
                                    }   else if (jsonData.action === 'update') {
                                        _.remove(taskList, function (su) { return su.id === updatedTask.id });
                                        taskList.push(updatedTask);
                                    }
                                }
                                this.setState({tasks: taskList});
                            });
                        }
                    }
                    this.setState({ tasks: taskList, isLoading: false, loadingStatus: false });
                }
                break;
            }
            case 'task_blueprint': {
                if (this.state.taskType === 'Blueprint') {
                    let taskList = this.state.tasks;
                    let isSearchable = false;
                    let filterQry = '';
                    if (this.idFilterValues.length === 0) {
                        filterQry = 'id='+jsonData.object_details.id;
                        filterQry += this.filterQry !== ''? '&'+this.filterQry:'';
                        isSearchable = true;                        
                    }   else if (_.includes(this.idFilterValues, jsonData.object_details.id)) {
                        let idFilter = 'id='+this.idFilterValues.toString();
                        filterQry = _.replace(this.filterQry, idFilter, ('id='+jsonData.object_details.id));
                        isSearchable = true;
                    }                    
                    if (isSearchable) {
                        let updatedTask = {};
                        TaskService.getExpandedTasksWithFilter('blueprint', this.TASK_BLUEPRINT_EXPAND, this.TASK_BLUEPRINT_FIELDS.join(","), filterQry, this.orderBy, this.state.tableState.pageSize, (this.state.tableState.pageIndex*this.state.tableState.pageSize))
                        .then(async(response) => {
                            if (response.data.count>0) {
                                updatedTask = await this.getFormattedTaskBlueprint(response.data.results[0]);
                                if (jsonData.action === 'create') {
                                    taskList.push(updatedTask);
                                    this.totalPage++;
                                }   else if (jsonData.action === 'update') {
                                    _.remove(taskList, function (su) { return su.id === updatedTask.id });
                                    taskList.push(updatedTask);
                                }
                                taskList = await this.formatDataProduct(taskList);
                                let projectTask = _.uniqBy(_.map(taskList, function(task) {
                                                            return {
                                                                project: task.project,
                                                                id: task.id
                                                            }
                                                        }),'project');
                                let taskPermissions = {
                                    cancel : true,
                                    obsolete : true,
                                    delete : true
                                }
                                for (const task of projectTask) {
                                    let taskWithProjectPermission = await AuthUtil.getUserPermissionByModuleId(this.state.taskType.toLowerCase() === 'draft'? 'task_draft' : 'task_blueprint', task.id);
                                    if(!taskWithProjectPermission[task.id].cancel) {
                                        taskPermissions.cancel = false;
                                    }
                                    if(!taskWithProjectPermission[task.id].obsolete) {
                                        taskPermissions.obsolete = false;
                                    }
                                    if((!taskWithProjectPermission[task.id].delete)) {
                                        taskPermissions.delete = false;
                                    }
                                }
                                this.setState({taskPermissions : taskPermissions});
                            }
                        });
                        this.setState({tasks: taskList}); 
                    }
                    this.setState({ tasks: taskList, isLoading: false, loadingStatus: false });
                }
                break;
            }
            default: { break; }
        }
    }

    render() {
        if (this.state.redirect) {
            return <Redirect to={{ pathname: this.state.redirect }}></Redirect>
        }
        const {task} = this.state.userrole.userRolePermission

        return (
            <React.Fragment>
                {task &&
                <>
                    {task.list?
                    <>
                        <PageHeader location={this.props.location} title={'Task - List'} 
                            actions={this.state.actions}/>
                        {this.state.isLoading ? <AppLoader /> :
                        <>
                            <div className="p-select " style={{position: 'relative'}}>
                                <div className="p-field p-grid" style={{position: 'absolute', marginLeft: '-11em', top: '-2em'}}>
                                    <label>Select Task Blueprint/Draft</label>
                                        <span className="p-float-label">
                                            <Dropdown inputId="tasktype" optionLabel="name" optionValue="name"  data-testid="tasktype-dropdown" 
                                                tooltip="Task Type" tooltipOptions={this.tooltipOptions}
                                                value={this.state.taskType}
                                                options={this.taskTypeList} 
                                                onChange={(e) => {this.changeTaskType( e.value)}} 
                                                style={{width: '10em', marginLeft: '0.5em'}}
                                                />
                                        </span>
                                </div>
                            </div>
                            <ViewTable
                                data={this.state.tasks}
                                totalPage={this.totalPage}
                                defaultcolumns={this.state.tmpDefaulcolumns ? this.state.tmpDefaulcolumns : this.state.defaultcolumns} 
                                optionalcolumns={this.state.tmpOptionalcolumns ? this.state.tmpOptionalcolumns : this.state.optionalcolumns}
                                columnclassname={this.state.columnclassname}
                                // columnOrders={this.state.tmpColumnOrders}    //Changed to avoid column shuffling 
                                columnOrders={this.state.columnOrders}
                                defaultSortColumn={this.defaultSortColumn}
                                showFilterOption={this.getFilterOptions} //Callback function to provide inputs for option-list in Select Dropdown filter
                                showaction={true}
                                keyaccessor="id"
                                paths={this.state.paths}
                                unittest={this.state.unittest}
                                tablename={"su_task_list_"+this.state.taskType}
                                allowRowSelection={true}
                                onRowSelection={this.onRowSelection}
                                lsKeySortColumn={this.lsKeySortColumn}
                                toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                                ignoreSorting={this.ignoreSorting}
                                pageUpdated={this.pageUpdated}
                                callBackFunction={this.fetchTableData} 
                                loadingStatus={this.state.loadingStatus}
                                showGlobalFilter={false} showTopPagination={true}
                                storeFilter={true}
                            />
                        </>
                        }
                        {this.state.showStatusLogs &&
                            <Dialog header={`Status change logs - ${this.state.task ? this.state.task.name : ""}`}
                                visible={this.state.showStatusLogs} maximizable maximized={false} position="left" style={{ width: '50vw' }}
                                onHide={() => { this.setState({ showStatusLogs: false }) }}
                                className="content_dlg">
                                <TaskStatusLogs taskId={this.state.task.id}></TaskStatusLogs>
                            </Dialog>
                        }
                        <CustomDialog type="confirmation" visible={this.state.dialogVisible}
                            header={this.state.dialog.header} message={this.state.dialog.detail} actions={this.state.dialog.actions}
                            content={this.state.dialog.content} width={this.state.dialog.width} showIcon={this.state.dialog.showIcon}
                            onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.state.dialog.onSubmit} />
                        {!this.state.isLoading &&
                        <Websocket ref={websocket => this.websocket = websocket} url={process.env.REACT_APP_WEBSOCKET_URL} 
                            onOpen={this.onConnect} onMessage={this.handleData} />}
                    </>:<AccessDenied/>}
                </>}
            </React.Fragment>
        );
    }
}

TaskList.propTypes = {
  location: PropTypes.oneOfType([PropTypes.object, PropTypes.string])
}
