import PropTypes from 'prop-types';
import React, { Component } from 'react';
import { Link, Redirect } from 'react-router-dom'
import moment from 'moment';
import _ from 'lodash';
import { Checkbox } from 'primereact/checkbox';
import Jeditor from '../../components/JSONEditor/JEditor';
import TaskService from '../../services/task.service';
import UIConstants from '../../utils/ui.constants';
import { Dialog } from 'primereact/dialog';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { appGrowl } from '../../layout/components/AppGrowl';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import TaskStatusLogs from './state_logs';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import UtilService from '../../services/util.service';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import ViewTable from '../../components/ViewTable/ViewTable';
import Websocket from 'react-websocket';

export class TaskView extends Component {
    TASK_NOT_STARTED_STATUSES = ['defined', 'schedulable', 'scheduled'];
    TASK_ACTIVE_STATUSES = ['queued', 'started', 'observing', 'observed', 'processing', 'processed', 'ingesting'];
    TASK_END_STATUSES = ['finished', 'error', 'cancelled'];
    
    constructor(props) {
        super(props);
        this.state = {
            isLoading: true,
            confirmDialogVisible: false,
            hasBlueprint: true,
            dialog: {},
            userrole: {
                userRolePermission: {}
            },
            actions: [],
            permissionById: {},
            subtaskRowList: [],
            defaultcolumns: [{
                type: {
                    name:"Subtask Type",
                    filter:"select",
                    tooltip: 'Select Subtask type'
                },
                id: {
                    name: "Subtask ID",
                    tooltip: 'Enter few characters'
                },
                status: {
                    name: "Subtask Status",
                    filter:"select",
                    tooltip: 'Select Subtask status'
                },
                on_sky_start_time: {
                    name: "Start Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT,
                },
                on_sky_stop_time: {
                    name: "End Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                process_start_time: {
                    name: "Process Start Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT,
                },
                process_stop_time: {
                    name: "Process End Time",
                    filter: "flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                duration:{
                    name:"Duration (HH:mm:ss)",
                    filter: "durationMinMax",
                    format:UIConstants.CALENDAR_TIME_FORMAT
                },
                parset: {
                    name: "Link to Parset",
                    filter: 'none',
                    newTab: true,
                },
                obsolete: {
                    name: "Obsolete",
                    filter:'none'
                },
                error_reason: {
                    name: "Error Details",
                },
                unschedulable_reason: {
                    name:"Unschedulable Reason",    
                },
            }],
            optionalcolumns:  [{    
            }],
            subtasks:[],
            obsoleteTasks:[],cancelledSubtasks:[],
            columnclassname: [{
                "Duration (Days HH:mm:ss)":"filter-input-75",
                "Subtask Type":"filter-input-125",
                "Subtask ID":"filter-input-100",
                "Link to Parset":"filter-input-150",
                "Subtask Status":"filter-input-125"          
            }],
            defaultSortColumn: [{id: "Start Time", desc: false}],
        };
        this.lsKeySortColumn = "";
        this.statusList = [];
        this.subtaskTypeList = [];
        this.selectedRows=[];
        this.fetchSubtask = true;
        this.access_denied_message = "Don't have permission";
        this.setEditorFunction = this.setEditorFunction.bind(this);
        this.deleteTask = this.deleteTask.bind(this);
        this.showDeleteConfirmation = this.showDeleteConfirmation.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
        this.getTaskDeleteDialogContent = this.getTaskDeleteDialogContent.bind(this);
        this.showCancelConfirmation = this.showCancelConfirmation.bind(this);
        this.cancelTask = this.cancelTask.bind(this);
        this.getFilterOptions = this.getFilterOptions.bind(this);
        this.cancelView = this.cancelView.bind(this);
        this.showObsoleteConfirmation = this.showObsoleteConfirmation.bind(this);
        this.markTaskObsolete = this.markTaskObsolete.bind(this);
        this.onRowSelection=this.onRowSelection.bind(this);
        this.showSubtaskCancelConfirmation=this.showSubtaskCancelConfirmation.bind(this);
        this.cancelSubtasks=this.cancelSubtasks.bind(this);
        this.getSubtaskCancelConfirmContent = this.getSubtaskCancelConfirmContent.bind(this);
        this.showSubtaskObsoleteConfirmation=this.showSubtaskObsoleteConfirmation.bind(this);
        this.markSubtasksObsolete=this.markSubtasksObsolete.bind(this);
        this.getSubtaskObsoleteConfirmContent = this.getSubtaskObsoleteConfirmContent.bind(this);
        this.getSubtaskObsoleteStatusContent = this.getSubtaskObsoleteStatusContent.bind(this);
        this.getSubtaskCancelStatusContent = this.getSubtaskCancelStatusContent.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.handleData = this.handleData.bind(this);
        this.refreshTask = this.refreshTask.bind(this);
        this.getTaskDetails = this.getTaskDetails.bind(this);
        this.prepareActionMenus = this.prepareActionMenus.bind(this);
        this.showRestartConfirmation = this.showRestartConfirmation.bind(this);
        this.restartTask = this.restartTask.bind(this);
        this.showSubtaskRestartConfirmation = this.showSubtaskRestartConfirmation.bind(this);
        this.restartSubtask = this.restartSubtask.bind(this);
        this.getSubtaskRestartStatusContent = this.getSubtaskRestartStatusContent.bind(this);
        this.getSubtaskRestartConfirmContent = this.getSubtaskRestartConfirmContent.bind(this);

        if (this.props.match.params.id) {
            this.state.taskId  = this.props.match.params.id;
        }
        if (this.props.match.params.type) {
            this.state.taskType = this.props.match.params.type;
        }
    }

    onRowSelection(selectedRows) {
        this.selectedRows = selectedRows;
    }

    async componentDidUpdate(prevProps, prevState) {
        if (this.state.task && this.props.match.params &&
            (this.state.taskId.toString() !== this.props.match.params.id ||
            this.state.taskType !== this.props.match.params.type)) {
            this.getTaskDetails(this.props.match.params.id, this.props.match.params.type);
       }
       const moduleName = this.props.match.params.type === 'draft' ? 'task_draft': 'task_blueprint'
       if(prevState.taskId !== this.props.match.params.id || prevState.moduleName !== moduleName ) {
            const permissionById = await AuthUtil.getUserPermissionByModuleId(moduleName, this.props.match.params.id)
            this.setState({ permissionById: permissionById, moduleName: moduleName, taskId: this.props.match.params.id});
       }


    }

    async componentDidMount() {
        const taskId = this.props.match.params.id;
        let taskType = this.props.match.params.type;
        this.lsKeySortColumn = 'subtaskSort';
        this.setToggleBySorting();
        await this.getStatusList(taskType.toLowerCase());
        const moduleName = taskType === 'draft' ? 'task_draft': 'task_blueprint'
        const permission = await AuthUtil.getUserRolePermission();
        const permissionById = await AuthUtil.getUserPermissionByModuleId(moduleName, taskId);
        this.setState({userrole: permission, permissionById: permissionById, moduleName: moduleName, taskId: taskId});
        if (taskId && taskType) {
            this.getTaskDetails(taskId, taskType);
        }   else {
            this.setState({redirect: "/not-found"});
        }
    }

    /**
     * JEditor's function that to be called when parent wants to trigger change in the JSON Editor
     * @param {Function} editorFunction 
     */
    setEditorFunction(editorFunction) {
        this.setState({editorFunction: editorFunction});
    }

    /**
     * To get the task details from the backend using the service
     * @param {number} taskId 
     */
    getTaskDetails(taskId, taskType) {
        if (taskId && taskId>0) {
            taskType = taskType?taskType:'draft';
            TaskService.getTaskDetails(taskType, taskId, this.fetchSubtask)
            .then(async(task) => {
                if (task) {
                    taskType === 'blueprint' && await this.getSubtaskDetails(task.subtasks);
                    const suId = taskType==='draft'?task.scheduling_unit_draft_id:task.scheduling_unit_blueprint_id;
                    const taskTemplate = await TaskService.getTaskTemplate(task.specifications_template_id);
                    const schedulingUnit = await TaskService.getSchedulingUnit(taskType, suId, true, `id,name,blueprints,task_${taskType}s,task_${taskType}s.specifications_template,task_${taskType}s.specifications_doc,task_drafts.task_blueprints,task_drafts.task_blueprints_ids`);
                    let path = _.join(['/schedulingunit','view',((this.state.taskType === "draft")?'draft':'blueprint'),schedulingUnit.id], '/');
                    let targetTask = _.find(schedulingUnit[`task_${taskType}s`], suTask => (
                        _.includes(UIConstants.TARGET_OBSERVATION_NAMES,suTask.specifications_template.name)
                    ));
                    if (this.state.editorFunction) {
                        this.state.editorFunction();
                    }
                    if(taskType === 'draft' && task.task_blueprints_ids && task.task_blueprints_ids.length > 0) {
                        this.setState({schedulingUnit: schedulingUnit, supath:path, targetTask: targetTask, hasBlueprint: true, task: task, taskTemplate: taskTemplate, isLoading: false, taskId: taskId, taskType: taskType});
                    }   else {
                        this.setState({schedulingUnit: schedulingUnit, supath:path, targetTask: targetTask, hasBlueprint: false, task: task, taskTemplate: taskTemplate, isLoading: false, taskId: taskId, taskType: taskType});
                    }                    
                    this.prepareActionMenus();
                }   else {
                    this.setState({redirect: "/not-found"});
                }
            });

        }
    }

    async getSubtaskDetails(subtasks) {
        const subtaskPermission = await AuthUtil.getUserPermissionByModuleId('subtask', subtasks[0].id);
        let subtaskList = [];
        for(const subtask of subtasks) {
            let subtaskRow = {};
            subtaskRow['type'] = subtask.subtask_type;
            subtaskRow['id'] = subtask.id;
            subtaskRow['status'] = subtask.state_value;
            subtaskRow['unschedulable_reason'] = subtask.unschedulable_reason;
            subtaskRow['on_sky_start_time']= subtask.on_sky_start_time;
            subtaskRow['on_sky_stop_time']= subtask.on_sky_stop_time;
            subtaskRow['process_start_time']= subtask.process_start_time;
            subtaskRow['process_stop_time']= subtask.process_stop_time;
            subtaskRow['duration']= moment.utc((subtask.duration || 0) * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
            subtaskRow['obsolete'] = subtask.obsolete_since!==null;
            subtaskRow['error_reason'] = subtask.error_reason;
            subtaskRow['links'] = ['Link to Parset'];
            if(subtask.subtask_type === 'observation' || subtask.subtask_type === 'pipeline') {
                subtaskRow['parset']=  `Parset`;
                subtaskRow['linksURL'] = {
                    'Link to Parset': `../../../api/subtask/${subtask.id}/parset`
                };
            } else {
                    subtaskRow['parset']=  ``;
                    subtaskRow['linksURL'] = {
                        'Link to Parset': ``
                    };
            }
            subtaskList.push(subtaskRow);
        }
        this.setState({
            subtaskRowList: subtaskList, subtaskPermission: subtaskPermission[subtasks[0].id]
        });
    }

    toggleBySorting = (sortData) => {
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: sortData });
    }
    setToggleBySorting() {     
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if (sortData) {
            if (Object.prototype.toString.call(sortData) === '[object Array]') {
                this.defaultSortColumn = sortData;
            }
            else {
                this.defaultSortColumn = [{ ...sortData }];
            }
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] });
    }
    /**
     * Show confirmation dialog
     */
    showDeleteConfirmation() {
        let dialog = this.state.dialog;
        dialog.actions = [{ id: 'yes', title: 'Delete Task', callback: this.deleteTask, className:'act-btn-dispose' },
                            { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
        dialog.type = "confirmation";
        dialog.header = "Confirm to Delete Task";
        dialog.showIcon = false;
        dialog.detail = "Do you want to delete this Task?";
        dialog.width = '55vw';
        dialog.content = this.getTaskDeleteDialogContent;
        dialog.onSubmit = this.deleteTask;
        this.setState({dialog: dialog, confirmDialogVisible: true});
    }

    /**
     * Prepare Task details to show on confirmation dialog
     */
    getTaskDeleteDialogContent() {
        const subTask = this.state.task.subtasks ? this.state.task.subtasks.find(sTask => sTask.primary === true): null;
        let selectedTasks = [{suId: this.state.schedulingUnit.id, suName: this.state.schedulingUnit.name, taskId: this.state.task.id, 
            controlId: (subTask)?subTask.id: '', taskName: this.state.task.name, status: this.state.task.status}];
        return  <> 
                   <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                        <Column field="suId" header="Scheduling Unit Id"></Column>
                        <Column field="suName" header="Scheduling Unit Name"></Column>
                        <Column field="taskId" header="Task Id"></Column>
                        <Column field="controlId" header="Control Id"></Column>
                        <Column field="taskName" header="Task Name"></Column>
                        <Column field="status" header="Status"></Column>
                    </DataTable>
                </>
    }

    closeDialog() {        
        this.setState({confirmDialogVisible: false,obsoleteTasks:[],cancelledSubtasks:[]});
    }

    cancelView(){
        this.props.history.length>1?this.props.history.goBack():this.props.history.push(`/task`);
    }

    /**
     * Delete Task
     */
    async deleteTask() {
        let hasError = false;
        if(!await TaskService.deleteTask(this.state.taskType, this.state.taskId)){
            hasError = true;
        }
        if(hasError){
            appGrowl.show({severity: 'error', summary: 'error', detail: 'Error while deleting Task'});
            this.setState({confirmDialogVisible: false});
        }   else {
            appGrowl.show({severity: 'success', summary: 'Success', detail: 'Task deleted successfully'});
            this.setState({confirmDialogVisible: false});
            /* If the page is navigated to from another page of the app, goback to the origin origin else go to SU List page */
            if (this.props.history.length > 2) {
                this.props.history.goBack();
            }   else {
                this.setState({redirect: `/schedulingunit/view/${this.state.taskType}/${this.state.schedulingUnit.id}`});
            }
        }
    }

    /**
     * Show confirmation dialog before cancelling the task.
     */
     showCancelConfirmation() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Confirm to Cancel Task";
        dialog.actions = [{ id: 'yes', title: 'Cancel Task', callback: this.cancelTask, className:'act-btn-dispose' },
                            { id: 'no', title: 'No', className:'act-btn-cancel', callback: this.closeDialog }];
        if (this.TASK_NOT_STARTED_STATUSES.indexOf(this.state.task.status) >= 0) {
            dialog.detail = "Cancelling this task means it will no longer be executed. This action cannot be undone. Do you want to proceed?";
        }   else if (this.TASK_ACTIVE_STATUSES.indexOf(this.state.task.status) >= 0) {
            dialog.detail = "Cancelling this task means it will be aborted. This action cannot be undone. Do you want to proceed?";
        }
        dialog.submit = this.cancelTask;
        dialog.width = '40vw';
        dialog.showIcon = true;
        this.setState({ dialog: dialog, confirmDialogVisible: true });
    }

    showRestartConfirmation() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Confirm to restart Task";
        dialog.actions = [{ id: 'yes', title: 'Restart Task', callback: this.restartTask, className:'act-btn-dispose' },
                            { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
        dialog.detail = "Task on-hold will be queued to restart. Do you want to proceed?";
        dialog.content = "";
        dialog.submit = this.restartTask;
        dialog.width = '40vw';
        dialog.showIcon = true;
        this.setState({ dialog: dialog, confirmDialogVisible: true });
    }

    /**
     * Show confirmation before marking the task as obsolete
     */
    showObsoleteConfirmation() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Confirm to mark task as obsolete";
        dialog.actions = [{ id: 'yes', title: 'Mark as obsolete', className:'act-btn-dispose', callback: this.markTaskObsolete },
                            { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];       
        dialog.detail = "Warning: Obsolete tasks will not be copied and dataproducts will not be used for pipelines and ingest. Are you sure you want to make the task obsolete?";       
        dialog.content = '';
        dialog.submit = this.markTaskObsolete;
        dialog.width = '40vw';
        dialog.showIcon = true;
        this.setState({ dialog: dialog, confirmDialogVisible: true });
    }

    /**
     * Show confirmation dialog content while cancelling subtask.
     * @returns component
     */
    getSubtaskCancelConfirmContent() {
        let selectedTasks = [];
        let ignoredTasks = []; 
        for (const obj of this.selectedRows) {
            if (obj.status !== 'cancelled') {
                selectedTasks.push({
                    id: obj.id, type: obj.type, status: obj.status
                });
            }   else {
                ignoredTasks.push({
                    id: obj.id, type: obj.type, status: obj.status
                });
            }
        }
        this.selectedRows = [];
              
        return <>
            {selectedTasks.length > 0 &&
            <div style={{marginTop: '1em'}}>
                <b>Subtask(s) that can be cancelled</b>
                <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="id" header="Subtask Id"></Column>
                    <Column field="type" header="Type"></Column>                  
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>}
            {ignoredTasks.length > 0 &&
            <div style={{marginTop: '1em'}}>
                <b>Subtask(s) already cancelled will be ignored</b>
                <DataTable value={ignoredTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="id" header="Subtask Id"></Column>
                    <Column field="type" header="Type"></Column>                  
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>
            }
            </>
    }

    /**
     * Content for confirmation dialog to mark subtask as obsolete
     */
    getSubtaskObsoleteConfirmContent() {
        const subtaskWithObsolete = this.state.subtaskWithObsolete;
        const subtaskWithoutObsolete = _.difference(this.selectedRows, subtaskWithObsolete);
        let selectedSubtaskWithObsolete = [];
        let selectedSubtasksWithoutObsolete = [];
        for (const obj of subtaskWithObsolete) {           
            selectedSubtaskWithObsolete.push({
                    id: obj.id, type: obj.type, status: obj.status
                });           
        }
        for (const obj of subtaskWithoutObsolete) {           
            selectedSubtasksWithoutObsolete.push({
                    id: obj.id, type: obj.type, status: obj.status
                });           
        }
        this.selectedRows = [];
              
        return  <div style={{marginTop: '1em'}}>
                {selectedSubtasksWithoutObsolete.length>0 &&
                    <>
                        <b>Subtask(s) that can be marked as obsolete</b>
                        <DataTable value={selectedSubtasksWithoutObsolete} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Subtask Id"></Column>
                            <Column field="type" header="Type"></Column>                  
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>} 
                {selectedSubtaskWithObsolete.length>0 &&
                    <>
                        <b>Subtask(s) already marked as obsolete will be ignored</b>
                        <DataTable value={selectedSubtaskWithObsolete} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Subtask Id"></Column>
                            <Column field="type" header="Type"></Column>                  
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
                  
            </div>
    }

    getSubtaskRestartConfirmContent() {
        const subtaskWithOnhold = this.state.selectedOnHoldRows;
        const subtaskWithoutOnhold = _.difference(this.selectedRows, subtaskWithOnhold);
        let selectedSubtaskWithOnhold = [];
        let selectedSubtasksWithoutOnhold = [];
        for (const obj of subtaskWithOnhold) {           
            selectedSubtaskWithOnhold.push({
                    id: obj.id, type: obj.type, status: obj.status
                });           
        }
        for (const obj of subtaskWithoutOnhold) {           
            selectedSubtasksWithoutOnhold.push({
                    id: obj.id, type: obj.type, status: obj.status
                });           
        }
        this.selectedRows = [];
              
        return  <div style={{marginTop: '1em'}}>
                {selectedSubtaskWithOnhold.length>0 &&
                    <>
                        <b>Subtask(s) that will be queued to restart</b>
                        <DataTable value={selectedSubtaskWithOnhold} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Subtask Id"></Column>
                            <Column field="type" header="Type"></Column>                  
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>} 
                {selectedSubtasksWithoutOnhold.length>0 &&
                    <>
                        <b>Subtask(s) with status other than &apos;on hold&apos; will be ignored</b>
                        <DataTable value={selectedSubtasksWithoutOnhold} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Subtask Id"></Column>
                            <Column field="type" header="Type"></Column>                  
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
                  
            </div>
    }
    /**
     * Content for status dialog after cancelling subtask
     */
    getSubtaskCancelStatusContent () {
        let selectedTasks = [];
        for (const obj of this.state.cancelledSubtasks) {
            selectedTasks.push({
                id: obj.id, type: obj.type, status: obj.status
            })
           
        }
        this.selectedRows = [];
              
        return <>
            {selectedTasks.length > 0 &&
            <div style={{marginTop: '1em'}}>
                <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="id" header="Subtask Id"></Column>
                    <Column field="type" header="Type"></Column>                  
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>}
            </>
    }

    /**
     * Content for status dialog after marking subtask as obsolete
     */
    getSubtaskObsoleteStatusContent() {
        this.selectedRows = []; 
        return <div style={{marginTop: '1em'}}>
                <DataTable value={this.state.obsoleteTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="id" header="Subtask Id"></Column>
                    <Column field="type" header="Type"></Column>      
                    <Column field="actionStatus" header="Action Status"></Column>
                </DataTable>
            </div>
    }

    getSubtaskRestartStatusContent() {
        this.selectedRows = []; 
        return <div style={{marginTop: '1em'}}>
                <DataTable value={this.state.restartSubTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="id" header="Subtask Id"></Column>
                    <Column field="type" header="Type"></Column>      
                    <Column field="actionStatus" header="Action Status"></Column>
                </DataTable>
            </div>
    }

    /**
     * Function that cancels the subtask and set content to show in status dialog
     */
    async cancelSubtasks() {
        let subtasks = this.state.subtaskRowList;        
        let cancelledSubtasks = [];
        for (const selectedTask of this.state.selectedSubtasks) {          
                const cancelledTask = await TaskService.cancelSubtask(selectedTask.id);
                let task = _.find(subtasks, {'id': selectedTask.id});
                if (cancelledTask) {
                    task.status = cancelledTask.state_value;
                }
                cancelledSubtasks.push({
                    id: task.id,
                    type: task.type, 
                    status: task.status.toLowerCase()==='cancelled'?'Cancelled': 'Error Occured'
                });            
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Cancel subtask(s) - Status";
        dialog.actions = [{ id: 'no', title: 'Ok', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getSubtaskCancelStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ subtaskRowList:subtasks, cancelledSubtasks: cancelledSubtasks, dialog: dialog, confirmDialogVisible: true });        
    }

    /**
     * Function to mark subtask as obsolete
     */
    async markSubtasksObsolete() {
        let subtasks = this.state.subtaskRowList;      
        const subtaskWithoutObsolete = _.difference(this.selectedRows, this.state.subtaskWithObsolete);  
        let obsoleteTasks = []
        for (const selectedTask of subtaskWithoutObsolete) {          
                const obsoleteTask = await TaskService.markSubtaskObsolete(selectedTask.id);
                let task = _.find(subtasks, {'id': selectedTask.id});
                if (obsoleteTask) {
                    task.status = obsoleteTask.state_value;
                    task['obsolete'] = obsoleteTask.obsolete_since!==null;
                }
                obsoleteTasks.push({
                    id: task.id,
                    type: task.type, 
                    actionStatus:obsoleteTask?.obsolete_since?'Success':'Failed'
                });            
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Mark Subtask(s) as obsolete - Status";
        dialog.actions = [{ id: 'no', title: 'Ok', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getSubtaskObsoleteStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ subtaskRowList:subtasks, obsoleteTasks: obsoleteTasks, dialog: dialog, confirmDialogVisible: true });        
    }
    
    async restartSubtask() {
        let subtasks = this.state.subtaskRowList;
        let restartSubTasks = []
        for (const selectedTask of _.cloneDeep(this.state.selectedOnHoldRows)) {          
                const restartSubtask = await TaskService.restartSubtaskTaskOnHold(selectedTask.id);
                let task = _.find(subtasks, {'id': selectedTask.id});
                restartSubTasks.push({
                    id: task.id,
                    type: task.type, 
                    actionStatus:restartSubtask && restartSubtask.status === 200 ?'Success':'Failed'
                });            
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Restart Subtask(s) - Status";
        dialog.actions = [{ id: 'no', title: 'Close', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getSubtaskRestartStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ subtaskRowList:subtasks, restartSubTasks: restartSubTasks, dialog: dialog, confirmDialogVisible: true });        
    }
    /**
     * Shows confirmation dialog before cancelling subtask(s)
     */
    showSubtaskCancelConfirmation() {     
            this.pageUpdated = false;
            let selectedSubtasks = this.selectedRows.filter(task => {
                                            return task.status !== 'cancelled'});
            if (this.selectedRows.length === 0) {
                appGrowl.show({ severity: 'info', summary: 'Select Row', 
                                detail: 'Please Select at least one Subtask to cancel' });
            } else {
                if(selectedSubtasks.length === 0) {
                    appGrowl.show({ severity: 'warn', summary: 'Already Cancelled', 
                                detail: 'Selected subtask(s) are already cancelled' });
                }   else {
                    let dialog = this.state.dialog;
                    dialog.type = "confirmation";
                    dialog.header = "Confirm to cancel Subtask(s)";
                    dialog.actions = [{ id: 'yes', title: 'Cancel Subtasks', callback: this.cancelSubtasks, className:'act-btn-dispose' },
                                        { id: 'no', title: 'No', className:'act-btn-cancel', callback: this.closeDialog }];
                    dialog.detail = "Cancelling the subtask means, it will no longer be executed / will be aborted. This action cannot be undone. Already finished/cancelled subtask(s) will be ignored. Do you want to proceed?";
                    dialog.content = this.getSubtaskCancelConfirmContent;
                    dialog.submit = this.cancelSubtasks;
                    dialog.width = '55vw';
                    dialog.showIcon = false;
                    this.setState({ dialog: dialog, confirmDialogVisible: true, selectedSubtasks: selectedSubtasks });   
                }             
            }        
    }

    /**
     * Function to show confirmation dialog before restarting subtask
     */
     showSubtaskRestartConfirmation() {     
        this.pageUpdated = false;
        let selectedOnHoldRows = this.selectedRows.filter(task => {
            return task.status === 'on hold'});
        if(this.selectedRows.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Please select minimum one subtask with status "on hold"' });
        } else {
            if (selectedOnHoldRows.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Select at least one subtask(s) with status on hold', 
                            detail: 'Selected subtask(s) status should be "on hold"' });
            }
            else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header = "Confirm to restart Subtask(s)";
                dialog.actions = [{ id: 'yes', title: 'Restart', className:'act-btn-dispose', callback: this.restartSubtask },
                                    { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
                dialog.detail = "";
                dialog.content = this.getSubtaskRestartConfirmContent;
                dialog.submit = this.restartSubtask;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({ dialog: dialog, confirmDialogVisible: true,  selectedOnHoldRows: selectedOnHoldRows });   
            }
                          
        }
    }
    
    showSubtaskObsoleteConfirmation() {     
        this.pageUpdated = false;
        let selectedSubtasks = this.selectedRows.filter(task => {
            return task.status !== 'obsolete'});
        const  subtaskWithObsolete = this.checkObsolete(selectedSubtasks)
        const subtaskWithoutObsolete = _.difference(selectedSubtasks, subtaskWithObsolete);
        if(selectedSubtasks.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Please select minimum one subtask to mark as obsolete' });
        } else {
            if (subtaskWithoutObsolete.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Already marked as obsolete', 
                            detail: 'Selected subtask(s) are already marked as obsolete' });
            }
            else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header = "Confirm to mark Subtask(s) as obsolete";
                dialog.actions = [{ id: 'yes', title: 'Mark as obsolete', className:'act-btn-dispose', callback: this.markSubtasksObsolete },
                                    { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
                dialog.detail = "Warning: Obsolete subtask will not be copied and dataproducts will not be used for pipelines and ingest. Are you sure you want to make the subtask obsolete?";
                dialog.content = this.getSubtaskObsoleteConfirmContent;
                dialog.submit = this.markSubtasksObsolete;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({ dialog: dialog, confirmDialogVisible: true,  subtaskWithObsolete: subtaskWithObsolete });   
            }   
        }  
    }

    /**
     * Function to cancel the task and update its status.
     */
    async cancelTask() {
        let task = this.state.task;
        let cancelledTask = await TaskService.cancelTask(task.id);
        if (!cancelledTask) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while cancelling Task' });
            this.setState({ confirmDialogVisible: false });
        } else {
            task.status = cancelledTask.status;
            let actions = this.state.actions;
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Task is cancelled successfully' });
            this.setState({ confirmDialogVisible: false, task: task, actions: actions});
        }
    }

    
    async restartTask() {
        let task = this.state.task;
        let restartedTask = await TaskService.restartTaskOnHold(task.id);
        if (!restartedTask) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while cancelling Task' });
            this.setState({ confirmDialogVisible: false });
        } else {
            let actions = this.state.actions;
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Task is queued to restart successfully' });
            this.setState({ confirmDialogVisible: false, task: task, actions: actions});
        }
    }

    /**
     * Function to mark task as obsolete
     */
    async markTaskObsolete() {
        let task = this.state.task;
        let obsoleteTask = await TaskService.markTaskObsolete(task.id);
        if (!obsoleteTask.status) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while Obsolete Task' });
            this.setState({ dialogVisible: false });
        } else {
            task.status = obsoleteTask.status;
            task.obsolete_since = obsoleteTask.obsolete_since;
            let actions = this.state.actions;
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Task is marked as Obsolete' });
            this.setState({ confirmDialogVisible: false, task: task, actions: actions});
        }
    }

    /**
     * Function to check if subtask is marked as obsolete
     */
    checkObsolete(selectedSubtask) {
        const subtasksWithObsolete = selectedSubtask.filter(subtask => {
            return subtask.obsolete}); 
        return subtasksWithObsolete;
    }

    async getStatusList(type) {
        const taskFilters = await TaskService.getTaskFilterDefinition(type);
        if (taskFilters.data.filters['status']) {
            taskFilters.data.filters['status'].choices.forEach(choice => {
                this.statusList.push(choice.value);
            })
        }
        const subtaskTypes = await TaskService.getSubtaskType();
        subtaskTypes.forEach(subtaskType => {
            this.subtaskTypeList.push(subtaskType.value);
        })
    }

    getFilterOptions(id) {
        let options = null;
        if(id && id === 'Subtask Status') {
            options = this.statusList;
        }
        if(id && id === 'Subtask Type') {
            options = this.subtaskTypeList;
        }
        return options;
    }

    /**
     * Function to call wnen websocket is connected
     */
     onConnect() {
        try{
            const userDets = localStorage.getItem("user");
            if (userDets) {
                this.websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }

    /**
     * Function to call when websocket is disconnected
     */
    onDisconnect() {
        console.log("WS Closed")
    }

    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    handleData(data) {
        if (data) {
            console.log('received websocket data:', data)
            const jsonData = JSON.parse(data);
            this.refreshTask(jsonData);
        }
    }

    /**
     * Refresh the Task Draft details
     * @param {*} jsonData 
     */
     refreshTask(jsonData) {
        let task_type = 'blueprint';
        let taskId = 0;
        if ((jsonData.object_details.id+'') === (this.props.match.params.id+'')) {
            if (jsonData.object_type === 'task_draft' && jsonData.action === 'delete') {
                this.prepareActionMenus(true);
                appGrowl.show({ severity: 'error', summary: 'Error', sticky: true, detail: `This Task is deleted by another user.` });
                return;
            }   else if (jsonData.object_type === 'task_draft' && this.props.match.params.type === 'draft') {
                task_type = 'draft';
                taskId = jsonData.object_details.id;
            }   else if ((jsonData.object_type === 'task_draft' || jsonData.object_type === 'task_blueprint') && this.props.match.params.type === 'blueprint') {
                taskId = this.props.match.params.id;
            }
            this.getTaskDetails(taskId, task_type);
        }
    }

    /**
     * Prepare action menu
     * @param {*} disableUpdateAction : true - relevant button will be disabled
     */
    prepareActionMenus(disableUpdateAction) {
        let actions = [ ];
     
        if (this.state.taskType === 'draft') {
            const taskId = this.state.task?this.state.task.id:'';
            actions = [{   icon: 'fa-edit',
                            title:disableUpdateAction === undefined ?this.state.permissionById[this.state.taskId] && this.state.permissionById[this.state.taskId].edit ?'Click to Edit Task': `${this.access_denied_message} to edit` : 'This Task not available.',
                            disabled: disableUpdateAction === undefined ?this.state.permissionById[this.state.taskId] ? !this.state.permissionById[this.state.taskId].edit: true: disableUpdateAction,
                            props : { pathname:`/task/edit/draft/${taskId}`,
                                        state: {taskId: taskId} 
                                    } 
                        }];
            actions.push({icon: 'fa fa-trash',title:disableUpdateAction === undefined ?this.state.hasBlueprint ? 'Cannot delete Draft when Blueprint exists':
                this.state.permissionById?.[this.state.taskId].delete? 'Delete Task': `${this.access_denied_message} to delete`: 'This Task already deleted!',  
                type: 'button',  disabled: disableUpdateAction === undefined ?this.state.hasBlueprint ? true:
                this.state.permissionById?.[this.state.taskId].delete? false: true:disableUpdateAction,
                    actOn: 'click', props:{ callback: this.showDeleteConfirmation}});
        }   else {
            actions = [{    icon: 'fa-lock',
                            title: 'Cannot edit blueprint'}];
            if (this.state.task) {
                actions.push({icon: 'fa-sync',
                                    title: `Restart task`,
                                    disabled: this.state.task && !(this.state.task.status === 'on hold'),
                                    type: 'button', actOn: 'click', props: { callback: this.showRestartConfirmation }})
                actions.push({icon: 'fa-ban', type: 'button', actOn: 'click', 
                                    title: this.TASK_END_STATUSES.indexOf(this.state.task.status.toLowerCase())>=0?'Cannot Cancel Task':
                                    this.state.permissionById && this.state.permissionById[this.state.taskId].cancel? 'Cancel Task': `${this.access_denied_message} to cancel`,
                                    disabled:this.TASK_END_STATUSES.indexOf(this.state.task.status.toLowerCase())>=0? true:
                                    this.state.permissionById && this.state.permissionById[this.state.taskId].cancel? false: true, 
                                    props: { callback: this.showCancelConfirmation }
                                });
                actions.push({icon: 'fa-times-circle', iconType: 'far',
                                    title: this.state.task && this.state.task.obsolete_since?'Already marked as obsolete':this.state.permissionById[this.state.taskId] && this.state.permissionById[this.state.taskId].obsolete?'Mark as Obsolete':`${this.access_denied_message}`,
                                    disabled: this.state.task?.obsolete_since? true: this.state.permissionById[this.state.taskId] && this.state.permissionById[this.state.taskId].obsolete?false: true,
                                    type: 'button', actOn: 'click', props: { callback: this.showObsoleteConfirmation }});
            }
        }
        
        actions.push({  icon: 'fa-window-close', link: this.props.history.goBack,
                        title:'Click to Close Task', type: 'button',  actOn: 'click', props:{ callback: this.cancelView }});
        this.setState({actions: actions});
     }
     
    render() {
        if (this.state.redirect) {
            return <Redirect to={ {pathname: this.state.redirect} }></Redirect>
        }        
        let jeditor = null
        if (this.state.taskTemplate) {
            jeditor = React.createElement(Jeditor, {title: " Task Specification", 
                                                        schema: this.state.taskTemplate.ref_resolved_schema,
                                                        initValue: this.state.task.specifications_doc,
                                                        disabled: true,
                                                        parentFunction: this.setEditorFunction,
                                                        resolveExtRef: true,
                                                        observationType: this.state.taskTemplate.name,
                                                        targetObservation: this.state.targetTask,
                                                    });
        }

        // Child component to render predecessors and successors list
        const TaskRelationList = ({ list }) => (
            <ul className="task-list">
            {list && list.map(item => (
                <li key={item.id}>
                    {/* <Link to={ { pathname:'/task/view', state: {id: item.id, type: item.draft?'blueprint':'draft'}}}>{item.name}</Link> */}
                    <Link to={ { pathname:`/task/view/${item.task_blueprints?'draft':'blueprint'}/${item.id}`}}>{item.name}</Link>
                </li>
            ))}
            </ul>
          );
        return (
            <React.Fragment>
                {this.state.permissionById[this.state.taskId] &&
                    <>
                        { this.state.permissionById[this.state.taskId].view ?
                            <>
                                <PageHeader location={this.props.location} title={'Task - Details'} 
                                            actions={this.state.actions}/>
                                { this.state.isLoading? <AppLoader /> : this.state.task &&
                                        <div className="main-content">
                                        <div className="p-grid">
                                            <label className="col-lg-2 col-md-2 col-sm-12">Name</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.task.name}</span>
                                            <label className="col-lg-2 col-md-2 col-sm-12">Description</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.task.description}</span>
                                        </div>
                                        <div className="p-grid">
                                            <label className="col-lg-2 col-md-2 col-sm-12">Created At</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{moment.utc(this.state.task.created_at).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                                            <label className="col-lg-2 col-md-2 col-sm-12">Updated At</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{moment.utc(this.state.task.updated_at).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                                        </div>
                                        {this.state.taskType === 'blueprint' && this.state.task.task_type === 'observation' &&
                                        <div className="p-grid">
                                            <label className="col-lg-2 col-md-2 col-sm-12">Observation Start Time</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.task.on_sky_start_time?moment(this.state.task.on_sky_start_time,moment.ISO_8601).format(UIConstants.CALENDAR_DATETIME_FORMAT):""}</span>
                                            <label className="col-lg-2 col-md-2 col-sm-12">Observation End Time</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.task.on_sky_stop_time?moment(this.state.task.on_sky_stop_time,moment.ISO_8601).format(UIConstants.CALENDAR_DATETIME_FORMAT):""}</span>
                                        </div>}
                                        {this.state.taskType === 'blueprint' &&
                                        <div className="p-grid">
                                            <label className="col-lg-2 col-md-2 col-sm-12">Process Start Time</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.task.process_start_time?moment(this.state.task.process_start_time,moment.ISO_8601).format(UIConstants.CALENDAR_DATETIME_FORMAT):""}</span>
                                            <label className="col-lg-2 col-md-2 col-sm-12">Process End Time</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.task.process_stop_time?moment(this.state.task.process_stop_time,moment.ISO_8601).format(UIConstants.CALENDAR_DATETIME_FORMAT):""}</span>
                                        </div>}                  
                                        <div className="p-grid">                                         
                                            {this.state.schedulingUnit &&
                                            <>
                                                <label className="col-lg-2 col-md-2 col-sm-12">Scheduling Unit</label>
                                                <Link className="col-lg-4 col-md-4 col-sm-12" to={ { pathname:this.state.supath, state: {id: this.state.schedulingUnit.id}}}>{this.state.schedulingUnit?this.state.schedulingUnit.name:''}</Link>
                                            </>}                                            

                                                <label className="col-lg-2 col-md-2 col-sm-12">Short Description</label>
                                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.task.short_description}</span>

                                            {this.state.taskType === 'blueprint' && 
                                            <>
                                                <label className="col-lg-2 col-md-2 col-sm-12">Status</label>
                                                <div className="col-lg-4 col-md-4 col-sm-12">
                                                    <span className={"tag-content p-chips-token-label su-"+this.state.task.status}>{this.state.task.status}</span>
                                                    {this.state.task.status === 'unschedulable' &&
                                                        <span style={{marginLeft: '5px'}} > ({this.state.task.unschedulable_reason})</span>
                                                    }
                                                </div>
                                                {/* if obsolete_since is null then checkbox will be unchecked, otherwise date which specified will be displayed with checkbox checked */}
                                                <label className="col-lg-2 col-md-2 col-sm-12">Obsolete</label>
                                                <div className="col-lg-4 col-md-4 col-sm-12">                                                
                                                    <Checkbox checked={this.state.task?.obsolete_since ? true : false} disabled readOnly={true} />{' '}
                                                    <span>{this.state.task?.obsolete_since  && (`(since ${moment.utc(this.state.task?.obsolete_since).format(UIConstants.CALENDAR_DATETIME_FORMAT)})`)}</span>
                                                </div>
                                            </>}
                                        </div>
                                        <div className="p-grid">
                                            <label className="col-lg-2 col-md-2 col-sm-12">Predecessors</label>
                                            <div className="col-lg-4 col-md-4 col-sm-12">
                                                <TaskRelationList list={this.state.task.predecessors} />
                                            </div>
                                            <label className="col-lg-2 col-md-2 col-sm-12">Successors</label>
                                            <div className="col-lg-4 col-md-4 col-sm-12">
                                                <TaskRelationList list={this.state.task.successors} />
                                            </div>
                                        </div>
                                        <div className="p-grid">
                                            <label className="col-lg-2 col-md-2 col-sm-12">Template</label>
                                            <span className="col-lg-4 col-md-4 col-sm-12">{this.state.taskTemplate.name}</span>
                                            <label className="col-lg-2 col-md-2 col-sm-12">{this.state.taskType==='draft'?'Blueprints':'Draft'}</label>
                                            <div className="col-lg-4 col-md-4 col-sm-12">
                                                {this.state.taskType === 'draft' &&
                                                    <TaskRelationList list={this.state.task.blueprints} />
                                                }
                                                {this.state.taskType === 'blueprint' &&                                                    
                                                    <Link to={ { pathname:`/task/view/draft/${this.state.task.draft_id}`}}>{this.state.task.draftObject.name}</Link>
                                                }
                                            </div>
                                        </div>
                                        {this.state.taskType === 'blueprint' &&
                                            <div className="p-grid">
                                                <label className="col-lg-2 col-md-2 col-sm-12">Data Product</label>
                                                <div className="col-lg-4 col-md-4 col-sm-12">
                                                    <Link to={ { pathname:`/task/view/blueprint/${this.state.taskId}/dataproducts`}}> View Data Product</Link>
                                                </div>
                                                <label className="col-lg-2 col-md-2 col-sm-12">Status Logs</label>
                                                <div className="col-lg-4 col-md-4 col-sm-12">
                                                    <button className="p-link" onClick={() => { this.setState({showStatusLogs: true})}}><i className="fa fa-history"></i></button>
                                                    <Dialog header="State change logs" visible={this.state.showStatusLogs} maximizable position="right" style={{ width: '50vw' }} 
                                                            onHide={() => {this.setState({showStatusLogs: false})}}
                                                            maximized={false} className="content_dlg">
                                                        <TaskStatusLogs taskId={this.state.taskId}></TaskStatusLogs>
                                                    </Dialog>
                                                </div>
                                            </div>
                                        }
                                        {this.state.taskType === 'blueprint' && 
                                            <div style={{marginBottom: "10px"}}>
                                             <div style={{marginTop: "10px"}}> 
                                                <h3>Subtasks</h3>                                        
                                             </div>
                                             <div className="delete-option">
                                                <div >
                                                    <span className="p-float-label">
                                                            <button className="p-link" href="#" style={{ marginLeft: '8px'}} onClick={this.showSubtaskRestartConfirmation} 
                                                                title={`Restart subtask (s)`}>
                                                                <i className={`fa fa-sync`} aria-hidden="true" ></i>
                                                            </button>
                                                            <button className="p-link" href="#" style={{ marginLeft: '8px'}} onClick={this.showSubtaskCancelConfirmation} 
                                                                title={this.state.subtaskPermission.cancel?'Cancel Subtasks': `${this.access_denied_message} to cancel`}>
                                                                <i className={`fa fa-ban ${!this.state.subtaskPermission.cancel?'fa-disabled':''}`}  aria-hidden="true" ></i>
                                                            </button>
                                                            <button className="p-link" href="#" style={{ marginLeft: '8px'}} onClick={this.showSubtaskObsoleteConfirmation} 
                                                                title={this.state.subtaskPermission.obsolete?'Mark as Obsolete': `${this.access_denied_message} to mark as obsolete`}>
                                                                <i className={`far fa-times-circle ${!this.state.subtaskPermission.obsolete?'fa-disabled':''}`} aria-hidden="true" ></i>
                                                            </button>
                                                    </span>
                                                </div>
                                            </div>

                                                <ViewTable 
                                                    data={this.state.subtaskRowList} 
                                                    defaultcolumns={this.state.defaultcolumns} 
                                                    optionalcolumns={this.state.optionalcolumns}
                                                    columnclassname={this.state.columnclassname}
                                                    defaultSortColumn={this.defaultSortColumn}
                                                    allowRowSelection={true}
                                                    onRowSelection={this.onRowSelection}
                                                    showFilterOption={this.getFilterOptions}
                                                    paths={this.state.paths}
                                                    lsKeySortColumn={this.lsKeySortColumn}
                                                    storeFilter={true}
                                                    toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                                                    tablename="subtask_list"
                                                    unittest={this.props.isUnitTest}
                                                />
                                            </div>
                                        }
                                        <div className="p-fluid">
                                            <div className="p-grid"><div className="p-col-12" style={{width: "100%"}}>
                                                {this.state.taskTemplate?jeditor:""}
                                            </div></div>
                                        </div>
                                        </div>
                                        
                                }
                                <CustomDialog type="confirmation" visible={this.state.confirmDialogVisible} width={this.state.dialog.width}
                                    header={this.state.dialog.header} message={this.state.dialog.detail} 
                                    content={this.state.dialog.content} onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.callBackFunction}
                                    showIcon={this.state.dialog.showIcon} actions={this.state.dialog.actions}>
                                </CustomDialog>
                                {!this.state.isLoading &&
                                    <Websocket ref={websocket => this.websocket = websocket} url={process.env.REACT_APP_WEBSOCKET_URL} 
                                        onOpen={this.onConnect} onMessage={this.handleData} onClose={this.onDisconnect} />}
                            </>: <AccessDenied/>}
                    </> }
            </React.Fragment>
        );
    }
}

TaskView.propTypes = {
    match: PropTypes.object,
  history: PropTypes.object,
  location: PropTypes.oneOfType([PropTypes.object, PropTypes.string]),
  isUnitTest: PropTypes.any
}