import _ from 'lodash';

import { UnitConverter, SIPrefixFactor } from './../utils/unit.converter'

import axios from "axios"

const ProjectService = {
    getProjectCategories: async function() {
        try {
          const url = `/api/project_category/`;
          const response = await axios.get(url);
          return response.data.results;
        } catch (error) {
          console.error(error);
        }
    },
      getPressurePlot : async function(projectname) {
                    if (projectname===undefined) return;
                    let imageurl =   "/api/project/" + projectname + "/pressure_plot"
                    try {
                        const response = await axios.get( imageurl, {  responseType: "blob"  }  );
                        if (URL.createObjectURL) {
                            const dataUrl = URL.createObjectURL(response.data);
                            return dataUrl;
                        }
                        else
                        {
                            return response.data
                        }
                    } catch (error) {
                        if (error?.response?.status==404) {
                            return;
                        }
                        else
                        {
                            console.error("getPresurePlot Error fetching image:", imageurl, error.message);
                        }
                    }
               
            },
    getPeriodCategories: async function() {
        try {
          const url = `/api/period_category/`;
          const response = await axios.get(url);
          return response.data.results;
        } catch (error) {
          console.error(error);
        }
    },
    getFileSystem: async function(){
      try {
        const url = `/api/filesystem/`;
        const response = await axios.get(url);
        return response.data.results;
      } catch (error) {
        console.error(error);
      } 
      },
    getCluster:async function(){
     try {
        const url = `/api/cluster/`;
        const response = await axios.get(url);
        return response.data.results;
      } catch (error) {
        console.error(error);
      } 
    },
    getResources: async function() {
        try {
            const url = `/api/resource_type`;
            const response = await axios.get(url);
            return response.data.results;
          } catch (error) {
            console.error(error);
            return [];
          }
    },
    getDefaultProjectResources: async function() {
        try {
          return Promise.resolve({'LOFAR Observing Time': 3600, 
                    'LOFAR Observing Time prio A': 3600, 
                    'LOFAR Observing Time prio B': 3600,
                    'CEP Processing Time': 3600,
                    'LTA Storage': SIPrefixFactor ** 4,
                    'Number of triggers': 1,
                    'LOFAR Support Time': 3600});
        } catch (error) {
          console.error(error);
        }
    },
    getProjectStates: async function() {
      try {
        const url = `/api/project_state/`;
        const response = await axios.get(url);
        return response.data.results;
      } catch (error) {
        console.error(error);
        return [];
      }
    },
    saveProject: async function(project, projectQuota) {
      let archive_location = project.archive_location;
      try {
        const response = await axios.post(('/api/project/'), project);
        project = response.data;
        project['isQuotaCreated'] = true;
        for (let  quota of projectQuota) {
          quota.project = project.url;
          await this.saveProjectQuota(quota)
          .then( async response => {
            if (response.status > 299) {
              project['isQuotaCreated'] = false;
            }
            else {
              if(response.data.resource_type_id === 'LTA Storage') {
                let project_quota_archive_location = {
                  archive_location: archive_location,
                  project_quota: response.data.url
                  }
                if(project_quota_archive_location.archive_location && project_quota_archive_location.project_quota) {
                  await this.saveProjectQuotaArhiveLocation(project_quota_archive_location);
                }
              }
            }
          })
          if (response.status > 299) {
              project['isQuotaCreated'] = false;
          }
        }
        return response.data;
      } catch (error) {
        return error.response.data;
      }
    },
    updateProject: async function(id, project) {
      try {
        const response = await axios.put((`/api/project/${id}/`), project);
        project = response.data;
        project['isUpdated'] = true;
        return project;
      } catch (error) {
        console.log(error.response.data);
        project = error.response.data;
        project['isUpdated'] = false;
        return project;
      }
    },
    saveProjectQuota: async function(projectQuota) {
      try {
        const response = await axios.post(('/api/project_quota/'), projectQuota);
        return response;
      } catch (error) {
        console.error(error);
        return error.response;
      }
    },
    updateProjectQuota: async function(projectQuota, project) {
      let result = null;
      let archive_location = project.archive_location;
      try {
        await axios.put(`/api/project_quota/${projectQuota.id}/`, projectQuota)
        .then(async response => {
          result = response;
          if(response.data.resource_type_id === 'LTA Storage') {
            let project_quota_archive_location = {
              archive_location: archive_location,
              project_quota: response.data.url
              }
            if(project_quota_archive_location.archive_location && project_quota_archive_location.project_quota) {
              if(project.archiveId) {
                project_quota_archive_location.id = project.archiveId;
                result.data.project_quota_archive_location_ids.push((await this.updateProjectQuotaArhiveLocation(project_quota_archive_location)).id);
                
              } else {
                result.data.project_quota_archive_location_ids.push((await this.saveProjectQuotaArhiveLocation(project_quota_archive_location)).id);
              }
            }
          }
        })
      } catch (error) {
        console.error(error);
        return result;
      }
      return result;
    },
    deleteProjectQuota: async function(projectQuota) {
      try {
        const response = await axios.delete(`/api/project_quota/${projectQuota.id}/`);
        return response.status===204?{message: 'deleted'}:null;
      } catch (error) {
        console.error(error);
        return null;
      }
    },
    saveProjectQuotaArhiveLocation: async function(project_quota_archive_location) {
      try {
        const response = await axios.post(('/api/project_quota_archive_location/'), project_quota_archive_location);
        return response.data;
      } catch (error) {
        console.error(error);
        return error.response;
      }
    },
    updateProjectQuotaArhiveLocation: async function(project_quota_archive_location) {
      const response = null;
      try {
        const response = await axios.put(`/api/project_quota_archive_location/${project_quota_archive_location.id}/`, project_quota_archive_location)
        return response.data;
      } catch (error) {
        console.error(error);
        return response;
      }
    },
    getProjectQuotaArchiveLocation: async function(id) {
      try {
          const url = `/api/project_quota_archive_location/${id}/`;
          const response = await axios.get(url);
          return response.data;
        } catch (error) {
          console.error(error);
        }
    },
    getAllProjectQuotaArchiveLocation: async function() {
      try {
          const url = `/api/project_quota_archive_location/`;
          const response = await axios.get(url);
          return response.data;
        } catch (error) {
          console.error(error);
          return [];
        }
    },
    deleteProjectQuotaArchiveLocation: async function(id) {
      try {
        const response = await axios.delete(`/api/project_quota_archive_location/${id}/`);
        return response.status===204?{message: 'deleted'}:null;
      } catch (error) {
        console.error(error);
        return null;
      }
    },
    getProjects: async function() {
      try {
        const response = await axios.get(('/api/project/?limit=150'));
        let projects = response.data.results;
        const response1 = await axios.get(('/api/project_quota'));
        const allProjectQuota = response1.data.results;
        for (let project of projects) {
          let projectQuota = _.filter(allProjectQuota, function(projQuota) { return _.includes(project.project_quota_ids, projQuota.id)});
          for (const quota of projectQuota) {
            project[quota.resource_type_id] = quota;
          }
        }
        return response.data.results;
      } catch (error) {
        console.error(error);
        return [];
      }
    },
    getProjectDetails: async function(id) {
      try {
        const response = await axios.get((`/api/project/${id}`));
        let project = response.data;
        return project;
      } catch(error) {
        console.error(error);
        return null;
      }
    },
    getProjectQuota: async function(quotaId) {
      try {
        const response = await axios.get((`/api/project_quota/${quotaId}`));
        return response.data;
      } catch (error) {
        console.error(error);
        return null;
      }
    },
    /**
     * Function returns project list with or without quota
     * @param {boolean} fetchQuota - flag to fetch quota or not
     * @param fields
     * @returns [] - array of project objects with/without quota populated.
     */
    getProjectList: async function(fetchQuota, fields) {
      try {
        const response = await axios.get(`/api/project/?${fields?'fields='+fields:''}${fetchQuota? 'expand=quota': ''}&limit=150`);
        return response.data.results;
      } catch (error) {
        console.error('[project.services.getProjectList]',error);
      }
    },

    getResourceUnitType: async function(resource_type_id, resourceTypes){
      let res_unit_type = '';
      try{
        await resourceTypes.forEach(resourcetype => {
          if(resourcetype.name === resource_type_id){
            res_unit_type = resourcetype.quantity_value;
            return res_unit_type;
          }
        });
      } catch (error) {
        console.error('[project.services.getResourceUnitType]',error);
      }
      return res_unit_type;
    },
  
    getUpdatedProjectQuota: async function(projects) {
      let results = {};
      try{
        if(projects){
          await this.getResources()
          .then(resourcetypes =>{
              results.resourcetypes = resourcetypes;
          })
          .then( async ()=>{
            for(const project of projects){
              for(const  quota of project.quota){
                    const resourceType = _.find(results.resourcetypes, ["name", quota.resource_type_id]);
                    project[quota.resource_type_id] = resourceType?UnitConverter.getUIResourceUnit(resourceType.quantity_value, quota.value):quota.value;
              }
              projects.map((pro,index) => {
                if(pro.name === project.name){
                  project['actionpath']= `/project/view/${project.name}`;
                  projects[index] = project;
                }
                return pro;
              });
              project.quota = _.map(project.quota, quota => quota.url);
            }
          });
          results.projects = projects;
          return results.projects;
        }
      } catch (error) {
        console.error('[project.services.getUpdatedProjectQuota]',error);
      }
      return results;
    },
    /**
     * Function returns list of roles that the user has in the particular project
     * @param {String} projectName - Name of the project for which roles need to be fetched
     * @returns [] -  list of roles that the user has in this project
     */
    getRolesForProject: async function(projectName) {
      try {
        const response = await axios.get(`/api/project/${projectName}/my_roles`);
        return response.data;
      } catch (error) {
        console.error('[project.services.getRolesforProject]',error);
        return [];
      }
    },
    /**
     * Function returns list of users having friend role for the particular project
     * @param {*} projectName - Name of the project for which list of friends need to be fetched
     * @returns [] -  list of users having 'friend of project' role for this project
     */
    getFriendsOfProject: async function(projectName) {
      try {
        const response = await axios.get(`/api/project/${projectName}/friend`);
        return response.data;
      } catch (error) {
        if (error?.response?.data?.indexOf("Keycloak admin API token could not be obtained")>0) {
          console.info('The Friend list does not work. This is expected behavoir on Dev.',error);
        } else
        {
          console.error('[project.services.getFriendsforProject]',error);
        }
        return [];
      }
    },
    getMyRoles: async function() {
      try {
        const response = await axios.get(`/api/my_roles_overview/`);
        return response.data;
      } catch (error) {
        console.error('[project.services.getMyRoles]',error);
        return [];
      }
    }
}

export default ProjectService;