

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, fireEvent ,screen, waitFor} from '@testing-library/react';





import { MockAxiosSecurity  } from './axiosmocks.'
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { MemoryRouter } from "react-router-dom/cjs/react-router-dom.min";
import AffectedTasksSelector from "../routes/SystemEvent/affeted.tasks.selector";
import blueprintsearchresult from "../__mocks__/blueprint.expanded.affectedtasks.json"
import { remove404Errors } from "../utils/test.helper";

function MakePrimaryMock() {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    let mock = new MockAdapter(axios);
    mock.onGet("/api/util/utc").reply(200, "2023-12-24T22:58:22.413683");
    mock.onGet("/api/task_blueprint/?fields=id&limit=1").reply(200,{   "count": 103,   "results": [     {         "id": 26      }       ]    },{"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, cancel, copy_to_new_blueprint, copy_to_new_draft, create_subtasks, mark_as_obsolete, predecessors, restart_when_on_hold, schedule_independent_subtasks, successors, system_events"} );
    mock.onGet("/api/task_blueprint/?expand=scheduling_unit_blueprint,scheduling_unit_blueprint.draft.scheduling_set,subtasks&fields=id,name,short_description,on_sky_start_time,on_sky_stop_time,scheduling_unit_blueprint.id,scheduling_unit_blueprint.draft.scheduling_set.project_id,scheduling_unit_blueprint.name,subtasks.id,subtasks.primary&scheduling_unit_blueprint=123&name=observation&ordering=id&limit=103&offset=0").reply(200, blueprintsearchresult);
    mock.onGet("/api/task_blueprint/?expand=scheduling_unit_blueprint,scheduling_unit_blueprint.draft.scheduling_set,subtasks&fields=id,name,short_description,on_sky_start_time,on_sky_stop_time,scheduling_unit_blueprint.id,scheduling_unit_blueprint.draft.scheduling_set.project_id,scheduling_unit_blueprint.name,subtasks.id,subtasks.primary&scheduling_unit_blueprint=123&name=observation&ordering=id&limit=0&offset=0").reply(200, blueprintsearchresult);
    mock.onGet("/api/task_blueprint/?expand=scheduling_unit_blueprint,scheduling_unit_blueprint.draft.scheduling_set,subtasks&fields=id,name,short_description,on_sky_start_time,on_sky_stop_time,scheduling_unit_blueprint.id,scheduling_unit_blueprint.draft.scheduling_set.project_id,scheduling_unit_blueprint.name,subtasks.id,subtasks.primary&task_type=observation&name=observation&ordering=id&limit=103&offset=0").reply(200, blueprintsearchresult);  
    return mock

}


describe('AffectedTasksSelector  ', () => {  
    it('Gets Suid from correct Node', async () => {
        let {   component } = await RenderAffectedTasksSelector(true,true,{ suId:"456",searchBy:'SUB',startTime:'01-01-2024',stopTime:'01-0-2024'});
        let searchInputText = screen.getByTestId("SuidInputText")
        expect(searchInputText).toHaveValue("456")
    
        await act(async () =>  fireEvent.change(searchInputText,{ target: { value: '123'} }));
        expect(searchInputText).toHaveValue("123")

        let radioinput = screen.getByTestId("taskradiobutton")
        await act(async () =>  fireEvent.click(radioinput));

        let subradioinput = screen.getByTestId("subradiobutton")
        await act(async () =>  fireEvent.click(subradioinput));
        await act(async () =>  fireEvent.click(radioinput));
        const searchButton = screen.getByTestId("searchAffected")
        await act(async () =>  fireEvent.click(searchButton));
        expect(component.asFragment()).toMatchSnapshot("After Search Button on Task");
        component.unmount();
    });


     

    it('AffectedTasksSelector   renders correctly without security', async () => {
        let mock= new MockAdapter(axios);
        mock.onGet("/").reply(404);
        remove404Errors();
        let {  component } = await RenderAffectedTasksSelector(false);
        const componentSnapshot = component.asFragment();
        const inputElement = screen.getByTestId('SuidInputText');
        await waitFor(() => {    expect(inputElement.value).toBe('123');  });
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });
  
    it('AffectedTasksSelector  correctly with data but no security ', async () => {
        remove404Errors();
        let {  component } = await RenderAffectedTasksSelector(true);
        const inputElement = screen.getByTestId('SuidInputText');
        await waitFor(() => {    expect(inputElement.value).toBe('123');  });
        expect(component.asFragment()).toMatchSnapshot();
        
        component.unmount();
    });


    it('AffectedTasksSelector correctly with data and security', async () => {


        let {  instance, component  } = await RenderAffectedTasksSelector(true,true);
        expect(component.asFragment()).toMatchSnapshot();

        let searchInputText = screen.getByTestId("SuidInputText")
        expect(searchInputText).toHaveValue("123")
        
       
        const searchButton = screen.getByTestId("searchAffected")
        await act(async () =>  fireEvent.click(searchButton));

        await act(()=>instance.doSearch());
        expect(component.asFragment()).toMatchSnapshot("After Search No Result");
        
        await act(()=>instance.doSearch());
        expect(component.asFragment()).toMatchSnapshot("After Search With Result");
        
        const clearButton = screen.getByTitle("Clear search field")
        await act(async () =>  fireEvent.click(clearButton));
        expect(component.asFragment()).toMatchSnapshot("After Clear Button");
        
        component.unmount();
    });






    
});

async function RenderAffectedTasksSelector(isWisecurity=false,iswithMock=false,dialogfilters=[]) {
    
    let mock
    if (!isWisecurity && iswithMock) {
         mock = MakePrimaryMock();
    }
    if (isWisecurity && iswithMock) {
       mock= MockAxiosSecurity(MakePrimaryMock());
       await PermissionStackUtil.getPermissions(true);
    }
    
    let component;
    let instance;
    let syncfilter = jest.fn();
    let onRowSelection = jest.fn();
    await act(async () => {
        component = render(<MemoryRouter> <AffectedTasksSelector  dialogFilters={dialogfilters} history={{}}  ref={(Instance) => { instance = Instance; } } suId="123"  synchFilter={syncfilter} onRowSelection={onRowSelection}/> </MemoryRouter>);
    });
    return { instance, component ,mock};
}


