import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act } from '@testing-library/react';
import scheduling_unit_draft_extended from '../__mocks__/scheduling_unit_draft_extended.json'
import scheduling_unit_draft_list from '../__mocks__/scheduling_unit_draft_list.json'
import priority_queue_type from '../__mocks__/priority_queue_type.json'
import common_schema_stations from '../__mocks__/common_schema_template_stations.json'
import  projects  from '../__mocks__/projects.json'


import { MockAxiosSecurity  } from './axiosmocks.'
import { MemoryRouter } from "react-router-dom";
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import Scheduling from "../routes/Scheduling";
import { SchedulingUnitListLite } from "../routes/Scheduling/SchedulingUnitListLite";
import { remove404Errors } from "../utils/test.helper";


function MakePrimaryMock() {
    let mock = new MockAdapter(axios);
    mock.onOptions("/api/scheduling_unit_draft_extended/").reply(200, scheduling_unit_draft_extended);
    mock.onGet("/api/priority_queue_type/").reply(200, priority_queue_type);
    mock.onGet("/api/common_schema_template?name=stations&ordering=-id&state=active").reply(200, common_schema_stations);
    mock.onGet("/api/scheduling_unit_draft/?expand=scheduling_set,scheduling_set.project,observation_strategy_template,task_drafts,task_drafts.specifications_template,scheduling_unit_blueprints&fields=id,scheduling_unit_blueprints.name,scheduling_unit_blueprints.url,scheduling_unit_blueprints.name,scheduling_unit_blueprints_ids,scheduling_set.name,scheduling_set.project.name,scheduling_set.project.rank,name,description,priority,priority_queue_value,rank,scheduling_unit_blueprints.id,observation_strategy_template.id,observation_strategy_template.name,observation_strategy_template.description,duration,do_cancel,output_pinned,created_at,updated_at,task_drafts.id,task_drafts.name,task_drafts.output_pinned,task_drafts.url,task_drafts.specifications_doc,task_drafts.specifications_template.id,task_drafts.specifications_template.name,task_drafts.specifications_template.type_value,scheduling_constraints_template,observation_strategy_template.scheduling_unit_template,scheduling_constraints_doc&ordering=name&limit=10&offset=0").reply(200, scheduling_unit_draft_list);
    mock.onGet("/api/scheduling_unit_draft/6740/?fields=name&limit=1").reply(404, {"detail":"Not found."});
    mock.onGet("/api/scheduling_unit_draft/1839/?fields=name&limit=1").reply(200, {"name":"A"},{ "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_via_draft, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft" });
    mock.onGet("/api/scheduling_unit_draft/1019/?fields=name&limit=1").reply(200, {"name":"B"},{ "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_via_draft, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft" });
    mock.onGet("/api/scheduling_unit_draft/528/?fields=name&limit=1").reply(200, {"name":"C"},{ "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_via_draft, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft" });
    mock.onGet("/api/scheduling_unit_draft/1580/?fields=name&limit=1").reply(200, {"name":"D"},{ "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_via_draft, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft" });
    
    
    mock.onGet("/api/project/?fields=name,url&limit=150").reply(200, projects,{ "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, friend, my_roles, report" });

    
    return mock
}

describe('Schedulinng Unit List ', () => {

    it('Schedulinng List  correctly with data', async () => {
        let {  component } = await RenderList();
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();
    });

    it('Schedulinng List renders correctly without proper data', async () => {
        MakePrimaryMock();
        remove404Errors();
        let component
        await act(async () => { component = render(<MemoryRouter><SchedulingUnitListLite  match={{}}  history={{}} location="/schedulingunit/lite" /></MemoryRouter>) });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });
});



describe('Schedulinng Unit List Lite ', () => {

    it('Schedulinng List Lite  correctly with data', async () => {
        remove404Errors(); // we do have a special one here, we have on purpose a 404
        let {  component } = await RenderList();
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();
    });

    it('Schedulinng List Lite renders correctly without proper data', async () => {
        MakePrimaryMock();
        remove404Errors();
        let component
        await act(async () => { component = render(<MemoryRouter><SchedulingUnitListLite  match={{}}  history={{}} location="/schedulingunit/lite" /></MemoryRouter>) });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });
});



async function RenderList() {
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component;
    let ListInstance;
    await act(async () => {
        component = render(<MemoryRouter><Scheduling history={{}} location="/schedulingunit" ref={(instance) => { ListInstance = instance; } } /></MemoryRouter>);
    });
    return { ListInstance, component };
}

