

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act } from '@testing-library/react';
import { MockAxiosSecurity  } from './axiosmocks.'
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { MemoryRouter } from "react-router-dom/cjs/react-router-dom.min";
import WorkflowList from "../routes/Workflow/workflow.list";
import qaschedulingunittask from "../__mocks__/qa_scheduling_unit_task.json";
import qaschedulingunitblueprint from "../__mocks__/qa_scheduling_unit_blueprint.json";
import qaschedulingunitprocess from "../__mocks__/qa_scheduling_unit_process.json";
import { remove404Errors } from "../utils/test.helper";

function MakePrimaryMock() {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    let mock = new MockAdapter(axios);
    mock.onGet("/api/util/utc").reply(200, "2023-12-24T22:58:22.413683");
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/?&status=NEW&status=ASSIGNED&&ordering=created&limit=10&offset=0").reply(200, qaschedulingunittask);
    mock.onGet("/api/scheduling_unit_blueprint/?id=1&expand=draft,draft.scheduling_set&fields=id,name,status,unschedulable_reason,draft.scheduling_set.project_id").reply(200, qaschedulingunitblueprint);
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/?&status=NEW&status=ASSIGNED&&limit=10&offset=0").reply(200, qaschedulingunittask);
     mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/?id=1").reply(200, qaschedulingunitprocess);
    
    return mock
}

describe('Workflow List ', () => {
    it('Workflow  List  renders correctly without security', async () => {
        remove404Errors();
        let {  component } = await RenderWorkflow(false);
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('Workflow  List correctly with data but no security ', async () => {
        remove404Errors();
        let {  component } = await RenderWorkflow(true);
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });

    it('Workflow List correctly with data and security', async () => {
        let {  component  } = await RenderWorkflow(true,true);
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });
});

    describe('Workflow List handle WebSocket', () => {
        it('Workflow handles correctly Websocket Events', async () => {
        let {  component,instance  } = await RenderWorkflow(true,true);
        instance.handleData(JSON.stringify({ object_type: 'scheduling_unit_blueprint',  object_details:{id:'1'}}))
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });
});

async function RenderWorkflow(isWisecurity=false,iswithMock=false) {
    let mock
    if (!isWisecurity && iswithMock) {
         mock = MakePrimaryMock();
    }
    if (isWisecurity && iswithMock) {
       mock= MockAxiosSecurity(MakePrimaryMock());
       await PermissionStackUtil.getPermissions(true);
    }
    let component;
    let instance;
    await act(async () => {
        component = render(<MemoryRouter> <WorkflowList  match={{ }}  history={{}}  ref={(rinstance) => { instance = rinstance; }} location="/su/workflow"  /> </MemoryRouter>);
    });
    return { instance, component ,mock};
}


