

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, fireEvent } from '@testing-library/react';
import { MockAxiosSecurity   } from './axiosmocks.'
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { MemoryRouter } from "react-router-dom/cjs/react-router-dom.min";

import qaschedulingunitblueprintextended from "../__mocks__/qa_scheduling_unit_blueprint_extended.json";
import chedulingunitdraft from "../__mocks__/qa_scheduling_unit_draft.json";
import project_role from "../__mocks__/project_role.json";
import scheduling_unit_set from "../__mocks__/qa_scheduling_unit_set.json";
import qa_scheduling_unit_taskall from "../__mocks__/qa_scheduling_unit_taskall_.json";
import qa_scheduling_unit_blueprint_exttended_withtasks from "../__mocks__/qa_scheduling_unit_blueprint_exttended_withtasks.json";
import qa_scheduling_unit_blueprint_extended_subtasks from "../__mocks__/qa_scheduling_unit_blueprint_extended_subtasks.json";
import { remove404Errors } from "../utils/test.helper";
import Workflow from "../routes/Workflow";
import { LocalStorageMock  } from "./localstoragemocks";


const  qaflowitem = {
    "id": 1,
    "flow_class": "<class 'lofar.sas.tmss.tmss.workflowapp.flows.schedulingunitflow.SchedulingUnitFlow'>",
    "status": "NEW",
    "created": "2024-01-12T10:32:09.846089",
    "finished": null,
    "artifact_object_id": null,
    "data": null,
    "artifact_content_type": null,
    "su": 1,
    "qa_reporting_to": null,
    "qa_reporting_sos": null,
    "pi_verification": null,
    "decide_acceptance": null,
    "unpin_data": null
}
const flowstep =     {
        "count": 1,
        "next": null,
        "previous": null,
        "results": [
            qaflowitem
        ]
    }

    const currenTask = 
        [
            {
                "id": 4,
                "flow_task": "workflowapp/flows.schedulingunitflow.SchedulingUnitFlow.qa_reporting_to",
                "flow_task_type": "HUMAN",
                "status": "NEW",
                "created": "2024-01-14T15:07:13.048",
                "assigned": null,
                "started": null,
                "finished": null,
                "token": "start",
                "process_id": 1,
                "artifact_content_type_id": null,
                "artifact_object_id": null,
                "owner_id": null,
                "external_task_id": null,
                "owner_permission": "workflowapp.can_qa_reporting_to_schedulingunitprocess",
                "comments": null,
                "data": null,
                "task_ptr_id": 4
            }
        ]
    
        const currentTaskAfterAssign = 
        [
            {
                "id": 4,
                "flow_task": "workflowapp/flows.schedulingunitflow.SchedulingUnitFlow.qa_reporting_to",
                "flow_task_type": "HUMAN",
                "status": "ASSIGNED",
                "created": "2024-01-25T12:54:19.655",
                "assigned": "2024-01-25T13:48:58.180",
                "started": null,
                "finished": null,
                "token": "start",
                "process_id": 1,
                "artifact_content_type_id": null,
                "artifact_object_id": null,
                "owner_id": 1,
                "external_task_id": null,
                "owner_permission": "workflowapp.can_qa_reporting_to_schedulingunitprocess",
                "comments": null,
                "data": null,
                "task_ptr_id": 4
            }
        ]
        const qaitem = {
            "id": 4,
            "owner_email": null,
            "owner_username": null,
            "flow_task": "Qa Reporting To",
            "flow_task_type": "HUMAN",
            "status": "NEW",
            "created": "2024-01-14T15:07:13.048474",
            "assigned": null,
            "started": null,
            "finished": null,
            "token": "start",
            "artifact_object_id": null,
            "external_task_id": null,
            "owner_permission": "workflowapp.can_qa_reporting_to_schedulingunitprocess",
            "comments": null,
            "data": null,
            "process": 1,
            "artifact_content_type": null,
            "owner": null,
            "previous": [
                3
            ]
        }

        const qaitem_assigned = 
        {
            "id": 4,
            "owner_email": "test@lofar.test",
            "owner_username": "test",
            "flow_task": "Qa Reporting To",
            "flow_task_type": "HUMAN",
            "status": "ASSIGNED",
            "created": "2024-01-25T12:54:19.655440",
            "assigned": "2024-01-25T13:48:58.180635",
            "started": null,
            "finished": null,
            "token": "start",
            "artifact_object_id": null,
            "external_task_id": null,
            "owner_permission": "workflowapp.can_qa_reporting_to_schedulingunitprocess",
            "comments": null,
            "data": null,
            "process": 1,
            "artifact_content_type": null,
            "owner": 1,
            "previous": [
                3
            ]
        }

   
        
const affecttasks = {
    "count": 1,
    "next": null,
    "previous": null,
    "results": [
        {
            "id": 1,
            "url": "http://localhost:3000/api/system_event/1",
            "affected_hardware_doc": {
                "clusters": [
                    "COBALT2"
                ],
                "stations": [
                    "CS001",
                    "CS002",
                    "CS003"
                ]
            },
            "affected_hardware_template": "http://localhost:3000/api/system_event_template/1",
            "affected_hardware_template_id": 1,
            "affected_tasks": [
                "http://localhost:3000/api/task_blueprint/1",
                "http://localhost:3000/api/task_blueprint/3",
                "http://localhost:3000/api/task_blueprint/5"
            ],
            "affected_tasks_ids": [
                1,
                3,
                5
            ],
            "created_at": "2024-01-25T14:31:36.986044",
            "created_by": "test@lofar.test",
            "created_by_id": 1,
            "description": "Desc",
            "issue_subtype": "http://localhost:3000/api/system_event_subtype/crash",
            "issue_subtype_value": "crash",
            "issue_type": "http://localhost:3000/api/system_event_type/cep",
            "issue_type_value": "cep",
            "jira_url": null,
            "name": "A name",
            "notes": "test",
            "severity": "http://localhost:3000/api/system_event_severity/minor",
            "severity_value": "minor",
            "start": "2024-01-26T06:35:15",
            "status": "http://localhost:3000/api/system_event_status/closed",
            "status_value": "closed",
            "stop": "2024-01-25T15:31:36",
            "tags": [],
            "updated_at": "2024-01-25T14:31:36.986050"
        }
    ]
}
function MakePrimaryMock() {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    let mock = new MockAdapter(axios);
    mock.onGet("/api/scheduling_unit_blueprint_extended/1/").reply(200,  qaschedulingunitblueprintextended, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_in_draft, create_copies_of_failed_tasks_via_draft, create_lofar2_sibling, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft"});
    mock.onGet("/api/project_role/").reply(200,  project_role, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE"});
    mock.onGet("/api/scheduling_unit_draft/14/").reply(200,  chedulingunitdraft, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, copy, create_cleanuptask_for_scheduling_unit_draft, create_observing_strategy_template, create_scheduling_unit_blueprint_and_tasks_and_subtasks_tree, scheduling_constraints_plot, specifications_doc, update_task_graph_from_specifications_doc"});
    mock.onGet("/api/scheduling_set/3/?expand=project").reply(200,  scheduling_unit_set, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, copy"});
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/?su=1").reply(200,  flowstep );
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/?ordering=-id&process=1").reply(200,  qa_scheduling_unit_taskall );
    mock.onGet("/api/scheduling_unit_blueprint/1/?expand=task_blueprints,task_blueprints.subtasks,task_blueprints.subtasks.output_dataproducts&fields=task_blueprints.id,task_blueprints.subtasks.id,task_blueprints.subtasks.output_dataproducts.feedback_doc").reply(200,  qa_scheduling_unit_blueprint_extended_subtasks,{"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_in_draft, create_copies_of_failed_tasks_via_draft, create_lofar2_sibling, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft"} );
    mock.onGet("/api/task_blueprint/3/successors/?expand=subtasks").reply(200,  qa_scheduling_unit_blueprint_exttended_withtasks );
    mock.onGet("/api/task_blueprint/1/successors/?expand=subtasks").reply(200,  qa_scheduling_unit_blueprint_exttended_withtasks );
    mock.onGet("/api/task_blueprint/5/successors/?expand=subtasks").reply(200,  qa_scheduling_unit_blueprint_exttended_withtasks );
    mock.onPost("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/1/current_task/").reply(200,  currenTask);
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/1").reply(200,  qaflowitem);
    
    mock.onGet("/api/system_event/?affected_tasks=3&affected_tasks=7&affected_tasks=1&affected_tasks=8&affected_tasks=5&affected_tasks=4&affected_tasks=2&affected_tasks=6&undefined&limit=0&offset=0").reply(200,affecttasks);
    mock.onGet("/api/subtask/2000002/missing_stations").reply(200,  {"missing_stations": []} );
    mock.onGet("/api/system_event/?limit=1").reply(200, {"count":0,"next":null,"previous":null,"results":[]});
    mock.onGet("/api/util/utc").reply(200, "2023-12-24T22:58:22.413683");

    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/4").reply(200,  qaitem,{"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE"});
//{"count":0,"next":null,"previous":null,"results":[]}    
    mock.onPost("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/4/assign/?self=true").reply(200,  {"Assigned":"Scheduling Unit Task assigned to user=test@lofar.test"},{"access-control-allow-methods":"POST, OPTIONS"});
    
    return mock
}

describe('Workflow  ', () => {

    it('Workflow   renders correctly without security', async () => {
        let mock= new MockAdapter(axios);
        mock.onGet("/").reply(404);
        remove404Errors();
        let {  component } = await RenderWorkflow(false);
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('Workflow  correctly with data but no security ', async () => {
        remove404Errors();
        let {  component } = await RenderWorkflow(true);
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });

    it('Workflow correctly with data and security', async () => {
        let {  component ,mock } = await RenderWorkflow(true,true);
        expect(component.asFragment()).toMatchSnapshot();
        const assignLink = component.getByText('Assign to me');
        mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/1/current_task/").reply(200,  currentTaskAfterAssign);
        
        mock.onPost("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/4").reply(200,  qaitem_assigned,{"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE"});
        mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/4").reply(200,  qaitem_assigned,{"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE"});

        fireEvent.click(assignLink)
        expect(component.asFragment()).toMatchSnapshot("Assigned");


        const next  = component.getByText('Next');
        fireEvent.click(next)
        expect(component.asFragment()).toMatchSnapshot("Next");
        
        

        component.unmount();
    });


});



async function RenderWorkflow(isWisecurity=false,iswithMock=false) {
    global.localStorage = new LocalStorageMock;
    let mock
    
    if (!isWisecurity && iswithMock) {
         mock = MakePrimaryMock();
    }
    if (isWisecurity && iswithMock) {
        
        mock= MockAxiosSecurity(MakePrimaryMock());
       await PermissionStackUtil.getPermissions(true);
    }
    
    let component;
    let instance;
    await act(async () => {
        component = render(<MemoryRouter> <Workflow  match={{ params: { id: "1" } }} history={{}} location="/schedulingunit/1/workflow"  /> </MemoryRouter>);
    });
    return { instance, component ,mock};
}


