

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act, fireEvent } from '@testing-library/react';
import { MockAxiosSecurity } from '../../axiosmocks.'
import PermissionStackUtil from "../../../authenticate/permission.stack.handler";
import { MemoryRouter } from "react-router-dom/cjs/react-router-dom.min";
import { createMemoryHistory } from 'history';
import qaschedulingunitblueprintextended from "./apiScheduling_unit_blueprint_One.json";
import chedulingunitdraft from "./apiScheduling_unit_draft.json";
import project_role from "../../../__mocks__/project_role.json";
import scheduling_unit_set from "./apiScheduling_unit_set.json";
import inSystemevent from "../stepfour/systemevent.json"
import qa_scheduling_unit_taskall from "./workflowapi.schedulingunit_flow.json";
import Workflow from "../../../routes/Workflow";
import { LocalStorageMock } from "../.././localstoragemocks";
import systemeventSeverity from "../stepfour/systemeventseverity.json";
import systemeventStatus from "../stepfour/systemeventseverity.json";
import systemeventSubType from "../stepfour/systemeventsubtype.json";
import systemeventType from "../stepfour/systemeventtype.json";
import affectedTask from "./affectedtask.json";
import systemeventTemplate from "../stepfour/systemeventtemplate.json";

const taskeight =
{
    "id": 8,
    "owner_email": null,
    "owner_username": null,
    "flow_task": "Pi Verification",
    "flow_task_type": "HUMAN",
    "status": "NEW",
    "created": "2024-01-31T22:49:42.221210",
    "assigned": null,
    "started": null,
    "finished": null,
    "token": "start",
    "artifact_object_id": null,
    "external_task_id": null,
    "owner_permission": "workflowapp.can_pi_verification_schedulingunitprocess",
    "comments": null,
    "data": null,
    "process": 1,
    "artifact_content_type": null,
    "owner": null,
    "previous": [
        7
    ]
}

const currenTask =
    [
        {
            "id": 8,
            "flow_task": "workflowapp/flows.schedulingunitflow.SchedulingUnitFlow.pi_verification",
            "flow_task_type": "HUMAN",
            "status": "NEW",
            "created": "2024-01-31T22:49:42.221",
            "assigned": null,
            "started": null,
            "finished": null,
            "token": "start",
            "process_id": 1,
            "artifact_content_type_id": null,
            "artifact_object_id": null,
            "owner_id": null,
            "external_task_id": null,
            "owner_permission": "workflowapp.can_pi_verification_schedulingunitprocess",
            "comments": null,
            "data": null,
            "task_ptr_id": 8
        }
    ]





const systemEvent =
{
    "count": 1,
    "next": null,
    "previous": null,
    "results": [
        {
            "id": 1,
            "url": "http://localhost:3000/api/system_event/1",
            "affected_hardware_doc": {
                "clusters": [
                    "COBALT2"
                ],
                "stations": [
                    "CS001",
                    "CS002",
                    "CS003"
                ]
            },
            "affected_hardware_template": "http://localhost:3000/api/system_event_template/1",
            "affected_hardware_template_id": 1,
            "affected_tasks": [
                "http://localhost:3000/api/task_blueprint/1",
                "http://localhost:3000/api/task_blueprint/3",
                "http://localhost:3000/api/task_blueprint/5"
            ],
            "affected_tasks_ids": [
                1,
                3,
                5
            ],
            "created_at": "2024-01-25T14:31:36.986044",
            "created_by": "test@lofar.test",
            "created_by_id": 1,
            "description": "Desc",
            "issue_subtype": "http://localhost:3000/api/system_event_subtype/crash",
            "issue_subtype_value": "crash",
            "issue_type": "http://localhost:3000/api/system_event_type/cep",
            "issue_type_value": "cep",
            "jira_url": null,
            "name": "A name",
            "notes": "test",
            "severity": "http://localhost:3000/api/system_event_severity/minor",
            "severity_value": "minor",
            "start": "2024-01-26T06:35:15",
            "status": "http://localhost:3000/api/system_event_status/closed",
            "status_value": "closed",
            "stop": "2024-01-25T15:31:36",
            "tags": [],
            "updated_at": "2024-01-25T14:31:36.986050"
        }
    ]
}
const flowstep = {
    "count": 1,
    "next": null,
    "previous": null,
    "results": [
        {
            "id": 1,
            "flow_class": "<class 'lofar.sas.tmss.tmss.workflowapp.flows.schedulingunitflow.SchedulingUnitFlow'>",
            "status": "NEW",
            "created": "2024-01-31T22:38:02.945481",
            "finished": null,
            "artifact_object_id": null,
            "data": null,
            "artifact_content_type": null,
            "su": 1,
            "qa_reporting_to": 1,
            "qa_reporting_sos": 1,
            "pi_verification": null,
            "decide_acceptance": null,
            "unpin_data": null
        }
    ]
}


function MakePrimaryMock() {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    let mock = new MockAdapter(axios);
    const apd = { "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE" }
    mock.onGet("/api/util/utc").reply(200, "2023-12-24T22:58:22.413683");
    mock.onGet("/api/project_role/").reply(200, project_role, apd);
    mock.onGet("/api/scheduling_unit_blueprint_extended/1/").reply(200, qaschedulingunitblueprintextended, { "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_in_draft, create_copies_of_failed_tasks_via_draft, create_lofar2_sibling, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft" });
    mock.onGet("/api/scheduling_unit_draft/14/").reply(200, chedulingunitdraft, { "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, copy, create_cleanuptask_for_scheduling_unit_draft, create_observing_strategy_template, create_scheduling_unit_blueprint_and_tasks_and_subtasks_tree, scheduling_constraints_plot, specifications_doc, update_task_graph_from_specifications_doc" });
    mock.onGet("/api/scheduling_set/1/?expand=project").reply(200, scheduling_unit_set, { "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, copy" });
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/?su=1").reply(200, flowstep);
    mock.onOptions("/api/system_event/").reply(200, inSystemevent);
    mock.onGet("/api/system_event/?limit=1").reply(200, systemEvent);
    mock.onPost("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/1/current_task/").reply(200, currenTask)
    mock.onGet("/api/system_event_template").reply(200, systemeventTemplate, { "access-control-allow-methods": "GET, POST, PUT, PATCH, DELETE, default, ref_resolved_schema, schema, validate_specifications_doc" });
    mock.onGet("/api/system_event_type").reply(200, systemeventType, apd);
    mock.onGet("/api/system_event_subtype").reply(200, systemeventSubType, apd);
    mock.onGet("/api/system_event_severity").reply(200, systemeventSeverity, apd);
    mock.onGet("/api/system_event_status").reply(200, systemeventStatus, apd);
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/1").reply(200, {
        "id": 1,
        "flow_class": "<class 'lofar.sas.tmss.tmss.workflowapp.flows.schedulingunitflow.SchedulingUnitFlow'>",
        "status": "NEW",
        "created": "2024-01-31T22:38:02.945481",
        "finished": null,
        "artifact_object_id": null,
        "data": null,
        "artifact_content_type": null,
        "su": 1,
        "qa_reporting_to": 1,
        "qa_reporting_sos": 1,
        "pi_verification": null,
        "decide_acceptance": null,
        "unpin_data": null
    }, apd);


    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/?ordering=-id&process=1").reply(200, qa_scheduling_unit_taskall, apd);

    mock.onGet("/api/system_event/?affected_tasks=3&affected_tasks=1&affected_tasks=4&affected_tasks=5&affected_tasks=8&affected_tasks=7&affected_tasks=2&affected_tasks=6&limit=1&offset=0").reply(200, affectedTask, apd);
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/8").reply(200, taskeight, apd);

    mock.onGet("/workflow_api/scheduling_unit_flow/qa_reporting_sos/1").reply(200, {
        "id": 1,
        "sos_report": "<p>Subject- Observation Success</p><p>Dear Colleague,</p><p> The following message contains information regarding a LOFAR Cycle test project for which you are listed as the contact author.            Please forward this information to your collaborators.</p><p> We would like to inform you that observations related to your LOFAR Cycle test project have been performed. Please find detailed information below.</p><p><strong>General notes:</strong></p><p> The validation plots linked below remains online for 3 weeks from the date of observation. After that, they are compressed and transferred to offline storage. users can request access to them by submitting a ticket through the <a href=\"https://support.astron.nl/sdchelpdesk\">helpdesk</a>.</p><p><strong>Observations:</strong></p><table><thead><tr><th><div>Observation ID</div></th><th><div> Scheduling Unit ID</div></th><th><div> Task ID</div></th><th><div> Pipeline ID</div></th><th><div> Project</div></th><th><div> Target Name</div></th><th><div> Station plots</div></th><th><div> Completeness</div></th><th><div> Antenna Set</div></th><th><div> Filter</div></th><th><div> Start Time</div></th><th><div> End time</div></th><th><div> Number of Sub-bands</div></th><th><div> Number of files</div></th><th><div> Parset</div></th></tr></thead><tbody><tr><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000000/index.html\">2000000</a></div></td><td><div>1</div></td><td><div>1</div></td><td><div>2000001</div></td><td><div>high</div></td><td><div><br></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000000/Stations/station_beamlets.html\">BST</a></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000000/index.html\">0%</a></div></td><td><div>HBA_DUAL_INNER</div></td><td><div>HBA_110_190</div></td><td><div>2024-02-01 15:15:21</div></td><td><div>2024-02-01 15:25:21</div></td><td><div>243</div></td><td><div>243</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/2000000/rtcp-2000000.parset\">parset</a></div></td></tr><tr><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000002/index.html\">2000002</a></div></td><td><div>1</div></td><td><div>3</div></td><td><div>2000003</div></td><td><div>high</div></td><td><div>P306+46</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000002/Stations/station_beamlets.html\">BST</a></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000002/index.html\">0%</a></div></td><td><div>HBA_DUAL_INNER</div></td><td><div>HBA_110_190</div></td><td><div>2024-02-01 07:14:21</div></td><td><div>2024-02-01 15:14:21</div></td><td><div>243</div></td><td><div>243</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/2000002/rtcp-2000002.parset\">parset</a></div></td></tr><tr><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000004/index.html\">2000004</a></div></td><td><div>1</div></td><td><div>5</div></td><td><div>2000005</div></td><td><div>high</div></td><td><div><br></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000004/Stations/station_beamlets.html\">BST</a></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000004/index.html\">0%</a></div></td><td><div>HBA_DUAL_INNER</div></td><td><div>HBA_110_190</div></td><td><div>2024-02-01 07:03:21</div></td><td><div>2024-02-01 07:13:21</div></td><td><div>243</div></td><td><div>243</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/2000004/rtcp-2000004.parset\">parset</a></div></td></tr></tbody></table><p><br></p><p><strong>Performance of the system:</strong><br><br></p><p><strong>Data recording:</strong><br><br></p><p><strong>Data processing:</strong><br><br></p><p><strong>Archiving:</strong><br><br></p><p><strong>Remarks:</strong></p><p> Please analyse the validation plots at             <a href=\"https://proxy.lofar.eu/inspect/HTML/\">https://proxy.lofar.eu/inspect/HTML/</a> within 24 hours after this notification and submit your             findings using the form at <a href=\"/schedulingunit/1/workflow\">http://localhost:3000/schedulingunit/1/workflow </a>.            After this time window has passed, we will assume that your judgement is that the observation was successful and we will complete the actions described above ourselves to support your run.</p><p><br></p><p>From the moment the data are made available to you at the LTA you have four weeks to check their quality and to report any problems to the             Observatory through the <a href=\"https://support.astron.nl/sdchelpdesk\">helpdesk</a>. After this time window has passed, no requests for re-observations will be considered.</p><p><strong>Actions:</strong></p><p>If you need any further clarification, please do not hesitate to contact us through the SDC helpdesk at <a href=\"https://support.astron.nl/sdchelpdesk\">https://support.astron.nl/sdchelpdesk</a>, specifying your project code in the subject.            </p><p>Best regards,</p><p>TO name (Telescope Operations)</p><p>SDCO name (Science Data center Operations)</p>",
        "quality_within_policy": true,
        "sos_accept_show_pi": true
    });
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_reporting_to/1").reply(200, {
        "id": 1,
        "operator_report": "<p>Subject- Observation Success</p><p>Dear Colleague,</p><p> The following message contains information regarding a LOFAR Cycle test project for which you are listed as the contact author.            Please forward this information to your collaborators.</p><p> We would like to inform you that observations related to your LOFAR Cycle test project have been performed. Please find detailed information below.</p><p><strong>General notes:</strong></p><p> The validation plots linked below remains online for 3 weeks from the date of observation. After that, they are compressed and transferred to offline storage. users can request access to them by submitting a ticket through the <a href=\"https://support.astron.nl/sdchelpdesk\">helpdesk</a>.</p><p><strong>Observations:</strong></p><table><thead><tr><th><div>Observation ID</div></th><th><div> Scheduling Unit ID</div></th><th><div> Task ID</div></th><th><div> Pipeline ID</div></th><th><div> Project</div></th><th><div> Target Name</div></th><th><div> Station plots</div></th><th><div> Completeness</div></th><th><div> Antenna Set</div></th><th><div> Filter</div></th><th><div> Start Time</div></th><th><div> End time</div></th><th><div> Number of Sub-bands</div></th><th><div> Number of files</div></th><th><div> Parset</div></th></tr></thead><tbody><tr><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000000/index.html\">2000000</a></div></td><td><div>1</div></td><td><div>1</div></td><td><div>2000001</div></td><td><div>low</div></td><td><div><br></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000000/Stations/station_beamlets.html\">BST</a></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000000/index.html\">0%</a></div></td><td><div>HBA_DUAL_INNER</div></td><td><div>HBA_110_190</div></td><td><div>2024-01-26 14:20:45</div></td><td><div>2024-01-26 14:30:45</div></td><td><div>243</div></td><td><div>243</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/2000000/rtcp-2000000.parset\">parset</a></div></td></tr><tr><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000002/index.html\">2000002</a></div></td><td><div>1</div></td><td><div>3</div></td><td><div>2000003</div></td><td><div>low</div></td><td><div>P306+46</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000002/Stations/station_beamlets.html\">BST</a></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000002/index.html\">0%</a></div></td><td><div>HBA_DUAL_INNER</div></td><td><div>HBA_110_190</div></td><td><div>2024-01-26 06:19:45</div></td><td><div>2024-01-26 14:19:45</div></td><td><div>243</div></td><td><div>243</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/2000002/rtcp-2000002.parset\">parset</a></div></td></tr><tr><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000004/index.html\">2000004</a></div></td><td><div>1</div></td><td><div>5</div></td><td><div>2000005</div></td><td><div>low</div></td><td><div><br></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000004/Stations/station_beamlets.html\">BST</a></div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/HTML/2000004/index.html\">0%</a></div></td><td><div>HBA_DUAL_INNER</div></td><td><div>HBA_110_190</div></td><td><div>2024-01-26 06:08:45</div></td><td><div>2024-01-26 06:18:45</div></td><td><div>243</div></td><td><div>243</div></td><td><div><a href=\"https://proxy.lofar.eu/inspect/2000004/rtcp-2000004.parset\">parset</a></div></td></tr></tbody></table><p><br></p><p><strong>Performance of the system:</strong><br><br></p><p><strong>Data recording:</strong><br><br></p><p><strong>Data processing:</strong><br><br></p><p><strong>Archiving:</strong><br><br></p><p><strong>Remarks:</strong></p><p> Please analyse the validation plots at             <a href=\"https://proxy.lofar.eu/inspect/HTML/\">https://proxy.lofar.eu/inspect/HTML/</a> within 24 hours after this notification and submit your             findings using the form at <a href=\"/schedulingunit/1/workflow\">http://localhost:3000/schedulingunit/1/workflow </a>.            After this time window has passed, we will assume that your judgement is that the observation was successful and we will complete the actions described above ourselves to support your run.</p><p><br></p><p>From the moment the data are made available to you at the LTA you have four weeks to check their quality and to report any problems to the             Observatory through the <a href=\"https://support.astron.nl/sdchelpdesk\">helpdesk</a>. After this time window has passed, no requests for re-observations will be considered.</p><p><strong>Actions:</strong></p><p>If you need any further clarification, please do not hesitate to contact us through the SDC helpdesk at <a href=\"https://support.astron.nl/sdchelpdesk\">https://support.astron.nl/sdchelpdesk</a>, specifying your project code in the subject.            </p><p>Best regards,</p><p>TO name (Telescope Operations)</p><p>SDCO name (Science Data center Operations)</p>",
        "operator_accept": true
    });

    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/1").reply(200, {
        "id": 1,
        "flow_class": "<class 'lofar.sas.tmss.tmss.workflowapp.flows.schedulingunitflow.SchedulingUnitFlow'>",
        "status": "NEW",
        "created": "2024-01-25T20:18:06.720375",
        "finished": null,
        "artifact_object_id": null,
        "data": null,
        "artifact_content_type": null,
        "su": 1,
        "qa_reporting_to": 1,
        "qa_reporting_sos": null,
        "pi_verification": null,
        "decide_acceptance": null,
        "unpin_data": null
    });

    mock.onPost("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/8/assign/?self=true").reply(200, { "Assigned": "Scheduling Unit Task assigned to user=test@lofar.test" });


    return mock
}

describe('Workflow  Step Five', () => {


    it('Workflow Step FIve  Renders correctly with data and security', async () => {
        let {component, mock} = await RenderStepFive();
        expect(component.asFragment()).toMatchSnapshot();
        
          
        


        
        mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/1/current_task/").reply(200, [
            {
                "id": 8,
                "flow_task": "workflowapp/flows.schedulingunitflow.SchedulingUnitFlow.pi_verification",
                "flow_task_type": "HUMAN",
                "status": "ASSIGNED",
                "created": "2024-01-31T22:49:42.221",
                "assigned": "2024-01-31T23:27:11.374",
                "started": null,
                "finished": null,
                "token": "start",
                "process_id": 1,
                "artifact_content_type_id": null,
                "artifact_object_id": null,
                "owner_id": 1,
                "external_task_id": null,
                "owner_permission": "workflowapp.can_pi_verification_schedulingunitprocess",
                "comments": null,
                "data": null,
                "task_ptr_id": 8
            }
        ]
        );

        mock.onGet("http://localhost:3000/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/8").reply(200, {
            "id": 8,
            "owner_email": "test@lofar.test",
            "owner_username": "test",
            "flow_task": "Pi Verification",
            "flow_task_type": "HUMAN",
            "status": "ASSIGNED",
            "created": "2024-01-31T22:49:42.221210",
            "assigned": "2024-01-31T23:27:11.374259",
            "started": null,
            "finished": null,
            "token": "start",
            "artifact_object_id": null,
            "external_task_id": null,
            "owner_permission": "workflowapp.can_pi_verification_schedulingunitprocess",
            "comments": null,
            "data": null,
            "process": 1,
            "artifact_content_type": null,
            "owner": 1,
            "previous": [
                7
            ]
        });


        const assignLink = component.getByText('Assign to me');
        act(() => { fireEvent.click(assignLink); });
        expect(component.asFragment()).toMatchSnapshot("Assigned to Me");

   

        const piAccept = component.container.querySelector('#piAccept');
        act(() => {          fireEvent.click(piAccept);      });
        expect(piAccept.checked).toBe(true);

  

        



        const nextButton = component.getByText('Next');
        act(() => { fireEvent.click(nextButton); });
        const cancelButton = component.getByText('Cancel');
        fireEvent.click(cancelButton);
        expect(component.asFragment()).toMatchSnapshot("After Cancel");
        component.unmount();
    });



});

async function RenderStepFive() {
    global.localStorage = new LocalStorageMock;
    localStorage.setItem("user", JSON.stringify({ name: "test" }));
    let mock = MockAxiosSecurity(MakePrimaryMock());
    const history = createMemoryHistory();
    history.push("/schedulingunit/1/workflow");
    await PermissionStackUtil.getPermissions(true);
    let component;
    await act(async () => {
        component = render(<MemoryRouter> <Workflow match={{ params: { id: "1" } }} history={history} location="/schedulingunit/1/workflow" /> </MemoryRouter>);
    });
    return {component, mock};
}
