

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act,fireEvent } from '@testing-library/react';
import { MockAxiosSecurity   } from '../../axiosmocks.'
import PermissionStackUtil from "../../../authenticate/permission.stack.handler";
import { MemoryRouter } from "react-router-dom/cjs/react-router-dom.min";
import { createMemoryHistory } from 'history';
import qaschedulingunitblueprintextended from "./apiScheduling_unit_blueprint_One.json";
import chedulingunitdraft from "./apiScheduling_unit_draft.json";
import project_role from "../../../__mocks__/project_role.json";
import scheduling_unit_set from "./apiScheduling_unit_set.json";
import qa_scheduling_unit_taskall from "./workflowapi.schedulingunit_flow.json";
import Workflow from "../../../routes/Workflow";
import { LocalStorageMock  } from "../.././localstoragemocks";


const  qaflowitem =  {
    "id": 1,
    "flow_class": "<class 'lofar.sas.tmss.tmss.workflowapp.flows.schedulingunitflow.SchedulingUnitFlow'>",
    "status": "NEW",
    "created": "2024-01-24T21:32:18.966372",
    "finished": null,
    "artifact_object_id": null,
    "data": null,
    "artifact_content_type": null,
    "su": 1,
    "qa_reporting_to": null,
    "qa_reporting_sos": null,
    "pi_verification": null,
    "decide_acceptance": null,
    "unpin_data": null
}
const flowstep =     {
        "count": 1,
        "next": null,
        "previous": null,
        "results": [
            qaflowitem
        ]
    }


function MakePrimaryMock() {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    let mock = new MockAdapter(axios);
    mock.onGet("/api/util/utc").reply(200, "2023-12-24T22:58:22.413683");
    mock.onGet("/api/project_role/").reply(200,  project_role, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE"});
    mock.onGet("/api/scheduling_unit_blueprint_extended/1/").reply(200,  qaschedulingunitblueprintextended, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_in_draft, create_copies_of_failed_tasks_via_draft, create_lofar2_sibling, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft"});
    mock.onGet("/api/scheduling_unit_draft/14/").reply(200,  chedulingunitdraft, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, copy, create_cleanuptask_for_scheduling_unit_draft, create_observing_strategy_template, create_scheduling_unit_blueprint_and_tasks_and_subtasks_tree, scheduling_constraints_plot, specifications_doc, update_task_graph_from_specifications_doc"});
    mock.onGet("/api/scheduling_set/1/?expand=project").reply(200,  scheduling_unit_set, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, copy"});
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_process/?su=1").reply(200,  flowstep );
    mock.onGet("/workflow_api/scheduling_unit_flow/qa_scheduling_unit_task/?ordering=-id&process=1").reply(200,  qa_scheduling_unit_taskall );
    return mock
}

describe('Workflow  Step One', () => {
    it('Workflow Step One Renders correctly with data and security', async () => {
        global.localStorage = new LocalStorageMock; 
        MockAxiosSecurity(MakePrimaryMock());
        const history = createMemoryHistory();
        history.push("/schedulingunit/1/workflow")
        await PermissionStackUtil.getPermissions(true);
        let component
        await act(async () => {
            component = render(<MemoryRouter> <Workflow  match={{ params: { id: "1" } }} history={history} location="/schedulingunit/1/workflow"  /> </MemoryRouter>);
        });
        expect(component.asFragment()).toMatchSnapshot();
        const cancelButton = component.getByText('Cancel');
        fireEvent.click(cancelButton);
        expect(component.asFragment()).toMatchSnapshot();

        component.unmount();
    });
});