

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act } from '@testing-library/react';
import { MockAxiosSecurity   } from './axiosmocks.'
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { MemoryRouter } from "react-router-dom/cjs/react-router-dom.min";
import qaschedulingunitblueprintextended_scheduable from "../__mocks__/qa_scheduling_unit_blueprint_extended_scheduble.json";
import qaschedulingunitblueprintextended from "../__mocks__/qa_scheduling_unit_blueprint_extended.json";
import chedulingunitdraft from "../__mocks__/qa_scheduling_unit_draft.json";
import project_role from "../__mocks__/project_role.json";
import scheduling_unit_set from "../__mocks__/qa_scheduling_unit_set.json";


import { remove404Errors } from "../utils/test.helper";
import Workflow from "../routes/Workflow";
import { LocalStorageMock  } from "./localstoragemocks";




function MakePrimaryMock(mock) {
    jest.useFakeTimers()
    jest.setSystemTime(new Date('2023-08-06T00:00:00Z'));
    mock.onGet("/api/project_role/").reply(200,  project_role, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE"});
    mock.onGet("/api/scheduling_unit_draft/14/").reply(200,  chedulingunitdraft, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, copy, create_cleanuptask_for_scheduling_unit_draft, create_observing_strategy_template, create_scheduling_unit_blueprint_and_tasks_and_subtasks_tree, scheduling_constraints_plot, specifications_doc, update_task_graph_from_specifications_doc"});
    mock.onGet("/api/scheduling_set/1/?expand=project").reply(200,  scheduling_unit_set, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, copy"});
    mock.onGet("/api/system_event/?limit=1").reply(200, {"count":0,"next":null,"previous":null,"results":[]});
    mock.onGet("/api/util/utc").reply(200, "2023-12-24T22:58:22.413683");

    return mock
}

describe('Workflow  ', () => {

    it('Workflow   renders correctly without security', async () => {
        let mock= new MockAdapter(axios);
        mock.onGet("/").reply(404);
        remove404Errors();
        let {  component } = await RenderWorkflow(false);
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('Workflow  correctly with data but no security ', async () => {
        remove404Errors();
        let {  component } = await RenderWorkflow(true);
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });

    it('Workflow correctly with data and security', async () => {
        let mock =  new MockAdapter(axios)
        mock.onGet("/api/scheduling_set/3/?expand=project").reply(200,  scheduling_unit_set, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, copy"});
        mock.onGet("/api/scheduling_unit_blueprint_extended/1/").reply(200,  qaschedulingunitblueprintextended, {"access-control-allow-methods":"GET, POST, PUT, PATCH, DELETE, cancel, copy_specifications_doc_back_into_draft, copy_specifications_doc_including_copies_for_failed_tasks_back_into_draft, copy_to_draft, create_cleanuptask_for_scheduling_unit_blueprint, create_copies_of_failed_tasks_in_draft, create_copies_of_failed_tasks_via_draft, create_lofar2_sibling, get_all_subtasks_log_urls, mark_as_obsolete, mark_dynamically_scheduled, mark_fixed_time_scheduled_at_scheduled_starttime, reset_schedule, schedule, scheduling_constraints_plot, specifications_doc, specifications_doc_including_copies_for_failed_tasks, specifications_doc_with_just_the_parameters, subtasks, system_events, unschedule, update_task_blueprints_and_subtasks_graph_from_draft"});
        
        let {  component  } = await RenderWorkflow(true,true);
        
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });

    it('Workflow  schedubale correctly with data and security step 1', async () => {

        
        let {  component  } = await RenderWorkflow(true,true,qaschedulingunitblueprintextended_scheduable);
        expect(component.asFragment()).toMatchSnapshot();
        component.unmount();
    });
    
});



async function RenderWorkflow(isWisecurity=false,iswithMock=false,mock =  new MockAdapter(axios)) {
    global.localStorage = new LocalStorageMock;
    if (!isWisecurity && iswithMock) {
          MakePrimaryMock(mock);
    }
    if (isWisecurity && iswithMock) {
        
        MockAxiosSecurity(MakePrimaryMock(mock));
       await PermissionStackUtil.getPermissions(true);
    }
    
    let component;
    let instance;
    await act(async () => {
        component = render(<MemoryRouter> <Workflow  match={{ params: { id: "1" } }} history={{}} location="/schedulingunit/1/workflow"  /> </MemoryRouter>);
    });
    return { instance, component };
}


