import { find, findIndex, remove } from 'lodash';

import UtilService from '../services/util.service'

/** Utility to handle table filter in localstorage */
const TableUtil = {
    /**
     * Function to store the filter values applied in a table against each column/header
     * @param {String} tableName 
     * @param {String} headerName 
     * @param {any} updatedValue 
     */
    saveFilter: (tableName, headerName, updatedValue) => {
        let tableStore =  UtilService.localStore({ type: 'get', key: tableName });
        if(tableStore && tableStore.length>0) {
            let columnFilter = {id: headerName, value: updatedValue};
            let index = findIndex(tableStore, ['id', headerName]);
            if (index >= 0) {
                tableStore.splice(index, 1, columnFilter);
            }   else {
                tableStore.push(columnFilter);
            }
        }   else {
            tableStore = [{id: headerName, value: updatedValue}]
        }
        UtilService.localStore({ type: 'set', key: tableName, value: tableStore });
    },
    /**
     * Get the stored filtered value of the header in the table
     * @param {String} tableName 
     * @param {String} headerName 
     * @returns any
     */
    getFilter: (tableName, headerName) => {
        let tableStore =  UtilService.localStore({ type: 'get', key: tableName }); 
        let columnFilter = tableStore?find(tableStore, ['id', headerName]):null;
        return columnFilter?columnFilter.value:"";
    },
    /**
     * Remove the filter value of the column in the table
     * @param {String} tableName 
     * @param {String} columnName 
     */
    clearColumnFilter: (tableName, columnName) => {
        let tableFilters = UtilService.localStore({type: 'get', key: tableName});
        if (tableFilters) {
            remove(tableFilters, filter => filter.id === columnName);
            UtilService.localStore({type: 'set', key: tableName, value: tableFilters});
        }
    },
    /**
     * Remove all the values stored for the table
     * @param {String} tableName 
     */
    clearTableFilter: (tableName) => {
        UtilService.localStore({type: 'remove', key: tableName});
    }
}

export default TableUtil;  