import {act} from "react-dom/test-utils";
import {fireEvent, render, screen,waitFor} from "@testing-library/react";
import {BrowserRouter as Router} from "react-router-dom";
import { load} from 'cheerio';

import beautify from 'js-beautify';
const TIMEOUT = 5

export async function clickItem(item, timeout = TIMEOUT) {
    await act(async () => {
        fireEvent.click(item);
        await new Promise((r) => setTimeout(r, timeout));
    } );
}

export async function setMultiSelectValue(pageContent, multiSelectTestId, multiSelectValue, timeout = TIMEOUT) {
    await waitFor(() => expect(pageContent.queryByTestId(multiSelectTestId)).toBeInTheDocument()); // give rendering the time to have it.
    await clickItem(pageContent.queryByTestId(multiSelectTestId));

    let items = []
    try {
        items = screen.getAllByText(multiSelectValue);
    } catch (error) {
        throw new ReferenceError("No items found to select for multi selection element with test id '"
            + multiSelectTestId + "' and value: " + multiSelectValue)
    }

    for (let item of items) {
        await clickItem(item, timeout);
    }
}

export async function renderPage(pageComponent) {
    let content;
    await act(async () => {
        content = render(<Router>{pageComponent}</Router>);
        await new Promise((r) => setTimeout(r, 5));
    });
    return content;
}

/**
 * Since the current version is React 17 but it has been patched to React 18, the behaviour is still the older version.
 * Therefore these specific warnings should be ignored until the app has been fully upgraded
 */
export function removeReact18ConsoleErrors() {
    jest.spyOn(global.console, 'error').mockImplementationOnce((message) => {
        if (!message.includes(' ReactDOM.render is no longer supported in React 18') ) {
            global.console.info(message);
        }
    });
}

// remove 404 errors who are un purpose due to testing 404 :)
export function remove404Errors() {
    jest.spyOn(global.console, 'error').mockImplementation((error,message) => {
        MuteConsoleLogging(error, message);
    });

    jest.spyOn(global.console, 'info').mockImplementation((error,message) => {
        MuteConsoleLogging(error, message);
    });
}

function MuteConsoleLogging(error, message) {
    if (!(
        error?.message == "Request failed with status code 400" ||
        error == "Request failed with status code 404" ||
        error?.message == "Request failed with status code 404" ||
        message?.message == "Request failed with status code 404" ||

        error?.message == "Network Error" ||
        error?.message == "Request failed with status code 500" ||
        message?.message == "Request failed with status code 500"
    )) {
        global.console.debug(message);
    }
}

// Remove all warnings. do use with care. Used now to Stop Ag-Grid bloating the logs.
export function removeAllWarnings() {
    jest.spyOn(global.console, 'warn').mockImplementation(() => {
    });
}


export function giveConsistentNodes(html) {
    const htmlwithcheerio =load(html, { xmlMode: false });
    htmlwithcheerio('[data-rbd-drag-handle-context-id]').attr('rbd-drag-handle-context-id', 'X');
    htmlwithcheerio('[data-rbd-drag-handle-context-id]').attr('rbd-drag-handle-context-id', 'X');
                                                                
    htmlwithcheerio('[datetime]').attr('datetime', 'X');
    htmlwithcheerio('.notfixed-column-td-StartTime div').text('XXXX-XX-XX XXXX:XX:XX');
    htmlwithcheerio('.notfixed-column-td-EndTime div').text('XXXX-XX-XX XXXX:XX:XX');
    htmlwithcheerio('.rct-dateHeader span').text('XX:XX');
    htmlwithcheerio('.rct-dateHeader').attr('data-testid',"X");
    htmlwithcheerio('.rct-dateHeader').text('XX:XX');
    htmlwithcheerio('.rct-vertical-lines div').attr('style', 'X');
    htmlwithcheerio('.form-check-label').attr('for', 'X');
    htmlwithcheerio('.form-check-input').attr('id', 'X');
    htmlwithcheerio('.form-check-input').attr('id', 'X');

    
    

    
    

    let result = htmlwithcheerio.html()
    const formattedHtml = beautify.html(result, {
        indent_size: 2,
        end_with_newline: true,
        preserve_newlines: true,
      });
      return  formattedHtml
  }
